-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

require "app.assets.asset_game_flag"

game.CAssetGameBuffSpine = {
    shield = "spine/shield/skeleton",
    magnet = "spine/magnet/skeleton",
}

-- 受击表现
local hit_skin = {
    add = function(node)
        local action = node:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.TintTo:create(0.1, cc.c3b(255,0,0)), cc.TintTo:create(0.1, cc.c3b(255,255,255)))))
        return action
    end,

    remove = function(node, action)
        if action ~= nil then
            node:stopAction(action)
            node:setColor(cc.c3b(255,255,255))
        end
    end,
}

-- 无敌表现
local invincible_skin = {
    add = function(node)
        node:setOpacity(255)
        local action = node:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.FadeOut:create(0.1), cc.FadeIn:create(0.1))))
        return action
    end,

    remove = function(node, action)
        if action ~= nil then
            node:stopAction(action)
            node:setOpacity(255)
        end
    end,
}

-- 护盾表现
local shield_skin = {
    add = function(node)
        local animation_name = game.CAssetGameBuffSpine.shield
        local spine = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(animation_name..".json", 
        animation_name..".atlas", true)
        if spine then
            spine:setAnimation(0,"division",true)
            node:addChild(spine)
        end
        return spine
    end,

    remove = function(node, shield_spine)
        if shield_spine then
            shield_spine:removeFromParent()
            shield_spine = nil
        end
    end,
}

-- 吸铁石表现
local magnet_skin = {
    add = function(node)
        local animation_name = game.CAssetGameBuffSpine.magnet
        local spine = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(animation_name..".json", 
        animation_name..".atlas", true)
        if spine then
            spine:setAnimation(0,"hit",false)
            spine:setCompleteListener(function()
                spine:setAnimation(0,"continued",true)
            end)
            node:addChild(spine)
        end

        return spine
    end,
    remove = function(node,magnet_spine)
        if magnet_spine then
            magnet_spine:removeFromParent()
        end
    end,
}

-- BUFF清单
game.CAssetGameBuff = {

    -- 受击无敌效果
    hit_invincible = {
        flag = { game.CAssetGameFlag.invincible },
        consume = { dt = 1 },
        skin = hit_skin,
        add_sound = "",
        remove_sound = "",
        priority = 3,
    },

    -- 冲刺结束后的无敌效果
    super_dash_finish_invincible = {
        flag = { game.CAssetGameFlag.invincible },
        consume = { dt = 2 },
        skin = invincible_skin,
        add_sound = "",
        remove_sound = "",
        priority = 3,
    },

    -- 道具无敌效果
    item_invincible = {
        flag = { game.CAssetGameFlag.invincible },
        consume = { dt = 6 },
        skin = invincible_skin,
        add_sound = "",
        remove_sound = "",
        priority = 3,
        icon_file = "images/map/real_object/Invincible.png",
    },

    -- 道具护盾效果
    item_shield = {
        flag = { game.CAssetGameFlag.invincible, },
        consume = {  hit_count = 1 },
        skin = shield_skin,
        add_sound = "",
        remove_sound = "",
        priority = 1,
        icon_file = "images/map/real_object/Shield.png",
    },

    -- 飞行状态
    super_dash = {
        flag = { 
            game.CAssetGameFlag.cannot_move_accelerate,
            game.CAssetGameFlag.cannot_move_decelerate,
            game.CAssetGameFlag.cannot_move_dash,
            game.CAssetGameFlag.cannot_jump,
            game.CAssetGameFlag.cannot_use_weapon,
            game.CAssetGameFlag.invincible,
            game.CAssetGameFlag.magnet,
            game.CAssetGameFlag.never_die,
        },
        add_sound = "",
        remove_sound = "",
        priority = 1,
        desc = "die_invincible2",
        icon_file = "images/map/real_object/sprint.png",
    },
    
    -- 道具吸铁石
    item_magnet = {
        flag = { game.CAssetGameFlag.magnet },
        consume = { dt = 6 },
        skin = magnet_skin,
        add_sound = "",
        remove_sound = "",
        icon_file = "images/map/real_object/magnet.png",
    },

    -- 距离分数加成
    more_distance_score = {
        add_effect = function(target_object) 
            local distance_score_times = 0.3
            target_object.map.distance_score_times = target_object.map.distance_score_times + distance_score_times
            return distance_score_times
        end,
        remove_effect = function(target_object, value) target_object.map.distance_score_times = target_object.map.distance_score_times - value end,
        desc = "more_distance_score",
    },

    -- 伤害加成
    more_damage = {
        add_effect = function(target_object) 
            local addition_damage = math.floor(target_object.addition_damage * 0.05)
            target_object.addition_damage = target_object.addition_damage + addition_damage
            return addition_damage
        end,
        remove_effect = function(target_object, value) target_object.addition_damage = target_object.addition_damage - value end,
        desc = "more_damage",
    },

    -- 分数加成
    more_score = {
        consume = { dt = 6 },
        add_effect = function(target_object) target_object.map.score_times = target_object.map.score_times + 1 end,
        remove_effect = function(target_object, value) target_object.map.score_times = target_object.map.score_times - 1 end,
        desc = "more_score",
        skin = magnet_skin,
        icon_file = "images/map/real_object/ScorePromotion.png",
    },
}

for key,value in pairs(game.CAssetGameBuff) do
    value.id = key
end

return game.CAssetGameBuff

-- endregion
