--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

game.CPlayerModuleItemBox = class("game.CPlayerModuleItemBox")

local CPlayerModuleItemBox = game.CPlayerModuleItemBox


-- 有 n 个槽位
-- 奖励获得宝箱
-- 解锁宝箱需要等待时间到才能打开
-- 每次只能解锁一次宝箱
-- 可以使用钻石立马打开，钻石通过剩余时间计算

function CPlayerModuleItemBox:ctor()

    -- 宝箱槽;
    player.db.item_box.slots = player.db.item_box.slots or {}
    player.db.item_box.slot_pos_unlocking = player.db.item_box.slot_pos_unlocking or 0

    -- 宝箱数量;
    self.item_box_count = 0
    for _, _ in pairs(player.db.item_box.slots) do
        self.item_box_count = self.item_box_count + 1
    end

    self.full_tip = self.item_box_count>=assets.const.item_box_max
end

-- 添加宝箱, 槽位满则无法添加;
-- 不能添加则返回 false;
function CPlayerModuleItemBox:AddBox(item_box_id)

    if self.item_box_count>=assets.const.item_box_max then
        return false
    end

    local entry = assets.item_boxes:GetEntry(item_box_id)
    if not entry then
        return false
    end

    local slot_pos = 0
    for i=1, assets.const.item_box_max do

        if not player.db.item_box.slots[i] then
            player.db.item_box.slots[i] = {}
            player.db.item_box.slots[i].id = item_box_id
            player.db.item_box.slots[i].unlock_time = 0
            player.db.item_box.slots[i].unlocked = false
            slot_pos = i
            break
        end
    end

    self.item_box_count = self.item_box_count + 1
    self.full_tip = self.item_box_count>=assets.const.item_box_max

    sMessageDispatcher:Dispatch("item_box_added", slot_pos)
    return true
end

-- 解锁宝箱;
-- 解锁失败返回 false;
function CPlayerModuleItemBox:UnlockBox(slot_pos)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        return false
    end

    if player.db.item_box.slot_pos_unlocking > 0 then
        return false
    end

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        return false
    end

    item_box.unlock_time = os.time()
    self:_SetUnlockSlotPos(slot_pos)
    return true
end

-- 开宝箱;
-- 开宝箱失败返回 false;
function CPlayerModuleItemBox:OpenBox(slot_pos, force)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        return true
    end

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        return false
    end

    local last_unlock_seconds = self:GetLastUnlockSeconds(slot_pos)
    if not force and last_unlock_seconds ~= 0 then
        return false
    end

    return self:_OpenBox(slot_pos)
end

function CPlayerModuleItemBox:_OpenBox(slot_pos)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        return true
    end

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        return false
    end

    -- 获取奖励;
    player.player_modules.reward:TakeRewardString(entry.reward)
    
    -- 删除宝箱;
    player.db.item_box.slots[slot_pos] = nil
    if player.db.item_box.slot_pos_unlocking == slot_pos then
        self:_SetUnlockSlotPos(0)
    end

    self.item_box_count = self.item_box_count - 1

    sMessageDispatcher:Dispatch("item_box_opened", slot_pos)
    return true
end

-- 钻石开宝箱;
function CPlayerModuleItemBox:OpenBoxWithPoint(slot_pos, callback)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        if callback then callback(false) end
        return
    end

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        if callback then callback(false) end
        return
    end

    -- 计算剩余时间, 扣除点券;
    local point = self:GetUnlockPoint(slot_pos)
    sPlayer.player_modules.point:SubPoint(point, function(is_successful)
        if is_successful then
            self:_OpenBox(slot_pos)
        end
        if callback then callback(is_successful) end
    end)
end

-- 剩余解锁时间;
function CPlayerModuleItemBox:GetLastUnlockSeconds(slot_pos)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box or item_box.unlock_time == 0 then
        return -1
    end  

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        return -1
    end
    local unlock_seconds = entry.unlock_seconds
    local delta = item_box.unlock_time + unlock_seconds - os.time()
    if delta <= 0 then
        if item_box.unlocked == false then
            self:_SetUnlockSlotPos(0)
            item_box.unlocked = true
        end
        return 0
    end
    return delta
end

-- 剩余解锁时间;
function CPlayerModuleItemBox:GetUnlockPoint(slot_pos)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        return 0
    end  

    local entry = assets.item_boxes:GetEntry(item_box.id)
    if not entry then
        return 0
    end
    
    local last_unlock_seconds = self:GetLastUnlockSeconds(slot_pos)
    if last_unlock_seconds == -1 then
        last_unlock_seconds = entry.unlock_seconds
    end
    local second_per_point = entry.seconds_per_point
    local point = math.ceil(last_unlock_seconds / second_per_point)
    return point
end

-- 获取解锁中的槽位;
function CPlayerModuleItemBox:GetUnlockSlotPos()

    local last_unlock_seconds = self:GetLastUnlockSeconds(player.db.item_box.slot_pos_unlocking)
    if last_unlock_seconds>0 then
        return player.db.item_box.slot_pos_unlocking
    end
    return 0
end

function CPlayerModuleItemBox:_SetUnlockSlotPos(slot_pos)

    if player.db.item_box.slot_pos_unlocking ~= slot_pos then
        player.db.item_box.slot_pos_unlocking = slot_pos
        sMessageDispatcher:Dispatch("item_box_unlocked")
    end
end

-- 获取宝箱;
function CPlayerModuleItemBox:GetBox(slot_pos)

    local item_box = player.db.item_box.slots[slot_pos]
    if not item_box then
        return nil
    end  
    return item_box
end

function CPlayerModuleItemBox:IsFull()

    return self.item_box_count>=assets.const.item_box_max
end

-- 获取满时解锁的宝箱;
function CPlayerModuleItemBox:GetFullOpenSlotPos()

    if self.item_box_count == 0 then
        return 0
    end

    -- 优先级; 已解锁的第一位置; 解锁中; 未解锁的第一位置;
    for i=1, assets.const.item_box_max do
        local delta = self:GetLastUnlockSeconds(i)
        if delta == 0 then
            return i
        end
    end

    if player.db.item_box.slot_pos_unlocking ~=0 then
        return player.db.item_box.slot_pos_unlocking
    end

    return 1
end

return CPlayerModuleItemBox

--endregion
