-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成
game.RewardType =
{
    kTypeItem = "item",
    kTypeActionPoint = "action_point",
    kTypePoint = "point",
    kTypeGamebill = "gamebill",
    kTypeExp = "exp",
    kTypeItemBox = "item_box",
    kTypeMount = "mount",
    kTypeSkin = "skin",
}

local function IsRewardIDType(type_name)

    return type_name == game.RewardType.kTypeItem or
        type_name == game.RewardType.kTypeItemBox or
        type_name == game.RewardType.kTypeMount or
        type_name == game.RewardType.kTypeSkin
end


-- CRewardListList
game.CRewardList = class("game.CRewardList")
local CRewardList = game.CRewardList
game.sRewardHelper = CRewardList

function CRewardList:ctor()
    self.list = { }
end

-- Reward: {reward_type="", id=11, count=1}
-- Add(string)
-- Add(reward_list)
-- Add(reward_type, reward_id, reward_count)
-- Add(reward_type, reward_count)
function CRewardList:Add(...)

    local args = { ...}
    if #args == 1 then
        if type(args[1]) == "string" then

            local reward_list = CRewardList:ParseFromString(args[1])
            self:Add(reward_list)
        elseif type(args[1]) == "table" then

            local reward_list = args[1].list
            for k, v in pairs(reward_list) do
                self:_AddReward(v)
            end
        end
    else
        self:_AddReward(unpack(args))
    end
end

function CRewardList:GetList()

    return self.list
end

function CRewardList:ParseFromString(reward_string)

    local reward_list = CRewardList:new()
    local token_list = dxm.CToken:Parse(reward_string, "/")
    local pos = 1
    while pos <= #token_list do
        local reward_elem = { }
        reward_elem.reward_type = token_list[pos]
        pos = pos + 1
        if IsRewardIDType(reward_elem.reward_type) then
            reward_elem.id = tonumber(token_list[pos])
            pos = pos + 1
            reward_elem.count = tonumber(token_list[pos])
            pos = pos + 1
        else
            reward_elem.count = tonumber(token_list[pos])
            pos = pos + 1
        end
        reward_list:_AddReward(reward_elem)
    end
    return reward_list
end

function CRewardList:SerializeToString()

    local reward_string
    for key, value in pairs(self.list) do
        if not reward_string then
            reward_string = ""
        else
            reward_string = reward_string .. "/"
        end
        
        if IsRewardIDType(value.reward_type) then
            reward_string = reward_string .. value.reward_type .. "/" .. value.id .. "/" .. value.count
        else
            reward_string = reward_string .. value.reward_type .. "/" .. value.count
        end
    end
    return reward_string
end

-- _AddReward 重载
-- _AddReward(reward)
-- _AddReward(reward_type, reward_id, reward_count)
-- _AddReward(reward_type, reward_count)
function CRewardList:_AddReward(...)

    local args = {...}
    -- dump(args, "args")
    if #args == 1 and type(args[1])=="table" then
        if IsRewardIDType(args[1].reward_type) then
            self:_AddWithID(args[1].reward_type, args[1].id, args[1].count)
        else
            self:_AddWithoutID(args[1].reward_type, args[1].count)
        end        
    elseif #args == 2 then
        if IsRewardIDType(args[1]) then
            self:_AddWithID(args[1], args[2], 1)
        else
            self:_AddWithoutID(args[1], args[2])
        end
    elseif #args == 3 then
        if IsRewardIDType(args[1]) then
            self:_AddWithID(args[1], args[2], args[3])
        end
    end
    -- dump(self.list, "self.list")
end

function CRewardList:_AddWithID(reward_type, reward_id, reward_count)

    if not reward_type or not reward_id then return end
    
    local reward = self:_FindReward(reward_type, reward_id)
    -- print("_AddWithID", reward_type, reward_id, reward_count, reward)
    if reward then
        reward.count = reward.count + reward_count
    else
        reward = { }
        reward.reward_type = reward_type
        reward.id = reward_id
        reward.count = reward_count or 1
        self.list[#self.list + 1] = reward
    end
end

function CRewardList:_AddWithoutID(reward_type, reward_count)

    if not reward_type then return end
    
    local reward = self:_FindReward(reward_type)
    -- print("_AddWithoutID", reward_type, reward_count, reward)
    if reward then
        reward.count = reward.count + reward_count
    else
        reward = { }
        reward.reward_type = reward_type
        reward.count = reward_count or 1
        self.list[#self.list + 1] = reward
    end
end

function CRewardList:_FindReward(reward_type, reward_id)

    for key, value in pairs(self.list) do      
        if value.reward_type == reward_type and (not reward_id or value.id == reward_id) then
            return value
        end
    end
    return nil
end

function CRewardList:Double()

    for key, value in pairs(self.list) do    
        value.count = value.count * 2
    end
end

------
-- 兼容老代码的类

game.CReward = class("game.CReward")
local CReward = game.CReward
function CReward:ctor()
    self.reward_type = nil
    self.id = nil
    self.count = nil
end

function CReward:Create(reward_string)

    local reward = CReward.new()
    local token_list = dxm.CToken:Parse(reward_string,"/")
    if IsRewardIDType(token_list[1]) then
        self.reward_type = token_list[1]
        self.id = tonumber(token_list[2])
        self.count = tonumber(token_list[3])
    else
        self.reward_type = token_list[1]
        self.count = tonumber(token_list[2])
    end
    
    return reward
end



return CRewardList

-- endregion
