--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

require "app.views.game.game_map_command"
require "app.views.game.game_map_recording"

game.CGameMap = class("game.CGameUI", cc.Node)
local CGameMap = game.CGameMap

CGameMap.logic_interval = 1/50
CGameMap.visible_size = {width=1160,height=870}
CGameMap.camera_rect = {x=CGameMap.visible_size.width*0.25,y=CGameMap.visible_size.height*0.55,width=1,height=1}

-- 计算距离
local function GetSectionConfigListDistance(section_config_list)

    local distance = 0
    for i=1,#section_config_list do

        local section_config = section_config_list[i]
        if section_config then
            
            local tmx_config = require("app.views.game.tmx_cache"):Add(section_config.tmx_file)
            if tmx_config then

                if tmx_config.start_pos then
                    distance = -tmx_config.start_pos.x
                end 
                if tmx_config.end_pos then
                    distance = distance + tmx_config.end_pos.x
                else
                    distance = distance + tmx_config.width * game.CAssetGame.tile_width
                end
            end
        end
    end
    return distance
end


function CGameMap:ctor()

    -- 当前地图的缩放比例
    self.scale = 1

    -- 地图的运行时间
    self.dt = 0
    self.total_dt = 0
    self.count_down = nil

    -- 关卡

    -- 地图区域的推进情况
    self.current_section_group_id = 0
    self.current_section_group_config = nil
    self.current_section_list = nil

    -- 当前是否在录像
    self.is_recording = false

    -- 回调函数（做成事件更好）
    self.section_changed_callback = nil
    self.section_group_changed_callback = nil
    self.object_enter_callback = nil
    self.gameover_callback = nil
    self.item_collect_event = dxm.CEvent.new()
    self.score_changed_event = dxm.CEvent.new()
    
    -- 地图事件
    self.event = nil

    -- 是否开始
    self.is_prepare = false
    self.is_started = false

    -- 是否暂停
    self.is_pause = false
    self.pause_ref = 0

    -- 是否结束
    self.is_gameover = false
    self.is_win = false

    -- 游戏帧
    self.frame_count = 0

    -- 显示元素
    self.tmx = nil
    self.player = nil
    self.background = {}
    self.particle = nil

    -- 当前关卡分数
    self.score = 0
    self.gold_score_times = 1
    self.pickup_score_times = 1
    self.time_score_times = 1
    self.distance_score_times = 1
    self.score_times = 1
    self.more_time = 0
    self.kill_boss_score_times = 1

    -- 需要的信息纪录
    self.record = {}
    self.record.item_collect = {}

    -- 
    self.last_player_x = nil

    -- 米数统计
    self.pixel = 0
    self.meter = 0

    -- 总米数
    self.total_meter = nil

    -- 累计怒气
    self.fury = 0
    self.fury_condition = game.CAssetGame.fury_condition

    -- 速度
    self.time_scale = 1
end

function CGameMap:Init(stage_type, stage_id, child_stage_id, player_id, mount_id)

    sProfileSystem:Reset()

    -- 判断是否存在录像，不存在则创建一个
    self.recording = game.CGameMapRecording:Create(math.random(1, 65535))
    if self.recording == nil then
        return false
    end
    self.is_recording = true
    self.recording.stage_type = stage_type
    self.recording.stage_id = stage_id
    self.recording.child_stage_id = child_stage_id
    self.recording.player_id = player_id
    self.recording.mount_id = mount_id

    self.child_stage_entry = game.CPlayerModuleStage:GetChildStageEntry(stage_type, stage_id, child_stage_id)
    self.game_rank = require("app.views.game.game_dummy_rank").new()

    -- 初始化地图
    return self:_Init()
end

function CGameMap:InitWithRecording(recording)

    if recording == nil then
        return false
    end

    -- 录像赋值
    self.recording = recording
    self.is_recording = false

    -- 初始化地图
    return self:_Init()
end

function CGameMap:_Init()

    -- 配置获取
    self.config = sPlayer.player_modules.stage:GetChildStageEntry(self.recording.stage_type, self.recording.stage_id, self.recording.child_stage_id)
    if self.config == nil then
        return false
    end

    -- 普通和竞速的BOSS关卡需要计算总路程
    if not self.child_stage_entry.is_boss and (
    self.recording.stage_type == "normal" or 
    self.recording.stage_type == "compete") then

        local total_distance = 0
        for i=1,#self.config.section_group_list do

            local section_group_config = self.config.section_group_list[i]
            if section_group_config then

                -- 动态区域和循环静态区域无法支持路程计算
                if not section_group_config.static_section_list or section_group_config.onUpdate then
                    total_distance = 0
                    break
                end

                -- 将静态区域的路程计算出来
                local distance = GetSectionConfigListDistance(section_group_config.static_section_list)
                total_distance = total_distance + distance
            end
        end
        if total_distance > 0 then
            self.total_meter = total_distance / game.CAssetGame.m_to_pixel
        end
    end

    -- 监听进入离开事件
    self:registerScriptHandler( function(event)
        if event == "enter" then
            self:onEnter()
        elseif event == "exit" then
            self:onExit()
        elseif event == "cleanup" then
            self:Dispose()
        end
    end )
    return true
end

function CGameMap:Create(...)

    local map = CGameMap.new()
    if map == nil or map:Init(...) == false then
        return nil
    end
    return map
end

function CGameMap:CreateWithRecording(...)
    
    local map = CGameMap.new()
    if map == nil or map:InitWithRecording(...) == false then
        return nil
    end
    return map
end

function CGameMap:onEnter()

    -- 地图缩放比例（临时配置）
    --self.scale = 0.9

    -- 回调函数
    self.section_changed_callback = section_changed_callback

    -- 随机发生器
    self.random = dios.util.CRandom:new()
    if self.random then

        local random_seed = 0
        if self.recording then
            random_seed = self.recording.data.random_seed
        end
        self.random:Init(random_seed)
    end

    -- 更新统一管理
    self.updater = require("app.views.game.game_updater"):Create()
    if self.updater then
        self.updater:InstallOnNode(self)
    end

    -- 设置缩放
    self:setScale(self.scale)

    -- 可视大小
    self.visible_size = { }
    self.visible_size.width = CGameMap.visible_size.width / self.scale
    self.visible_size.height = CGameMap.visible_size.height / self.scale

    -- 获取第一个区域组
    if not self:NextSectionGroup() then
        return
    end

    -- 创建主游戏TMX层，并加载第一张区域图
    self.tmx = require("app.views.game.game_tmx"):Create(self, self.current_section_list.current_section_config.tmx_file)
    if self.tmx then
        self:addChild(self.tmx, 0)

        -- 缩放一下
        --self.tmx:setScale(self.scale)

        -- 尝试获取一次必要信息
        self.start_pos = self.tmx.tmx_config.start_pos
        
        -- 初始化角色
        self.player = require("app.views.game.game_player"):Create(self, game.CAssetGamePlayer[self.recording.player_id])
        if self.player then
            self.tmx:AddObject(self.player, math.floor(self.start_pos.x), math.floor(self.start_pos.y), 1)

            -- 骑上坐骑
            if self.recording.mount_id then
                self.player:Ride(self.recording.mount_id)
            end

            -- 角色默认隐藏
            self.player:setVisible(false)
        end
    end

    -- 初始化摄像机矩形（角色需要始终处于该矩形内）
    self.camera_rect = { }
    self.camera_rect.x = CGameMap.camera_rect.x / self.scale
    self.camera_rect.y = CGameMap.camera_rect.y / self.scale
    self.camera_rect.width = CGameMap.camera_rect.width / self.scale
    self.camera_rect.height = CGameMap.camera_rect.height / self.scale

    -- 监听总的更新体系
    self.entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc( function(dt) self:Update(dt) end, 0, false)

    -- 排行设置
    if self.game_rank then
        self.game_rank:SetMaxMeter(self.total_meter)
    end

    -- 播放音乐
    dxm.audio:playMusic(assets.sound.config.music.main, true)

    -- 初始更新一次
    self:_Update(0)

    -- 判断是否开始
    self.is_prepare = true
    if self.is_started then
        self:_Start()
    end
end

function CGameMap:onExit()

    if self.random then
        self.random:delete()
        self.random = nil
    end

    if self.entry then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.entry)
        self.entry = nil
    end

    if self.updater then
        self.updater:Dispose()
        self.updater = nil
    end
end

function CGameMap:Start(item_id_list, attributes)

    if self.is_started then
        return
    end
    self.is_started = true

    -- 记录赋值
    self.recording.item_id_list = item_id_list
    self.recording.attributes = attributes

    -- 判断游戏是否准备完毕
    if self.is_prepare then
        self:_Start()
    end
end

function CGameMap:_Start()

    -- 正式开始游戏
    if self.player then
        self.player:setVisible(true)

        -- 属性开始解析
        if self.recording.attributes then

            for key,value in pairs(self.recording.attributes) do

                if key == enum.AttributeType.kTypeHP then
                    if self.player then 
                        local component_hp = self.player.component_list.hp
                        if component_hp then
                            component_hp:MoreHP(value)
                        end
                    end
                elseif key == enum.AttributeType.kTypeDefense then
                    if self.player then 
                        self.player.reduction_damage = self.player.reduction_damage + (value / 100)
                    end
                elseif key == enum.AttributeType.kTypeDoubleJumps then
                elseif key == enum.AttributeType.kTypeAttackPower then
                    if self.player then 
                        self.player.addition_damage = self.player.addition_damage + value
                    end
                elseif key == enum.AttributeType.kTypeMonsterScorePercentAdd then
                    self.kill_boss_score_times = self.kill_boss_score_times + (value / 100)
                elseif key == enum.AttributeType.kTypeTimeScorePercentAdd then
                    self.time_score_times = self.time_score_times + (value / 100)
                elseif key == enum.AttributeType.kTypeDistanceScorePercentAdd then
                    self.distance_score_times = self.distance_score_times + (value / 100)
                elseif key == enum.AttributeType.kTypePickScorePercentAdd then
                    self.pickup_score_times = self.pickup_score_times + (value / 100)
                elseif key == enum.AttributeType.kTypeTimeFrozen then
                elseif key == enum.AttributeType.kTypeClearAllBlocks then
                elseif key == enum.AttributeType.kTypeStrengthMagnet then
                    if self.player then 
                        self.player.magnet_dt_times = self.player.magnet_dt_times + (value / 100)
                    end
                elseif key == enum.AttributeType.kTypeStrengthSprint then
                    if self.player then 
                        self.player.super_dash_distance_addition = self.player.super_dash_distance_addition + value
                    end
                end
            end
        end

        -- 把道具全部都使用一遍
        for _,item_id in pairs(self.recording.item_id_list) do
                
            -- 获取道具配置
            local item_entry = assets.items:GetEntry(item_id)
            if item_entry then

                if item_entry.item_type == assets.items.ItemType.kTypeHP then
                    if not self.has_item_hp then
                        self.has_item_hp = true

                        local component_hp = self.player.component_list.hp
                        if component_hp then
                            component_hp:MoreHP(component_hp.max_hp * tonumber(item_entry.value))
                        end
                    end
                elseif item_entry.item_type == assets.items.ItemType.kTypeBuff then
                    self.player:AddBuff(game.CAssetGameBuff[item_entry.value])
                elseif item_entry.item_type == assets.items.ItemType.kTypeGoldScore then
                    self.gold_score_times = self.gold_score_times + tonumber(item_entry.value)
                elseif item_entry.item_type == assets.items.ItemType.kTypeScore then
                    self.score_times = self.score_times + tonumber(item_entry.value)
                elseif item_entry.item_type == assets.items.ItemType.kTypeTime then
                    self.more_time = tonumber(item_entry.value)
                end 
            end
        end

        self.player:Dash(true)
    end
end

function CGameMap:_ChangeBackground(background_id)

    -- 判断地图背景有没有改变
    local old_background_id = nil
    if self.current_section_group_config then
        old_background_id = self.current_section_group_config.background
    end

    -- 判断背景层是否存在改变
    if old_background_id ~= background_id then

        -- 过去的地图背景渐变离场
        for _,background in pairs(self.background) do
            background:runAction(cc.Sequence:create(cc.FadeOut:create(1), cc.CallFunc:create(function() 
                background:removeFromParent() 
                self.background[background] = nil
            end)))
        end

        -- 新创建地图背景
        if background_id then

            local background_config = game.CAssetGameBackground[background_id]
            if background_config then
                for _,background_element_config in pairs(background_config) do

                    local background = require("app.views.game.game_background"):Create(self, background_element_config)
                    if background then
                        self.background[background] = background
                        self:addChild(background, -background_element_config.z)
                        background:setOpacity(0)
                        background:runAction(cc.FadeIn:create(1))
                    end
                end
            end
        end
    end
end

function CGameMap:_ChangeWeather(weather_id)
   
    -- 判断天气有没有改变
    local old_weather_id = nil
    if self.current_section_group_config then
        old_weather_id = self.current_section_group_config.weather
    end
    if old_weather_id ~= weather_id then

        -- 过去的粒子效果渐变离场
        if self.particle then
            local particle = self.particle
            particle:runAction(cc.Sequence:create(cc.FadeOut:create(3), cc.CallFunc:create(function() particle:removeFromParent() end)))
        end
        self.particle = nil

        -- 新创建场景效果粒子
        if weather_id then

            local weather_config = game.CAssetGameWeather[weather_id]
            if weather_config then

                if weather_config.particle_path then
                    local particle = cc.ParticleSystemQuad:create(weather_config.particle_path)
                    if particle then

                        self.particle = cc.ParticleBatchNode:createWithTexture(particle:getTexture())
                        if self.particle then
                            particle:setPosition(cc.p(CGameMap.visible_size.width / 2, CGameMap.visible_size.height/ 2))
                            self.particle:addChild(particle)

                            -- TODO: 这个缩放未来是否可以取消掉？
                            self.particle:setScale(1.5)

                            self:addChild(self.particle, 1000)

                            self.particle:setOpacity(0)
                            self.particle:runAction(cc.FadeIn:create(3))
                        end
                    end
                end
            end
        end
    end 
end

function CGameMap:_ChangeEvent(event_id)

    -- 判断事件有没有改变
    local old_event_id = nil
    if self.current_section_group_config then
        old_event_id = self.current_section_group_config.event
    end
    if old_event_id ~= event_id then

        -- 终止事件
        if self.event then
            self.event:Stop()
            self.event = nil
        end

        -- 创建事件
        if event_id then

            local event_config = game.CAssetGameEvent[event_id]
            if event_config then

                -- 创建事件并执行
                local class_file = "app.views.game.game_event."..event_config.class_name
                self.event = require(class_file):Create(self)
                if self.event then
                    self.event:Run()
                end
            end
        end
    end
end

function CGameMap:NextSectionGroup()

    -- 判断是否到达终点
    if self.current_section_group_id >= #self.config.section_group_list then
        return false
    end

    -- 获取下一个区域组配置
    local next_section_group_id = self.current_section_group_id + 1
    local next_section_group_config = self.config.section_group_list[next_section_group_id]
    local next_section_list = nil
    if next_section_group_config then

        -- 临时修正
        next_section_group_config.g = next_section_group_config.g or game.CAssetGame.default_g
        next_section_group_config.move_speed = next_section_group_config.move_speed or game.CAssetGame.default_move_speed
        next_section_group_config.exit_move_speed = next_section_group_config.exit_move_speed  or game.CAssetGame.default_exit_move_speed

        -- 根据动态静态的列表标记来创建对应的区域列表
        if next_section_group_config.dynamic_section_list then
            next_section_list = require("app.views.game.game_section.game_section_dynamic_list"):Create(self, 
            next_section_group_config.dynamic_section_list.random_section_list, 
            next_section_group_config.dynamic_section_list.random_section_times)
        elseif next_section_group_config.static_section_list then
            next_section_list = require("app.views.game.game_section.game_section_static_list"):Create(self, 
            next_section_group_config.static_section_list)
        end
    end

    -- 变更背景
    if next_section_group_config then
        self:_ChangeBackground(next_section_group_config.background)
    end

    -- 变更天气（暂时关闭天气）
    --[[
    if next_section_group_config then
        self:_ChangeWeather(next_section_group_config.weather)
    end 
    ]]--

    -- 变更事件
    if next_section_group_config then
        self:_ChangeEvent(next_section_group_config.event)
    end

    -- 无限跳跃标记
    self.ignore_jump_times = next_section_group_config.ignore_jump_times

    -- 区域组离开
    if self.current_section_group_config and self.current_section_group_config.onExit then
        self.current_section_group_config.onExit(self)
    end

    -- 正式切换到下一个区域组
    self.current_section_group_id = next_section_group_id
    self.current_section_group_config = next_section_group_config
    self.current_section_list = next_section_list

    -- 区域组进入
    if self.current_section_group_config.onEnter then
        self.current_section_group_config.onEnter(self)
    end

    -- 赋值区域变更回调
    if self.current_section_list then

        self.current_section_list.section_changed_callback = function() 

            if self.section_changed_callback then
                self.section_changed_callback()
            end
        end
    end

    -- 执行回调函数
    if self.section_group_changed_callback then
        self.section_group_changed_callback()
    end

    return (self.current_section_list ~= nil)
end

function CGameMap:Dispose()
end

function CGameMap:Update(dt)

    -- 游戏未开始或者已经暂停
    if self.is_started == false or self.is_pause == true then
        return
    end

    -- sProfileSystem:StartSampling("CGameMap:Update")
    
    -- dt不能过大
    if dt > CGameMap.logic_interval * 10 then
        print("dt is too big", dt)
        dt = CGameMap.logic_interval * 10
    end

    -- 逻辑全部都存起来
    self.dt = self.dt + dt
    local _update_times = 0
    while self.dt >= CGameMap.logic_interval do  
       
        -- sProfileSystem:StartSampling("CGameTmx:Update")

        -- 运行一次逻辑帧
        self.dt = self.dt - CGameMap.logic_interval
        if self:_Update(CGameMap.logic_interval*self.time_scale) then
            return
        end

        -- sProfileSystem:StopSampling("CGameTmx:Update")

        -- 当累积量不足2帧时，将下一帧留在之后运行
        if self.dt < CGameMap.logic_interval*2 then
            break
        end
        _update_times = _update_times + 1
    end

    -- sProfileSystem:StopSampling("CGameMap:Update")
    -- sProfileSystem:AddSampling("CGameTmx:UpdateTimes", _update_times)
    -- sProfileSystem:AddSampling("CGameCircle:Update", dt*10000, dt/10)
end

function CGameMap:_Update(dt)

    -- 演示状态播放
    -- sProfileSystem:StartSampling("CGameCom:Update")
    if self.recording then

        local frame_command_list = self.recording.data.command_list[tostring(self.frame_count)]
        if frame_command_list then
            for _,command in ipairs(frame_command_list) do
                self:ExecuteCommand(command)
            end
        end
    end
    self.frame_count = self.frame_count + 1
    -- sProfileSystem:StopSampling("CGameCom:Update")
       
    -- 动画更新
    -- sProfileSystem:StartSampling("CGameUpdater:Update")
    if self.updater then
        self.updater:Update(dt)
    end
    -- sProfileSystem:StopSampling("CGameUpdater:Update")

    -- 场景跟随人物移动并更新
    if self.tmx then
        self.tmx:Update(dt)
    end

    -- 背景更新
    -- sProfileSystem:StartSampling("CGameBackGround:Update")
    for _,background in pairs(self.background) do
        background:Update(dt)
    end
    -- sProfileSystem:StopSampling("CGameBackGround:Update")

    -- 判断游戏是否结束
    if not self.is_gameover then

        -- 增加累计时间
        self.total_dt = self.total_dt + dt
    end

    -- 计算跑的得分
    self:CalcuRunScore()

    -- 计算游戏时间
    self:CalcuGameTime()

    -- 显示分数
    self:ShowScore(dt)

    return false
end

function CGameMap:ShowScore(dt)

    -- 角色身上播放效果
    self.show_item_score_dt = self.show_item_score_dt or 0
    self.show_item_score_dt = self.show_item_score_dt + dt
    if self.show_item_score_dt >= 0.1 then
        self.show_item_score_dt = self.show_item_score_dt - 0.1

        if self.player and self.show_item_score and self.show_item_score > 0 then
            local number = require("app.views.game.game_score"):Create(self.show_item_score)
            if number then
                local x = math.random(-30, 30)
                local y = math.random(-30, 30)
                number:setPosition(cc.p(x, y+self.player:GetBodyRect().height/2))
                self.player:addChild(number)
            end
            self.show_item_score = 0
        end
    end
end

function CGameMap:Pause()
    self.pause_ref = self.pause_ref + 1
    if self.pause_ref == 1 then
        self.is_pause = true
    end
end

function CGameMap:Resume()
    self.pause_ref = self.pause_ref - 1
    if self.pause_ref == 0 then
        self.is_pause = false
    end
end

function CGameMap:Finish()

    -- 角色站立
    if self.player then
        self.player.will_die = false
    end

    -- 游戏结束
    self:GameOver(true)
end

function CGameMap:Quit()

    -- 为record进行数据填充
    self.record.player_id = self.recording.player_id
    self.record.mount_id = self.recording.mount_id
    self.record.score = math.floor(self.score)
    self.record.meter = math.floor(self.meter)
    self.record.time = self.total_dt or 0
    self.record.pickup_list = clone(self.player.pickup_list)
    self.record.jump_times = self.player.total_jump_times
    self.record.jump_twice_times = self.player.total_jump_twice_times
    self.record.dash_times = self.player.total_dash_times
    self.record.rank = clone(self.game_rank:GetList())
end

function CGameMap:GameOver(is_win)
    if self.is_gameover then
        return
    end
    self.is_gameover = true
    self.is_win = is_win

    -- 排行调整
    if not self.is_win then
        self.game_rank:Break()
    end

    -- 退出游戏
    self:Quit()

    -- 游戏结束执行回调
    if self.gameover_callback then
        self.gameover_callback(self.is_win)
    end

    -- 音乐播放
    if self.is_win then
        dxm.audio:playMusic(assets.sound.config.sound.game_win, false)
    else
        dxm.audio:playMusic(assets.sound.config.sound.game_fail, false)
    end
end

function CGameMap:PushCommand(command)

    if self.recording and self.is_recording then
        self.recording:PushCommand(self.frame_count, command)
    end
end

function CGameMap:ExecuteCommand(command)
    if command == game.GameMapCommand.decelerate then
        if self.player ~= nil then
            self.player:MoveDecelerate()
        end
    elseif command == game.GameMapCommand.accelerate then
        if self.player ~= nil then
            self.player:MoveAccelerate()
        end
    elseif command == game.GameMapCommand.jump then
        if self.player ~= nil then
            self.player:Jump()
        end
    elseif command == game.GameMapCommand.cancel_decelerate then
        if self.player ~= nil then
            self.player:Idle()
        end
    elseif command == game.GameMapCommand.cancel_accelerate then
        if self.player ~= nil then
            self.player:Idle()
        end
    elseif command == game.GameMapCommand.attack then
        if self.player ~= nil then
            self.player:UseWeapon()
        end
    elseif command == game.GameMapCommand.dash then
        if self.player ~= nil then
            self.player:Dash()
        end
    end
end

function CGameMap:OnObjectEnter(object)

    -- 判断是否为BOSS
    if object.is_monster then
        self.boss = object
        sCtrlGuide:Listen("boss_enter")
    end

    if self.object_enter_callback then
        self.object_enter_callback(object)
    end
end

function CGameMap:OnObjectExit(object)

    -- 判断是否为BOSS
    if object.is_monster then

        -- 撤销BOSS单件
        self.boss = nil

        -- 触发事件
        sCtrlGuide:Listen("boss_exit")
    end
end

function CGameMap:GetNextTMXConfig()

    -- 判断当前区块组的任务是否完成，未完成则需要循环
    if self.current_section_group_config.onUpdate and self.current_section_group_config.onUpdate(self) == false then
        self.current_section_list:NextSection(true)
    else
        -- 切换区块
        self.current_section_list:NextSection(false)

        -- 判断区块是否存在
        if self.current_section_list.current_section_config == nil then

            -- 进入下一个组别
            self:NextSectionGroup()
        end

        -- 依旧没有有效区块
        if self.current_section_list.current_section_config == nil then
            return nil
        end
    end

    -- 新的地形
    return require("app.views.game.tmx_cache"):Add(self.current_section_list.current_section_config.tmx_file)
end

function CGameMap:AddScore(score)

    if score == 0 then
        return
    end

    -- 增加积分
    self.score = self.score + score

    -- 怒气系统
    if not self.is_gameover and 
    not self.child_stage_entry.is_boss and 
    self.player and self.player.current_move_status ~= game.CGamePlayer.MoveStatus.super_dash then
        self:AddFury(score)
    end

    -- 触发事件
    if self.score_changed_event then
        self.score_changed_event:Fire(score)
    end
end

function CGameMap:AddFury(fury)

    self.fury = self.fury + fury
    if self.fury >= self.fury_condition then
        
        if self.player then
            self.player:SuperDash(game.CAssetGame.fury_super_dash_distance)
        end
        self.fury = 0
        self.fury_condition = self.fury_condition * game.CAssetGame.next_fury_condition_times

        dxm.CMessageTipUI:Show(sStrings:GetString("score_super_dash"))
    end
end

function CGameMap:AddItem(item_id)

    if not self.player or self.is_gameover then
        return false
    end

    local item_config = game.CAssetGameObject[item_id]
    if item_config.score then

        local score_times = self.score_times
        if item_config.id == "gold" then
            score_times = score_times + (self.gold_score_times - 1)
        end
        if self.pickup_score_times > 0 then
            score_times = score_times + (self.pickup_score_times - 1)
        end

        local score = math.floor(item_config.score*score_times)

        -- 添加逻辑分数
        self:AddScore(score)

        -- 排行系统监控分数
        if self.game_rank then
            self.game_rank:AddItemScore(score)
        end

        -- 表现数据
        self.show_item_score = self.show_item_score or 0
        self.show_item_score = self.show_item_score + score
    end

    self.record.item_collect = self.record.item_collect or {}
    self.record.item_collect[item_id] = self.record.item_collect[item_id] or 0
    self.record.item_collect[item_id] = self.record.item_collect[item_id] + 1

    if self.item_collect_event then
        self.item_collect_event:Fire(item_id)
    end
end

function CGameMap:CalcuRunScore()

    if not self.last_player_x then
        self.last_player_x = self.player:getPositionX()
    end
    if not self.player or self.is_gameover then
        return false
    end

    local old_meter = self.meter

    -- 更新跑动距离
    self.pixel = self.pixel + (self.player:getPositionX() - self.last_player_x)
    self.last_player_x = self.player:getPositionX()
    self.meter = self.pixel / game.CAssetGame.m_to_pixel

    -- 计算分数
    if self.meter > old_meter then

        local add_meter = self.meter - old_meter
        local add_score = (self.meter - old_meter) * game.CAssetGame.m_to_score * (self.distance_score_times + self.score_times - 1)
        if add_score > 0 then
            self:AddScore(add_score)

            if self.game_rank then
                self.game_rank:AddMeterScore(add_score)
                self.game_rank:AddMeter(add_meter)
            end
        end
    end

    -- 引导监听
    sCtrlGuide:Listen("move_distance", self.pixel)
end

function CGameMap:CalcuGameTime()

    if self.is_gameover then
        return
    end

    -- 计算关卡剩余时间
    if self.config.game_time then
        self.count_down = self.config.game_time + self.more_time - self.total_dt
    end

    -- 游戏时间耗完
    if self.count_down and self.count_down <= 0 then

        -- 角色死亡
        self.player:Die()

        -- 时间清空为0
        self.count_down = 0
    end
end

function CGameMap:KillBoss(boss_id)

    print("boss is dead", boss_id)

    local boss_config = game.CAssetGameObject[boss_id]
    if boss_config and boss_config.score then

        -- 击杀BOSS积分加成
        self:AddScore(boss_config.score * self.kill_boss_score_times)
    end

    self.record["kill_boss"] = self.record["kill_boss"] or 0
    self.record["kill_boss"] = self.record["kill_boss"] + 1
end

function CGameMap:Revive(is_recover)
    
    -- 角色复活
    self.player:Revive(is_recover)

    -- 设置游戏结束标志
    self.is_gameover = false

    -- 播放音乐
    dxm.audio:playMusic(assets.sound.config.music.main, true)
end

return game.CGameMap

--endregion
