-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CObjectComponentMove = class("game.CObjectComponentMove")

local CObjectComponentMove = game.CObjectComponentMove

CObjectComponentMove.YStatus = {
    ground = 1,
    air = 2,
}

function CObjectComponentMove:ctor()
    self.game_object = nil

    -- 速度
    self.x_speed = 0
    self.y_speed = 0

    -- 当前空中状态
    self.current_y_status = CObjectComponentMove.YStatus.air

    -- 空中状态变更事件
    self.y_status_changed_event = dxm.CEvent.new()
end

function CObjectComponentMove:Init(game_object)

    if not game_object then
        return false
    end
    self.game_object = game_object

    -- 记录最近的坐标
    self.last_position_y = self.game_object:getPositionY()
    return true
end

function CObjectComponentMove:Create(...)
    local component = CObjectComponentMove.new()
    if component and component:Init(...) then
        return component
    end
    return nil
end

function CObjectComponentMove:Update(dt)

    -- 获取当前位置
    local x,y = self.game_object:getPosition()

    -- 变更移动坐标
    local x_distance = self.x_speed * dt

    -- 变更空中坐标
    local y_distance = self.y_speed * dt
    if self.ignore_gravity == false and self.current_y_status == CObjectComponentMove.YStatus.air then
        y_distance = y_distance - 0.5 * self.game_object.map.current_section_group_config.g * dt * dt
    end

    -- 设置新的坐标
    local new_x = x + x_distance
    local new_y = y + y_distance

    -- 获取当前状态下的形体大小
    local game_object_rect = self.game_object:GetCollideRect()
    if game_object_rect then
        game_object_rect.x = new_x
        game_object_rect.y = new_y
    end

    -- 判断是否横向碰撞
    if x_distance ~= 0 and self.game_object.ignore_ground_x == false then

        -- 判断竖向碰撞
        local tile_x = math.floor(new_x / game.CAssetGame.tile_width) + 1
        local tile_y = math.floor(new_y / game.CAssetGame.tile_height) + 1

        local collide_ground_rect = nil
        if self.game_object.map.tmx.ground[tile_x] then

            -- 从上至下扫描一轮地表
            local min_tile_y = tile_y
            if min_tile_y < 1 then
                min_tile_y = 1
            end
            for j=self.game_object.map.tmx.ground_height[tile_x],min_tile_y,-1 do

                local ground = self.game_object.map.tmx.ground[tile_x][j]
                if ground and ground.config.can_collide == true then

                    local ground_rect = ground:GetCollideRect()
                    if game_object_rect and ground_rect and cc.rectIntersectsRect(game_object_rect, ground_rect) then
                        collide_ground_rect = ground_rect
                        break
                    end
                end
            end
        end

        -- 判断当前是否存在横向碰撞地表
        if collide_ground_rect then

            -- 判断移动前的关系
            if x <= collide_ground_rect.x then
                x_distance = collide_ground_rect.x - x - 1
            elseif x >= collide_ground_rect.x + collide_ground_rect.width then
                x_distance = collide_ground_rect.x + collide_ground_rect.width - x + 1
            end
        end
    end
    x = x + x_distance

    -- 判断所在的横向位置，从上至下是否存在一个可支撑的面
    local ground_id = nil
    if self.game_object.ignore_ground == false then

        local tile_x = math.floor(x / game.CAssetGame.tile_width) + 1
        if y_distance <= 0 then

            -- 地表检测
            if self.game_object.map.tmx.ground[tile_x] then
                for i = self.game_object.map.tmx.ground_height[tile_x], 1, -1 do

                    local ground = self.game_object.map.tmx.ground[tile_x][i]
                    if ground and ground.config.top ~= true then

                        local tile_h = self.game_object.map.tmx:GetGroundHeight(i)
                        if (y >= tile_h and y + y_distance <= tile_h) then
                            ground_id = ground.config.id

                            -- 触发地块效果
                            ground:OnCollide(self.game_object)

                            y_distance = tile_h - y
                            break
                        end
                    end
                end
            end
        elseif y_distance > 0 then

            if self.game_object.map.tmx.ground[tile_x] then
                for i = 1, self.game_object.map.tmx.ground_height[tile_x] do

                    local ground = self.game_object.map.tmx.ground[tile_x][i]
                    if ground and ground.config.top == true then

                        local tile_h = self.game_object.map.tmx:GetGroundHeight(i - 1)
                        local object_collide_rect = self.game_object:GetCollideRect()
                        if (y + object_collide_rect.height <= tile_h and y + object_collide_rect.height + y_distance >= tile_h) then
                            y_distance = tile_h -(y + object_collide_rect.height)
                            break
                        end
                    end
                end
            end
        end

        -- 判断平台
        --[[
        for _, object in pairs(self.game_object.on_collide_objects) do
            if object.is_platform then
                local platform_y = object:GetPlatformHeight()
                if self.last_position_y >= platform_y and y <= platform_y then
                    ground_id = object.config.ground_id

                    y_distance = platform_y - y
                    x = x + object.component_list.move:GetXSpeed() * dt
                    y_distance = y_distance + object.component_list.move:GetYSpeed() * dt
                    break
                end
            end
        end
        ]]--
    end

    -- 更新地表逻辑
    self.game_object:UpdateGround(ground_id)

    if ground_id == nil and self.current_y_status == CObjectComponentMove.YStatus.ground then
        self:SetYStatus(CObjectComponentMove.YStatus.air)
    elseif ground_id ~= nil and self.current_y_status == CObjectComponentMove.YStatus.air then
        self:SetYStatus(CObjectComponentMove.YStatus.ground)
    end
    y = y + y_distance

    -- 记录最近的移动高度
    self.last_position_y = y

    -- 正式设置坐标
    self.game_object:setPosition(cc.p(x, y))
end

function CObjectComponentMove:ChgXSpeed(x_speed)
    self.x_speed = x_speed
end

function CObjectComponentMove:ChgYSpeed(y_speed)
    self.y_speed = y_speed
end

function CObjectComponentMove:GetXSpeed()
    return self.x_speed
end

function CObjectComponentMove:GetYSpeed()
    return self.y_speed
end

function CObjectComponentMove:SetYStatus(status)
    if self.current_y_status == status then
        return
    end
    self.current_y_status = status

    -- 触发事件
    self.y_status_changed_event:Fire(status)

    -- 落地后速度为0
    if self.current_y_status == CObjectComponentMove.YStatus.ground then
        self.y_speed = 0
    end
end

return CObjectComponentMove

-- endregion
