-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CGameObject = class("game.CGameObject", cc.Node)
local CGameObject = game.CGameObject

function CGameObject:ctor()

    -- 组件列表
    self.component_list = {}
    
    -- 朝向
    self.direction = game.GameObjectDirection.left

    -- 当前是否忽略重力
    self.ignore_gravity = false

    -- 当前是否忽略地表
    self.ignore_ground = false
    self.ignore_ground_x = false

    -- 碰撞矩形缓存
    self.body_rect = nil
    self.collide_rect = nil
	self.update_collide_rect = false

    -- 能否攻击与被攻击，攻击组别（不同则会互相伤害）
    self.can_attack = false
    self.damage = 1
    self.can_be_attacked = false
    self.attack_group = 0

    -- 能否捡取与被捡取
    self.can_pick_up = false
    self.can_be_picked_up = false

    -- 当前动画名称与是否循环
    self.current_animation_name = animation_name
    self.current_animation_is_loop = animation_is_loop

    -- 是否移除
    self.is_removed = false

    -- 地面
    self.ground_id = nil
    self.is_dead = false

    -- 碰撞物体
    self.on_collide_objects = nil

    -- 碰撞体积
    self.collide_rect_list = nil

    -- 碰撞优先级
    self.collide_level = 0

    -- 显示优先级
    self.zorder = 0

    -- 是否为平台类物件
    self.is_platform = false

    -- 是否为静态物件
    self.is_static = false

    -- 是否需要进行移除判断
    self.check_behind = true
end

function CGameObject:Init(map, config)
    if map == nil then
        return false
    end

    -- 设置地图
    self.map = map

    -- 安装更新器
    if self.map.updater then
        self.map.updater:InstallOnNode(self)
    end

    -- 物件配置，指定属性
    self.config = config
    if self.config == nil then
        return false
    end

    -- 伤害赋值
    self.damage = self.config.damage or game.CAssetGame.default_damage

    -- 设置级联
    self:setCascadeColorEnabled(true)
    self:setCascadeOpacityEnabled(true)

    -- 判断类型
    if self.config.file_type == "image" then

        local file_name = self.config.file_path
        if file_name then
            self.image = cc.Sprite:create(file_name)
            if self.image then
                self.image:setAnchorPoint(cc.p(0.5, 0))
                self:addChild(self.image)

                -- 设置级联
                self.image:setCascadeColorEnabled(true)
                self.image:setCascadeOpacityEnabled(true)
            end
        end
    elseif self.config.file_type == "spine" then
        self.animation = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(self.config.file_path .. ".json",
        self.config.file_path .. ".atlas", true)
        if self.animation ~= nil then

            -- 安装更新器
            if self.map.updater then
                self.map.updater:InstallOnNode(self.animation)
            end

            -- 临时处理，翻转一下角色
            if self.direction ~= self.config.direction then
                self.animation:setScaleX(-self.animation:getScaleX())
            end
            self:addChild(self.animation)

            -- 设置级联
            self.animation:setCascadeColorEnabled(true)
            self.animation:setCascadeOpacityEnabled(true)
        end
    end

    -- 设置是否静态物件
    self.is_static = self.config.is_static or false

    -- 添加所有组件
    if not self.is_static then
        self.component_list.hp = game.CObjectComponentHP:Create(self)
        self.component_list.move = game.CObjectComponentMove:Create(self)
        self.component_list.gravity = game.CObjectComponentGravity:Create(self)
    end

    if self.config.shadow_file_path then
        self.component_list.shadow = game.CObjectComponentShadow:Create(self)
    end

    -- 设置默认循环动画
    if self.animation then
        self.animation:setAnimation(0, self.config.default_animation or "animation", true)
    end

    -- 监听进入离开事件
    self:registerScriptHandler( function(event)
        if event == "enter" then
            self:OnEnter()
        elseif event == "exit" then
            self:OnExit()
        elseif event == "cleanup" then
            self:Dispose()
        end
    end )
    return true
end

function CGameObject:OnEnter()
    -- 播放音效
    if self.config.enter_sound then
        dxm.audio:playSound(self.config.enter_sound)
    end
end

function CGameObject:OnExit()
end

function CGameObject:Dispose()
end

function CGameObject:Create(...)

    local object = CGameObject.new()
    if object == nil or object:Init(...) == false then
        return nil
    end
    return object
end

function CGameObject:Update(dt)

    -- 判断是否被甩在角色后边过远的位置
    if self.is_removed or (self.check_behind and self:CheckBehind()) then
        return true
    end

    -- 清理碰撞体积
	self.update_collide_rect = true

    -- 组件更新
    for _,component in pairs(self.component_list) do
        component:Update(dt)
    end

    return false
end

function CGameObject:CanCollide(game_object)

    -- 无效个体
    if self.is_dead then
        return false
    end

    -- 统计碰撞对象
    if game_object.is_platform then
        return true
    end

    -- 主动攻击
    if self.can_attack and game_object.can_be_attacked and self.attack_group ~= game_object.attack_group then
        return true
    end

    -- 主动拾取
    if self.can_pick_up and game_object.can_be_picked_up and game_object.is_dead == false then
        return true
    end
    return false
end

function CGameObject:OnCollide(game_object)

    -- 无效个体
    if self.is_dead then
        return
    end

    -- 主动攻击
    if self.can_attack and game_object.can_be_attacked and self.attack_group ~= game_object.attack_group then
        self:Attack(game_object)
    end

    -- 主动拾取
    if self.can_pick_up and game_object.can_be_picked_up and game_object.is_dead == false then
        self:Pickup(game_object)
    end
end

function CGameObject:Attack(game_object)

    -- 默认对方直接受到伤害
    game_object:Hurt(self, self.damage, self.config.buff)
end

function CGameObject:Hurt(attacker, damage, buff)

    if self.is_dead then
        return
    end

    if self.image then
        self.image:runAction(cc.Repeat:create(cc.Sequence:create(cc.TintTo:create(0.1, cc.c3b(255,0,0)), cc.TintTo:create(0.1, cc.c3b(255,255,255))), 5))
    end

    local component_hp = self.component_list.hp
    if component_hp then
        component_hp:ChgHP(-damage)
        if component_hp.hp <= 0 then
            self:Die()
        end
    end
    return true
end

function CGameObject:Die()

    if self.is_dead then
        return
    end
    self.is_dead = true

    -- 全部受重力影响
    self.ignore_gravity = false
    self.ignore_ground = true
    self.ignore_ground_x = true

    -- 将怪物改为飞行死亡
    local component_move = self.component_list.move
    if component_move then
        component_move.x_speed = 1000
        component_move.y_speed = 500
    end

    --停止所有动画
    self:stopAllActions()
    if self.image then
        self.image:stopAllActions()
        self.image:setColor(cc.c3b(50,50,50))
    end
end

function CGameObject:Pickup(game_object)

    -- 默认道具生效
    game_object:Effect(self)
end

function CGameObject:Effect(picker)

    -- TODO: 不同的道具有不一样的效果
end

function CGameObject:GetBodyRect()

    if not self.body_rect then
        self.body_rect = self:CalcBodyRect()
    end
    return self.body_rect
end

function CGameObject:CalcBodyRect()

    if not self.config.rect then
        if self.image then
            self.config.rect = self.image:getBoundingBox()
        elseif self.animation then
            self.animation:update(0)
            self.config.rect = self.animation:getBoundingBox()
		else
			self.config.rect = {x=0,y=0,width=0,height=0}
        end
		
		if self.config.rect.width < 0 then
			self.config.rect.x = self.config.rect.x + self.config.rect.width
			self.config.rect.width = math.abs(self.config.rect.width)
		end
        if self.config.rect.height < 0 then
			self.config.rect.y = self.config.rect.y + self.config.rect.height
			self.config.rect.height = math.abs(self.config.rect.height)
		end
    end
    return self.config.rect
end

function CGameObject:GetCollideRect()

	if self.collide_rect and not self.update_collide_rect then
		return self.collide_rect
	end
	
	self.update_collide_rect = false
	
	local body_rect = self:GetBodyRect()
	if body_rect then
	
		if not self.collide_rect then
			self.collide_rect = {x=0,y=0,width=body_rect.width,height=body_rect.height}
		end
		
		local x,y = self:getPosition()
		self.collide_rect.x = body_rect.x + x
		self.collide_rect.y = body_rect.y + y
	end
    return self.collide_rect
end

function CGameObject:CheckBehind()

    local rect = self:GetCollideRect()
    if rect then
        if (rect.x + rect.width < - self.map.tmx:getPositionX() or rect.y < - rect.height) then
            return true
        end
    elseif self:getPositionX() < - self.map.tmx:getPositionX() then
        return true
    end
    return false
end

function CGameObject:Throw(game_object_id)
    -- TODO:
end

function CGameObject:UpdateZOrder()
    --获取地图上格子位置
    local title_y = math.floor(self:getPositionY() / game.CAssetGame.tile_height)
    self:setLocalZOrder(title_y * game.CGameTMX.ZOrder.max + game.CGameTMX.ZOrder.object)    
end

function CGameObject:UpdateGround(ground_id)
    
    if self.ground_id == ground_id then
        return false
    end

    --移除旧的地表属性
    if self.ground_id then
        local ground_config = game.CAssetGameGround[self.ground_id]
        
        --是否需要移除
        if ground_config.remove_effect then
            ground_config.remove_effect(ground_config,self)
        end
    end

    self.ground_id = ground_id

    --添加地表属性
    if self.ground_id then
        local ground_config = game.CAssetGameGround[self.ground_id]
        
        --添加地表属性
        if ground_config.add_effect then
            ground_config.add_effect(ground_config,self)
        end
    end
    return true
end

return game.CGameObject

-- endregion
