--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

game.CGameObjectEagle = class("game.CGameObjectEagle", game.CGameObject)
local CGameObjectEagle = game.CGameObjectEagle

CGameObjectEagle.Status = {
    idle = 1,
    run = 2,
    dead = 3,
}

function CGameObjectEagle:ctor()
    CGameObjectEagle.super.ctor(self)

    -- 物件朝向为右侧
    self.direction = game.GameObjectDirection.right

    -- 被非1势力攻击
    self.can_be_attacked = true
    self.attack_group = 1

    -- 无视重力与地形
    self.ignore_gravity = true
    self.ignore_ground = true
    self.ignore_ground_x = true

    -- 属于BOSS级别
    self.is_monster = true

    -- 行动时间计数
    self.total_dt = 0

    -- 攻击cd
    self.attack_cd = 1
    self.throw_count = 1
end

function CGameObjectEagle:Init(map, config, level)

    level = level or 1

    -- 初始化
    if CGameObjectEagle.super.Init(self, map, config) == false then
        return false
    end

    -- 设置血量
    local component_hp = self.component_list.hp
    if component_hp then
        local max_hp = 60*(level+1)
        component_hp:MoreHP(max_hp)
    end

    -- 默认状态
    self.status = CGameObjectEagle.Status.idle

    -- 累计时间
    self.total_dt = 0

    -- 监听摄像机移动
    self.listener = self.map.tmx.camera_position_changed_event:Connect(function() self:CameraPositionChangedCallback() end)

    -- 血量显示
    self.hp_widget = require("app.views.game.game_hp"):Create(self)
    if self.hp_widget then

        local body_rect = self:GetBodyRect()
        self.hp_widget:setPosition(cc.p(body_rect.x + body_rect.width/2, body_rect.y + body_rect.height + 50))
        self:addChild(self.hp_widget)
    end

    -- 攻击cd
    self.attack_cd = self.config.attack_cd
    self.throw_count = self.config.throw_count

    return true
end

function CGameObjectEagle:Dispose()

    -- 注销事件
    if self.listener then
        self.map.tmx.camera_position_changed_event:Disconnect(self.listener)
        self.listener = nil
    end

    -- 移除
    if self.hp_widget then
        self.hp_widget:removeFromParent()
        self.hp_widget = nil
    end
end

function CGameObjectEagle:Create(...)

    local eagle = CGameObjectEagle.new()
    if eagle == nil or eagle:Init(...) == false then
        return nil
    end
    return eagle
end

function CGameObjectEagle:Update(dt)

    if self.status == CGameObjectEagle.Status.run then

        self.total_dt = self.total_dt + dt
        if self.total_dt >= self.attack_cd then
            self:Throw()
            self.total_dt = self.total_dt - self.attack_cd
        end
    end

    return CGameObjectEagle.super.Update(self, dt)
end

function CGameObjectEagle:Throw()

    -- 角色死亡就不抛了
    if not self.map.player or self.map.player.is_dead then
        return
    end
    
    -- 扔火堆
    local count = self.map.random:RandomInt32(1,self.throw_count)
    for i=1,count do

        local throw_object_id = self.config.throw_object_id
        local game_object = self.map.tmx:CreateObjectByID(throw_object_id[self.map.random:RandomInt32(1,#throw_object_id)], self:getPositionX(), self:getPositionY())
        if game_object then

            local component_move = game_object.component_list.move
            if component_move then
                component_move.x_speed = self.map.random:RandomInt32(self.config.x_throw_speed_min,self.config.x_throw_speed_max)
                component_move.y_speed = self.map.random:RandomInt32(self.config.y_throw_speed_min,self.config.y_throw_speed_max)
            end
        end
    end
end

function CGameObjectEagle:Hurt(attack, damage)

    if self.status == CGameObjectEagle.Status.dead then
        return
    end

    self:runAction(cc.Repeat:create(cc.Sequence:create(cc.TintTo:create(0.1, cc.c3b(255,0,0)), cc.TintTo:create(0.1, cc.c3b(255,255,255))), 5))

    -- 受击音效
    dxm.audio:playSound(self.config.hurt_sound)

    -- 伤害显示
    local number = require("app.views.game.game_score"):Create(-damage, cc.c3b(255,0,0))
    if number then
        number:setPosition(cc.p(0, self:GetBodyRect().height))
        number:setScale(2)
        self:addChild(number)
    end

    local component_hp = self.component_list.hp
    if component_hp then
        component_hp:ChgHP(-damage)
        if component_hp.hp <= 0 then
            self:Die()
        elseif self.config.fury_percent and (component_hp.hp / component_hp.max_hp) <= self.config.fury_percent then
            self.is_fury = true
            self.attack_cd = self.config.fury_attack_cd
            self.throw_count = self.config.fury_throw_count

            -- TODO: 变更主动画
            if self.animation then
                --self.animation:setAnimation(0, "animation1", true)
            end
        end
    end
    return true
end

function CGameObjectEagle:Die()

    if self.status == CGameObjectEagle.Status.dead then
        return
    end

    self.status = CGameObjectEagle.Status.dead

    -- 死亡音效
    if self.config.die_sound then
        dxm.audio:playSound(self.config.die_sound)
    end

    -- 移动速度为0
    local component_move = self.component_list.move
    if component_move then
        component_move.x_speed = 0
    end

    -- 播放死亡动画
    if self.animation then
        self.map.time_scale = 0.1
        self.map:runAction(cc.Sequence:create(cc.DelayTime:create(0.2), 
        cc.CallFunc:create(function() 
            self.map.time_scale = 1 
            self.map:Finish()
        end)))
        self.animation:setAnimation(0, "death", false)
    end

    -- 统计BOSS击杀
    self.map:KillBoss(self.config.id)

    -- 死亡奖励爆炸
    --[[
    if self.config.die_reward then

        for i=1,#self.config.die_reward do

            local reward_entry = self.config.die_reward[i]
            for j=1,reward_entry.count do

                local game_object = self.map.tmx:CreateObjectByID(reward_entry.id, self:getPositionX(), self:getPositionY()+100)
                if game_object then
                    local component_move = game_object.component_list.move
                    if component_move then
                        component_move.x_speed = self.map.random:RandomInt32(500, 800)
                        component_move.y_speed = self.map.random:RandomInt32(600, 800)
                    end
                end
            end
        end
    end
    ]]--

    self.ignore_gravity = false
    self.ignore_ground = true
end

function CGameObjectEagle:CameraPositionChangedCallback()

    local target_pos_x = -self.map.tmx:getPositionX()+800

    -- 判断当前是否为空闲模式
    if self.status == CGameObjectEagle.Status.idle then

        -- 判断当前的坐标是否在屏幕小于800的位置
        if self:getPositionX() < target_pos_x then
            self.status = CGameObjectEagle.Status.run
        end
    end
    if self.status == CGameObjectEagle.Status.run then
        self:setPositionX(target_pos_x)
    end
end

return game.CGameObjectEagle

--endregion
