-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CGameObjectHangMonkey = class("game.CGameObjectHangMonkey", game.CGameObject)
local CGameObjectHangMonkey = game.CGameObjectHangMonkey

function CGameObjectHangMonkey:ctor()
    CGameObjectHangMonkey.super.ctor(self)

    self.ignore_gravity = true
    self.can_attack = true
    self.attack_group = 1

    --self.can_be_attacked = true
end

function CGameObjectHangMonkey:Init(map, config)
    if CGameObjectHangMonkey.super.Init(self, map, config) == false then
        return false
    end

    self.monkey_list = { }
    if self.config.file_type == "image" then
        self.monkey_list[1] = self.image
    else
        self.monkey_list[1] = self.animation
    end

    local parent = self
    for i = 1, self.config.monkey_count - 1 do
        local monkey = nil
        -- 判断类型
        if self.config.file_type == "image" then

            local file_name = self.config.file_path
            if file_name then
                monkey = cc.Sprite:create(file_name)
            end
        elseif self.config.file_type == "spine" then
            local monkey = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(self.config.file_path .. ".json",
            self.config.file_path .. ".atlas", true)
            if monkey ~= nil then
                -- 安装更新器
                if self.map.updater then
                    self.map.updater:InstallOnNode(monkey)
                end
            end
        end

        if monkey then
            monkey:setName("monkey")
            monkey:setAnchorPoint(0, 0)
            monkey:setPosition(0, -60)
            parent:addChild(monkey)
            parent = monkey
            self.monkey_list[i + 1] = monkey
        end
    end

    local hp_component = self.component_list.hp
    if hp_component then
        hp_component:MoreHP(self.config.max_hp or 1)
    end

    self.rotate_angle = 0
    self.child_rotate_angle = 0
    self.rotate_status = "right"

    return true
end

function CGameObjectHangMonkey:Create(...)
    local object = CGameObjectHangMonkey.new()
    if object and object:Init(...) then
        return object
    end
end

function CGameObjectHangMonkey:Update(dt)
    local ret = CGameObjectHangMonkey.super.Update(self, dt)

    local rotate_angle = self.config.rotate_angle / self.config.rotate_time * dt
    local child_rotate_angle = 10 / self.config.rotate_time * dt
    if self.rotate_status == "right" then
        if self.rotate_angle >= self.config.rotate_angle then
            self.rotate_status = "left"
        end
        self.rotate_angle = self.rotate_angle + rotate_angle
        self.child_rotate_angle = self.child_rotate_angle + child_rotate_angle
    elseif self.rotate_status == "left" then
        if self.rotate_angle <= - self.config.rotate_angle then
            self.rotate_status = "right"
        end

        self.rotate_angle = self.rotate_angle - rotate_angle
        self.child_rotate_angle = self.child_rotate_angle - child_rotate_angle
    end
    self:setRotation(self.rotate_angle)
    local parent = self
    while true do
        local monkey = parent:getChildByName("monkey")
        if not monkey then
            break
        end
        monkey:setRotation(self.child_rotate_angle)
        parent = monkey
    end

    --重置矩形
    self.collide_rect = nil
    self.collide_rect_list = self:CalcCollideRect()

    return ret
end

function CGameObjectHangMonkey:OnEnter()
    CGameObjectHangMonkey.super.OnEnter(self)
    self:setAnchorPoint(0.5, 1)
end

function CGameObjectHangMonkey:CalcCollideRect()
    local collide_rect_list = { }

    for i = 1, #self.monkey_list do
        local monkey = self.monkey_list[i]
        if monkey then
            local world_position = monkey:convertToWorldSpace(cc.p(monkey:getPositionX(), monkey:getPositionY()))
            local local_position = self:getParent():convertToNodeSpace(world_position)
            local rect = nil
            if self.image then
                rect = monkey:getBoundingBox()
            end
            if self.animation then
                rect = monkey:getBoundingBox()
            end

            if rect then
                rect.x = rect.x + local_position.x
                rect.y = rect.y + local_position.y

                if rect.width < 0 then
                    rect.x = rect.x + rect.width
                    rect.width = math.abs(rect.width)
                end
            end
            collide_rect_list[#collide_rect_list + 1] = rect
        end
    end

    return collide_rect_list
end

return CGameObjectHangMonkey

-- endregion
