-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CGamePlayer = class("game.CGamePlayer", game.CGameObject)
local CGamePlayer = game.CGamePlayer

-- 角色状态机枚举
CGamePlayer.MoveStatus = {
    idle = 1,
    accelerate = 2,
    decelerate = 3,
    cancel_accelerate = 4,
    cancel_decelerate = 5,
    dash = 6,
    super_dash = 7,
    stand = 8,
    drop = 9,
    drop_protect = 10,
    die = 11,
    drop_protect2 = 12,
}

-- 具体动画, 坐骑状态 + 基本动作
CGamePlayer.BaseAnimationStatus =
{
    stand = "standby",
    run = "run",
    jump = "jump",
    jump2 = "jump2",
    brake = "brake1",
    brake_run = "brake2",
    cancel_brake = "brake3",
    death = "death",
    dash = "sprint",
    super_dash = "sprint_loop",
    ride = "ride",
}

CGamePlayer.MountStatus = {
    not_mount = "",
    scooter = "scooter"
}

function CGamePlayer:ctor()
    CGamePlayer.super.ctor(self)

    -- 能否空中移动
    self.can_move_in_the_air = false

    -- BUFF系统
    self.flag = { }
    self.buff = { }
    self.buff_changed_event = dxm.CEvent.new()

    -- 捡取东西
    self.pickup_list = { }
    self.animation_throw = nil

    -- 吸引过来的东西
    self.waiting_for_pickup_list = {}

    -- 移动状态
    self.current_move_status = nil
    self.current_move_status_exit_callback = nil

    -- 跳跃状态
    self.current_jump_status = nil

    -- 坐骑信息
    self.mount_id = nil
    self.mount_animation = nil

    -- 场景物件的属性设置
    self.can_be_attacked = true
    self.can_pick_up = true
    self.attack_group = 0

    -- 地面状态
    self.ground_flag = nil

    -- 角色当前武器以及子弹数
    self.weapon_id = nil
    self.bullet_count = 0
    self.weapon_changed_event = dxm.CEvent.new()

    -- 当前跳跃次数与冲刺次数
    self.current_jump_times = 0
    self.current_dash_times = 0

    -- 设置碰撞优先级
    self.collide_level = 1

    -- 即将死亡
    self.will_die = false

    -- 额外伤害
    self.addition_damage = 0

    -- 减免伤害
    self.reduction_damage = 0

    -- 总的跳跃次数
    self.total_jump_times = 0
    self.total_jump_twice_times = 0

    -- 总的冲刺次数
    self.total_dash_times = 0

    -- 超级冲刺加成
    self.super_dash_distance_addition = 0
    self.super_dash_animation = nil
    self.super_dash_circle = nil
    self.super_dash_time = nil

    -- 不进行正常的移除判断
    self.check_behind = false

    -- 摄像机跟踪标记
    self.update_camera = true

    -- 吸铁石翻倍
    self.magnet_dt_times = 1
end

function CGamePlayer:Init(map, config)

    -- 基类初始化
    if CGamePlayer.super.Init(self, map, config) == false then
        return false
    end

    -- 角色动画需要反转一下
    if self.animation then
        self.animation:setScaleX(-self.animation:getScaleX())
    end

    -- 设置血量
    local component_hp = self.component_list.hp
    if component_hp then
        component_hp:MoreHP(self.config.max_hp or 1)
        component_hp.hp_changed_event:Connect(function(hp) self:HPChangedCallback(hp) end)
    end

    self.is_dead = false
    self.current_jump_times = 0
    self.current_move_speed = self.map.current_section_group_config.move_speed

    -- 提供初始速度
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
    end

    -- 操作状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.stand)

    -- 监听空中状态变更
    self.component_list.move.y_status_changed_event:Connect( function(y_status) self:YStatusChangedCallback(y_status) end)

    -- 更新动画
    self:UpdateAnimation()

    -- 扔的动画加载
    self.animation_throw = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation("spine/throw/skeleton.json", "spine/throw/skeleton.atlas", true)
    if self.animation_throw then
        self:addChild(self.animation_throw)
    end
    return true
end

function CGamePlayer:Dispose()

    if self.run_sound_handle then
        dxm.audio:stopSound(self.run_sound_handle)
        self.run_sound_handle = nil
    end
end

function CGamePlayer:Create(...)

    local player = CGamePlayer.new()
    if player == nil or player:Init(...) == false then
        return nil
    end
    return player
end

function CGamePlayer:Update(dt)

    --sProfileSystem:StartSampling("CGamePlayer:Update1")

    local move_component = self.component_list.move
    local y_speed = move_component.y_speed

    -- 更新物件
    CGamePlayer.super.Update(self, dt)

    -- 判断是否开始下落
    if y_speed >= 0 and move_component.y_speed < 0 then

        -- 引导监听
        sCtrlGuide:Listen("jump_down")
    end

    -- 更新BUFF
    self:ConsumeBuff("dt", dt)

    -- 地表影响
    if self.current_move_status == CGamePlayer.MoveStatus.cancel_accelerate then

        if self.ground_flag == game.CAssetGameGroundFlag.ice_ground then
            local current_move_speed = move_component:GetXSpeed()
            local move_speed = current_move_speed - game.CAssetGameGround[self.ground_id].decelerate_speed
            if move_speed > self.map.current_section_group_config.move_speed then
                move_component:ChgXSpeed(move_speed)
            else
                self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)
                move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
            end
        else
            self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)
            move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
        end
    elseif self.current_move_status == CGamePlayer.MoveStatus.super_dash then

        self.super_dash_time = self.super_dash_time - dt
        if self.super_dash_time < 0 then
            self.super_dash_time = 0
            self.total_super_dash_time = 0

            -- 判断前方是否有安全的落脚点


            if not self.super_dash_need_drop_protect then
                self:DropProtect2()
            else
                self:DropProtect()
            end
        end
    end

    -- 进行吸铁石的判定
    if self:GetFlag(game.CAssetGameFlag.magnet) > 0 then
        self:Magnet()
    end
    for key,object in pairs(self.waiting_for_pickup_list) do
        if object and object:Update(dt) then
            object:removeFromParent()
            self.waiting_for_pickup_list[object] = nil
        end
    end

    -- 扣减时间消耗体力
    if not self.map.is_gameover and self.component_list.hp then
        self.component_list.hp:ChgHP(-dt*game.CAssetGame.dt_to_hp)
    end

    -- 判断游戏是否结束
    self:CheckDrop()

    -- 判断角色是否需要死亡
    if self.will_die and self:GetFlag(game.CAssetGameFlag.never_die) <= 0 then
        self:Die()
    end

    --sProfileSystem:StopSampling("CGamePlayer:Update1")
    return false
end

function CGamePlayer:GetSafePlace()

    local x,y = self:getPosition()

    -- 计算角色当前或者前方的平地高度
    local distance = 0
    local height = 0
    local has_safe_place = false

    local current_tile_index = math.floor(x / game.CAssetGame.tile_width) + 1
    for i=current_tile_index, self.map.tmx.last_end_x do

        local grounds = self.map.tmx.ground[i]
        if grounds then

            -- 从低往高来判断地表
            local ground_height = self.map.tmx.ground_height[i]
            for j=1,ground_height do
                local ground = grounds[j]
                if ground then

                    local collide_rect = ground:GetCollideRect()
                    if collide_rect then

                        local target_y = collide_rect.y + collide_rect.height
                        if target_y >= y then

                            -- 高于当前坐标的地表直接采纳
                            if x >= collide_rect.x and x <= collide_rect.x + collide_rect.width then
                                distance = 0
                            else
                                distance = collide_rect.x - x
                            end
                            height = target_y - y
                            has_safe_place = true
                            break
                        else
                            
                            -- 判断能否在下落时间内着地
                            local height = y - target_y
                            local drop_dt = math.sqrt(2 * height / game.CAssetGame.default_g)
                            local min_distance = self.map.current_section_group_config.move_speed * drop_dt
                            local min_x = x + min_distance
                            if min_x <= collide_rect.x + collide_rect.width then

                                if min_x >= collide_rect.x then
                                    distance = min_distance
                                else
                                    distance = collide_rect.x - min_distance
                                end
                                height = 0
                                has_safe_place = true
                                break
                            end
                        end
                    end
                end
            end
        end

        if has_safe_place then
            break
        end
    end

    -- 返回结果集
    return distance, height, has_safe_place
end

function CGamePlayer:GotoSafePlace()

    local x_speed = self.map.current_section_group_config.move_speed + self.config.dash_speed*3

    local distance, height, has_safe_place = self:GetSafePlace()
    if not has_safe_place then

        -- 继续冲刺一个单元格
        self.super_dash_time = self.super_dash_time + (game.CAssetGame.tile_width / x_speed)
    else

        local super_dash_callback = function() 

            -- 判断是否需要蹦一下
            if height > 0 then
            
                local jump_dt = math.sqrt(2 * height / game.CAssetGame.default_g)
                local jump_speed = game.CAssetGame.default_g * jump_dt
                local component_move = self.component_list.move
                if component_move then
                    component_move:ChgXSpeed(0)
                    component_move:ChgYSpeed(jump_speed)
                end
            end

            -- TODO: 等待落地之后恢复操作
        end

        if distance > 0 then

            -- 继续冲刺指定的距离，然后做出对应的操作
            self.super_dash_time = self.super_dash_time + (distance / x_speed)
            self.super_dash_callback = super_dash_callback
        else

            -- 直接运行冲刺回调
            super_dash_callback()
        end
    end
end

function CGamePlayer:Magnet()

    local pos = cc.p(self:getPosition())
    local get_distance_function = cc.pGetDistance
    local pickup_function = function(object)
        if not object.can_be_picked_up or object.is_dead then
            return
        end

        local distance = get_distance_function(pos, cc.p(object:getPosition()))
        if distance < game.CAssetGame.magnet_distance then
            self:Pickup(object)
        end
    end

    for key, object in pairs(self.map.tmx.static_objects) do
        pickup_function(object)
    end
    for key, object in pairs(self.map.tmx.dynamic_objects) do
        pickup_function(object)
    end
end

function CGamePlayer:UpdateGround(ground_id)

    if CGamePlayer.super.UpdateGround(self, ground_id) then
        
        -- 播放跑步音效
        self:PlayRunSound()

        return true
    end
    return false
end

-- 掉坑救援
function CGamePlayer:CheckDrop()

    -- 判断是否在下落到某高度死亡
    local x,y = self:getPosition()
    if self.component_list.move.y_speed < 0 and y < game.CAssetGame.drop_height then

        -- 受到掉落伤害
        self:Hurt(nil, game.CAssetGame.drop_damage)

        -- 判断是否死亡
        if self.component_list.hp.hp > 0 then
            self:Drop()
        end

        --self.component_list.hp:ChgHP(-self.component_list.hp.hp)
        --self:Die()
    end
end

-- 掉落操作
function CGamePlayer:Drop()

    -- 如果当前已经处于空闲运动模式则无效
    if self.current_move_status == CGamePlayer.MoveStatus.drop then
        return false
    end

    -- 根据不同的模式进入不同的状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.drop, function() self:_CancelDrop() end)

    -- 将角色位置保留
    local component_move = self.component_list.move
    if component_move then
        component_move:ChgXSpeed(0)
        component_move:ChgYSpeed(0)
    end
    self.ignore_gravity = true

    -- 关闭摄像机跟踪
    self.update_camera = false

    -- 不允许用户做任何操作
    self:AddFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_jump)
    self:AddFlag(game.CAssetGameFlag.cannot_move_dash)

    -- 停顿一段时间
    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.1), cc.CallFunc:create(function()
        self:DropProtect()
    end)))
    return true
end

function CGamePlayer:_CancelDrop()

    -- 判断当前是否处于下落状态
    if self.current_move_status ~= CGamePlayer.MoveStatus.drop then
        return
    end

    -- 解开重力束缚
    self.ignore_gravity = false

    -- 解开摄像机跟踪
    self.update_camera = true

    -- 解开操作限制
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_jump)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_dash)
end

-- 掉落保护操作
function CGamePlayer:DropProtect()

    -- 如果当前已经处于空闲运动模式则无效
    if self.current_move_status == CGamePlayer.MoveStatus.drop_protect then
        return false
    end

    -- 根据不同的模式进入不同的状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.drop_protect, function() self:_CancelDropProtect() end)

    -- 不允许用户做任何操作
    self:AddFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_jump)
    self:AddFlag(game.CAssetGameFlag.cannot_move_dash)
    self:AddFlag(game.CAssetGameFlag.never_die)

    -- 获取当前坐标
    local x,y = self:getPosition()

    -- 计算角色当前或者前方的平地高度
    local target_ground = nil
    local current_tile_index = math.floor(x / game.CAssetGame.tile_width) + 1
    for i=current_tile_index, self.map.tmx.last_end_x do

        local grounds = self.map.tmx.ground[i]
        if grounds then

            -- 从低往高来判断地表
            local ground_height = self.map.tmx.ground_height[i]
            for j=1,ground_height do
                local ground = grounds[j]
                if ground then
                    target_ground = ground
                    break

                    --[[
                    -- 判断是否在当前坐标上
                    local collide_rect = ground:GetCollideRect()
                    if collide_rect then
                        local target_y = collide_rect.y + collide_rect.height
                        if target_y >= y then
                            target_ground = ground
                            break
                        else
                            
                            -- 判断能否在下落时间内着地
                            local height = y - target_y
                            local drop_dt = math.sqrt(2 * height / game.CAssetGame.default_g)
                            local distance = self.map.current_section_group_config.move_speed * drop_dt
                            if x + distance <= collide_rect.x + collide_rect.width then
                                target_ground = ground
                                break
                            end
                        end
                    end
                    ]]--
                end
            end
        end

        if target_ground then
            break
        end
    end

    -- 如果存在最近的平地，则先让角色跳跃至该高度，然后冲刺
    local jump_speed = 0
    local jump_dt = 0
    local super_dash_distance = 0
    if target_ground then

        -- 获取所在的高度
        local collide_rect = target_ground:GetCollideRect()
        if collide_rect then

            -- 计算跳跃初速度
            local target_y = collide_rect.y + collide_rect.height
            if target_y > y then

                local height = target_y - y + 1
                if height > 0 then
                    jump_dt = math.sqrt(2 * height / game.CAssetGame.default_g)
                    jump_speed = game.CAssetGame.default_g * jump_dt
                end
            end

            -- 计算距离
            super_dash_distance = collide_rect.x - x
            if super_dash_distance < 0 then
                super_dash_distance = 0
            end
        end
    end

    -- 原地跳跃
    if jump_speed > 0 then

        -- 将角色开始没有横向速度
        local component_move = self.component_list.move
        if component_move then
            component_move:ChgXSpeed(0)
            component_move:ChgYSpeed(jump_speed)
        end
        self:PlayAnimation(CGamePlayer.BaseAnimationStatus.jump)
        dxm.audio:playSound(assets.sound.config.sound.game_player_jump)

        -- 关闭摄像机跟踪
        self.update_camera = false

        -- 指定时间后进行超级冲刺
        self:runAction(cc.Sequence:create(cc.DelayTime:create(jump_dt), 
        cc.CallFunc:create(function() 
            if super_dash_distance <= 0 then
                self:DropProtect2()
            else
                self:SuperDash(super_dash_distance, false)
            end 
        end)))
    else

        -- 平着来
        if super_dash_distance <= 0 then
            self:DropProtect2()
        else
            self:SuperDash(super_dash_distance, false)
        end 
    end

    -- 送个无敌保护
    self:AddBuff(game.CAssetGameBuff.super_dash_finish_invincible)

    return true
end

function CGamePlayer:_CancelDropProtect()

    if self.current_move_status ~= CGamePlayer.MoveStatus.drop_protect then
        return
    end

    -- 解开摄像机跟踪标记
    self.update_camera = true

    -- 解开操作限制
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_jump)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_dash)
    self:RemoveFlag(game.CAssetGameFlag.never_die)
end

function CGamePlayer:DropProtect2()

    -- 如果当前已经处于空闲运动模式则无效
    if self.current_move_status == CGamePlayer.MoveStatus.drop_protect2 then
        return false
    end

    -- 根据不同的模式进入不同的状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.drop_protect2, function() self:_CancelDropProtect2() end)

    -- 速度调整为0等着落地
    local move_component = self.component_list.move
    if move_component then
        --move_component:ChgXSpeed(0)
    end

    -- 关闭摄像机跟踪
    --self.update_camera = false
end

function CGamePlayer:_CancelDropProtect2()

    if self.current_move_status ~= CGamePlayer.MoveStatus.drop_protect2 then
        return
    end
    
    -- 打开摄像机跟踪
    --self.update_camera = true
end

-- 待机操作
function CGamePlayer:Idle()

    -- 如果当前已经处于空闲运动模式则无效
    if self.current_move_status == CGamePlayer.MoveStatus.idle then
        return false
    end

    -- 根据不同的模式进入不同的状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)
end

-- 站立操作
function CGamePlayer:Stand()

    -- 如果当前已经处于空闲运动模式则无效
    if self.current_move_status == CGamePlayer.MoveStatus.stand then
        return false
    end

    -- 根据不同的模式进入不同的状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.stand)

    -- 速度调整回来
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(0)
    end

    -- 播放动画
    self:UpdateAnimation()
end

-- 加速操作
function CGamePlayer:MoveAccelerate()

    -- 有禁止移动标记
    if self:GetFlag(game.CAssetGameFlag.cannot_move_accelerate) > 0 then
        return false
    end

    -- 已在加减速过程中则无法控制加速
    if self.current_move_status == CGamePlayer.MoveStatus.accelerate then
        return false
    end

    -- 设置状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.accelerate, function() self:_CancelMoveAccelerate() end)

    -- 在陆地上根据标记判断能否享受前后操作
    if self.ground_id ~= nil and self.can_move_in_the_air == true then
        move_component:ChgXSpeed(move_component:GetXSpeed() + self.config.accelerate_speed)
    end

    -- 播放动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.run, true)

    return true
end

-- 减速操作
function CGamePlayer:MoveDecelerate()

    -- 有禁止移动标记
    if self:GetFlag(game.CAssetGameFlag.cannot_move_decelerate) > 0 then
        return false
    end

    -- 已在加减速过程中则无法控制加速
    if self.current_move_status == CGamePlayer.MoveStatus.decelerate then
        return false
    end

    -- 设置状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.decelerate, function() self:_CancelMoveDecelerate() end)

    -- 在空中则无法享受前后操作
    if self.ground_id ~= nil and self.can_move_in_the_air == true then
        move_component:ChgXSpeed(move_component:GetXSpeed() + self.config.decelerate_speed)
    end

    -- 播放动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.brake, true)

    return true
end

-- 冲刺操作
function CGamePlayer:Dash(force)

    -- 有禁止移动标记
    if not force and self:GetFlag(game.CAssetGameFlag.cannot_move_dash) > 0 then
        return
    end

    -- 判断当前是否在冲刺过程中
    if self.current_move_status == CGamePlayer.MoveStatus.dash then
        return
    end

    -- 判断当前是否拥有冲刺次数
    if self.current_dash_times >= self.config.max_dash_times then
        return
    end

    -- 进入冲刺模式
    self:_SetMoveStatus(CGamePlayer.MoveStatus.dash, function() self:_CancelDash() end)

    -- 增加冲刺次数
    self.total_dash_times = self.total_dash_times + 1

    -- 不允许用户做任何操作
    self:AddFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_jump)

    -- 带攻击能力
    -- self.can_attack = true

    -- 提供向前的速度
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed + self.config.dash_speed)
        move_component:ChgYSpeed(0)
    end

    -- 改为滞空模式且不受重力影响
    self.ignore_gravity = true
    self.ignore_ground = true

    -- 冲刺音效
    dxm.audio:playSound(assets.sound.config.sound.game_player_dash)

    -- 播放动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.dash)

    -- 增加冲刺次数
    self.current_dash_times = self.current_dash_times + 1

    -- 突刺一秒后离开突刺
    self.dash_action = self:runAction(cc.Sequence:create(cc.DelayTime:create(self.config.dash_dt), cc.CallFunc:create(function() self:Idle() end)))

    -- 创建一团灰
    local animation_dust = sSpineUtils:CreateNodeAnimation("spine/dust", true)
    if animation_dust then
        animation_dust:setAnimation(0, "animation", false)
        animation_dust:setCompleteListener(function() sActionUtils:Delay(0.1, nil, animation_dust) animation_dust:removeFromParent() end)

        animation_dust:setPosition(cc.p(self:getPosition()))
        self.map.tmx:addChild(animation_dust, self:getZOrder())
    end

    -- 创建加速线
    self.animation_speed_line = sSpineUtils:CreateNodeAnimation("spine/speed_line", true)
    if self.animation_speed_line then
        self.animation_speed_line:setAnimation(0, "animation", true)

        self.animation_speed_line:setPosition(dxm.VisibleRect:center())
        self.map:addChild(self.animation_speed_line)
    end

    -- 引导监听
    sCtrlGuide:Listen("dash")
end

-- 超级冲刺操作
function CGamePlayer:SuperDash(distance, need_drop_protect)

    need_drop_protect = need_drop_protect or true

    -- 判断能否进入超级冲刺
    if self.current_move_status == CGamePlayer.MoveStatus.super_dash then
        return
    end

    -- 进入超级冲刺模式
    self:_SetMoveStatus(CGamePlayer.MoveStatus.super_dash, function() self:_CancelSuperDash() end)

    -- 清空怒气
    self.map.fury = 0

    -- 添加冲刺BUFF
    self:AddBuff(game.CAssetGameBuff.super_dash)

    -- 置空角色的跳跃次数与冲刺次数
    self.current_jump_times = 0
    self.current_dash_times = 0

    -- 将角色跳跃到某个高度
    local target_y = self:getPositionY()
    if target_y < game.CAssetGame.super_dash_min_height then
        target_y = game.CAssetGame.super_dash_min_height
    end
    self:runAction(cc.Sequence:create(cc.MoveBy:create(0.5, cc.p(0, target_y-self:getPositionY()))))

    -- 提供向前的速度
    local x_speed = self.map.current_section_group_config.move_speed + self.config.dash_speed*3
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(x_speed)
        move_component:ChgYSpeed(0)
    end

    -- 改为滞空模式且不受重力影响
    self.ignore_gravity = true
    self.ignore_ground = true

    -- 冲刺音效
    self.super_dash_sound_handle = dxm.audio:playSound(assets.sound.config.sound.game_player_super_dash, true)

    -- 播放动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.super_dash)

    -- 距离加成
    distance = distance + (self.super_dash_distance_addition * game.CAssetGame.m_to_pixel)

    -- 创建一个动画
    self.super_dash_animation = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(game.CAssetGame.super_dash_animation..".json",
    game.CAssetGame.super_dash_animation..".atlas", true)
    if self.super_dash_animation then
        self.super_dash_animation:setAnimation(0, "division", true)
        self:addChild(self.super_dash_animation)
    end

    -- 突刺N秒后离开突刺
    self.total_super_dash_time = distance / x_speed
    self.super_dash_time = self.total_super_dash_time
    self.super_dash_need_drop_protect = need_drop_protect

    -- 创建加速线
    self.animation_speed_line = sSpineUtils:CreateNodeAnimation("spine/speed_line", true)
    if self.animation_speed_line then
        self.animation_speed_line:setAnimation(0, "animation", true)

        self.animation_speed_line:setPosition(dxm.VisibleRect:center())
        self.map:addChild(self.animation_speed_line)
    end

    -- 创建倒计时
    -- self.super_dash_circle = cc.Sprite:create("images/circle.png")
    if self.super_dash_circle then
        self:addChild(self.super_dash_circle)
        self.super_dash_circle:setVisible(self.super_dash_time <= 1)
    end
end

-- 跳跃操作
function CGamePlayer:Jump()

    -- 有禁止移动标记
    if self:GetFlag(game.CAssetGameFlag.cannot_jump) > 0 then
        return false
    end

    -- 判断当前能否跳跃
    if self.current_jump_times >= self.config.max_jump_times and self.map.ignore_jump_times ~= true then
        return false
    end

    -- 判断是否存在移动组件
    local move_component = self.component_list.move
    if not move_component then
        return false
    end

    -- 跳跃音效
    dxm.audio:playSound(assets.sound.config.sound.game_player_jump)

    -- 增加一次跳跃
    self.current_jump_times = self.current_jump_times + 1

    -- 忽略加速度环节，直接给一个空中速度
    local jump_speed = 0
    if self.current_jump_times == 1 then
        jump_speed = self.config.first_jump_speed
        self.total_jump_times = self.total_jump_times + 1
    elseif self.current_jump_times == 2 then
        jump_speed = self.config.second_jump_speed
        self.total_jump_twice_times = self.total_jump_twice_times + 1
    end

    -- 速度生效
    move_component:ChgYSpeed(jump_speed)

    -- 不同的角色加速状态产生不同的横向速度
    if self.current_move_status == CGamePlayer.MoveStatus.accelerate then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed + self.config.accelerate_speed)
    elseif self.current_move_status == CGamePlayer.MoveStatus.decelerate then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed + self.config.decelerate_speed)
    else
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
    end

    -- 播放跳跃动画
    if self.current_jump_times <= 1 then
        self:PlayAnimation(CGamePlayer.BaseAnimationStatus.jump)
    else
        self:PlayAnimation(CGamePlayer.BaseAnimationStatus.jump2)
    end

    -- 引导监听
    sCtrlGuide:Listen("jump")

    return true
end

-- 使用武器操作
function CGamePlayer:UseWeapon()

    -- 有禁止使用武器标记
    if self:GetFlag(game.CAssetGameFlag.cannnot_use_weapon) > 0 then
        return false
    end

    -- 判断当前是否存在武器
    if self.weapon_id == nil or self.bullet_count <= 0 then
        return false
    end

    -- 消耗子弹数
    self.bullet_count = self.bullet_count - 1

    -- 扔东西
    self:Throw(self.weapon_id)

    -- 子弹耗尽
    if self.bullet_count <= 0 then
        self.weapon_id = nil

        if self.weapon_changed_event then
            self.weapon_changed_event:Fire(self.weapon_id, self.bullet_count)
        end
    end
    return true
end

function CGamePlayer:_CancelMoveAccelerate()

    -- 取消加速
    if self.current_move_status ~= CGamePlayer.MoveStatus.accelerate then
        return false
    end

    if self.ground_flag == game.CAssetGameGroundFlag.ice_ground then
        self:_SetMoveStatus(CGamePlayer.MoveStatus.cancel_accelerate)
    else
        -- 设置状态
        self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)

        -- 在空中则无法享受前后操作
        if self.can_move_in_the_air == false and self.ground_id == nil then
            return true
        end

        -- 设置速度
        local move_component = self.component_list.move
        if not move_component then
            return false
        end
        move_component:ChgXSpeed(move_component:GetXSpeed() - self.config.accelerate_speed)
    end
    return true
end

function CGamePlayer:_CancelMoveDecelerate()

    -- 取消减速
    if self.current_move_status ~= CGamePlayer.MoveStatus.decelerate then
        return false
    end

    -- 判断是否存在移动组件
    local move_component = self.component_list.move
    if not move_component then
        return false
    end

    -- 设置状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)

    -- 在空中则无法享受前后操作
    if self.can_move_in_the_air == false and self.ground_id == nil then
        return true
    end

    -- 设置速度
    local move_component = self.component_list.move
    if not move_component then
        return false
    end
    move_component:ChgXSpeed(move_component:GetXSpeed() - self.config.decelerate_speed)

    -- 播放动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.cancel_brake, true)

    return true
end

function CGamePlayer:_CancelDash()

    -- 判断当前是否处于冲刺状态
    if self.current_move_status ~= CGamePlayer.MoveStatus.dash then
        return
    end

    -- 进入普通待机模式
    self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)

    -- 不允许用户做任何操作
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_jump)

    -- 取消攻击能力
    self.can_attack = false

    -- 速度调整回来
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
    end

    -- 调整为受重力影响
    self.ignore_gravity = false
    self.ignore_ground = false

    -- 播放动画
    self:UpdateAnimation()

    -- 结束动画
    if self.dash_action then
        self:stopAction(self.dash_action)
        self.dash_action = nil
    end

    -- 取消加速现
    if self.animation_speed_line then
        self.animation_speed_line:removeFromParent()
        self.animation_speed_line = nil
    end
end

function CGamePlayer:_CancelSuperDash()

    -- 判断当前是否处于冲刺状态
    if self.current_move_status ~= CGamePlayer.MoveStatus.super_dash then
        return
    end

    -- 取消BUFF
    self:RemoveBuff(game.CAssetGameBuff.super_dash)

    -- 速度调整回来
    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
    end

    -- 调整为受重力影响
    self.ignore_gravity = false
    self.ignore_ground = false

    -- 进入普通待机模式
    self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)

    -- 播放动画
    self:UpdateAnimation()

    -- 结束动画
    if self.dash_action then
        self:stopAction(self.dash_action)
        self.dash_action = nil
    end

    -- 取消加速线
    if self.animation_speed_line then
        self.animation_speed_line:removeFromParent()
        self.animation_speed_line = nil
    end

    -- 取消加速光效
    if self.super_dash_animation then
        self.super_dash_animation:removeFromParent()
        self.super_dash_animation = nil
    end

    -- 送冲刺后无敌BUFF
    if not self.will_die then
        self:AddBuff(game.CAssetGameBuff.super_dash_finish_invincible)
    end

    -- 关闭循环音效
    if self.super_dash_sound_handle then
        dxm.audio:stopSound(self.super_dash_sound_handle)
        self.super_dash_sound_handle = nil
    end

    -- 关闭圈
    if self.super_dash_circle then
        self.super_dash_circle:removeFromParent()
        self.super_dash_circle = nil
    end
end

function CGamePlayer:_SetMoveStatus(move_status, exit_callback)

    -- 判断状态是否存在变化
    if self.current_move_status == move_status then
        return
    end

    -- 判断是否有托管离开回调
    if self.current_move_status_exit_callback then

        local current_move_status_exit_callback = self.current_move_status_exit_callback
        self.current_move_status_exit_callback = nil
        current_move_status_exit_callback()
    else
        self.current_move_status = CGamePlayer.MoveStatus.idle
    end

    -- 理论上状态需要在exit_callback中被清理为idle
    if move_status ~= CGamePlayer.MoveStatus.idle then
        self.current_move_status = move_status
        self.current_move_status_exit_callback = exit_callback
    end
end

function CGamePlayer:HPChangedCallback(hp)
    if hp <= 0 then
        self:Die()
    end
end

function CGamePlayer:Hurt(attacker, damage, buff)

    if self.is_dead then
        return
    end

    -- 伤害减免
    if self.reduction_damage > 0 then
        damage = damage - math.ceil(damage * self.reduction_damage)
    end

    -- 非无敌模式才会收到影响
    if not attacker or self:GetFlag(game.CAssetGameFlag.invincible) <= 0 then
        local component_hp = self.component_list.hp
        if component_hp then
            component_hp:ChgHP(-damage)
            if component_hp.hp > 0 then
                self:AddBuff(game.CAssetGameBuff.hit_invincible)
            end
        end
        if buff then
            self:AddBuff(buff)
        end
    end
    self:ConsumeBuff("hit_count", 1)

    -- 受击音效
    dxm.audio:playSound(assets.sound.config.sound.game_player_hurt)
end

function CGamePlayer:AddFlag(flag)
    if self.flag[flag] == nil then
        self.flag[flag] = 1
    else
        self.flag[flag] = self.flag[flag] + 1
    end
end

function CGamePlayer:RemoveFlag(flag)
    self.flag[flag] = self.flag[flag] -1
end

function CGamePlayer:GetFlag(flag)
    if self.flag[flag] == nil then
        return 0
    else
        return self.flag[flag]
    end
end

function CGamePlayer:AddBuff(buff_entry)

    local buff = self.buff[buff_entry]
    if buff == nil then

        -- 构建新buff
        buff = { consume = { } }

        -- 添加flag
        if buff_entry.flag ~= nil then
            for i = 1, #buff_entry.flag do
                self:AddFlag(buff_entry.flag[i])
            end
        end

        -- 变更一下外形
        if buff_entry.skin ~= nil and buff_entry.skin.add ~= nil then
            buff.skin = buff_entry.skin.add(self.animation)
        end

        -- 添加 BUFF效果
        if buff_entry.add_effect then
            buff.effect = buff_entry.add_effect(self)
        end

        -- 添加进去
        self.buff[buff_entry] = buff

        -- BUFF变更事件
        if self.buff_changed_event then
            self.buff_changed_event:Fire()
        end
    end

    -- 根据消耗类型来填充buff
    if buff_entry.consume ~= nil then

        local dt_times = 1
        if self.magnet_dt_times > 1 and buff_entry.id == "item_magnet" then
            dt_times = self.magnet_dt_times
        end

        for consume_type, consume_value in pairs(buff_entry.consume) do

            if consume_type == "dt" then
                consume_value = consume_value * dt_times
            end
            buff.consume[consume_type] = consume_value
        end
    end
end

function CGamePlayer:RemoveBuff(buff_entry)

    -- 获取buff
    local buff = self.buff[buff_entry]
    if buff == nil then
        return
    end

    -- 移除flag
    if buff_entry.flag ~= nil then
        for i = 1, #buff_entry.flag do
            self:RemoveFlag(buff_entry.flag[i])
        end
    end

    -- 变更一下外形
    if buff_entry.skin ~= nil and buff_entry.skin.remove ~= nil then
        buff_entry.skin.remove(self.animation, buff.skin)
    end

    -- 移除BUFF效果
    if buff_entry.remove_effect then
        buff_entry.remove_effect(self, buff.effect)
    end

    -- 移除buff
    self.buff[buff_entry] = nil

    -- BUFF变更事件
    if self.buff_changed_event then
        self.buff_changed_event:Fire()
end
end

function CGamePlayer:ConsumeBuff(consume_type, consume_value)

    -- 遍历当前所有的BUFF
    for buff_entry, buff in pairs(self.buff) do

        if buff.consume[consume_type] ~= nil then

            -- 消耗消费计数
            buff.consume[consume_type] = buff.consume[consume_type] - consume_value
            if buff.consume[consume_type] <= 0 then
                self:RemoveBuff(buff_entry)
            end
        end
    end
end

function CGamePlayer:ClearAllBuff()

    for buff_entry, _ in pairs(self.buff) do
        self:RemoveBuff(buff_entry)
    end
end

function CGamePlayer:Die()

    -- 判断是否已经死亡
    if self.is_dead then
        return
    end

    -- 确保耗尽体力
    self.component_list.hp:ChgHP(-self.component_list.hp.hp)

    -- 更新BUFF
    self:ConsumeBuff("die_count", 1)

    -- 判断当前是否无敌
    if self:GetFlag(game.CAssetGameFlag.never_die) > 0 then
        
        -- 待死标记打打好
        self.will_die = true
        return
    end

    -- 切换为待机状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.die, function() self:_CancelDie() end)

    -- 设置死亡标记
    self.is_dead = true

    -- 前进速度为0
    local component_move = self.component_list.move
    if component_move then
        component_move:ChgXSpeed(0)
        component_move:ChgYSpeed(game.CAssetGame.die_jump_speed)
    end

    -- 无视地表
    self.ignore_ground = true
    self.ignore_ground_x = true

    -- 播放死亡动画
    self:PlayAnimation(CGamePlayer.BaseAnimationStatus.death, false)

    -- 不允许用户做任何操作
    self:AddFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:AddFlag(game.CAssetGameFlag.cannot_jump)
    self:AddFlag(game.CAssetGameFlag.cannot_move_dash)

    -- 死亡音效
    dxm.audio:playSound(assets.sound.config.sound.game_player_die)

    -- 播放2秒死亡动画后结束游戏
    self:runAction(cc.Sequence:create(cc.DelayTime:create(2), cc.CallFunc:create(function() 

        -- 通知游戏地图游戏结束
        self.map:GameOver(false)
    end)))
end

function CGamePlayer:_CancelDie()

    -- 调整成未死亡状态
    self.is_dead = false

    -- 允许用户做任何操作
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_accelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_decelerate)
    self:RemoveFlag(game.CAssetGameFlag.cannot_jump)
    self:RemoveFlag(game.CAssetGameFlag.cannot_move_dash)

    -- 恢复正常重力
    self.ignore_ground = false
    self.ignore_ground_x = false

    -- 冲刺一枚
    self:setPositionY(game.CAssetGame.drop_height)
end

function CGamePlayer:Throw(weapon_id)

    -- 获取武器配置
    local weapon_config = game.CAssetGameWeapon[self.weapon_id]

    -- 扔出武器
    local game_object = self.map.tmx:CreateObjectByID(weapon_config.object_id, self:getPositionX(), self:getPositionY() + 100)
    if game_object then

        local component_move = game_object.component_list.move
        if component_move then
            component_move.x_speed = self.component_list.move.x_speed + weapon_config.x_speed
            component_move.y_speed = weapon_config.y_speed
        end
        game_object.ignore_gravity = weapon_config.ignore_gravity
        game_object.damage = weapon_config.damage + self.addition_damage
        game_object.attack_group = 0

        -- 抛射音效
        dxm.audio:playSound(assets.sound.config.sound.game_player_throw)
    end

    -- 播放扔的动画
    if self.animation_throw then
        self.animation_throw:setAnimation(0, "animation", false)
    end
end

function CGamePlayer:PlayRunSound()

    local run_sound_name = self:GetRunSoundFile()
    if run_sound_name ~= self.run_sound_name then
        self.run_sound_name = run_sound_name

        if self.run_sound_handle then
            dxm.audio:stopSound(self.run_sound_handle)
            self.run_sound_handle = nil
        end

        if self.run_sound_name then
            self.run_sound_handle = dxm.audio:playSound(self.run_sound_name, true)
        end
    end
end

function CGamePlayer:GetRunSoundFile()

    local sound_file = nil

    -- 获取当前所踩的地表
    if self.ground_id ~= nil and self.current_move_status ~= CGamePlayer.MoveStatus.stand then

        local ground_config = game.CAssetGameGround[self.ground_id]
        if ground_config then
            sound_file = ground_config.run_sound
        end

        -- 如果没有指定音效，则使用默认的
        if sound_file == nil then
            sound_file = assets.sound.config.sound.game_player_run_default
        end
    end
    return sound_file
end

function CGamePlayer:UpdateAnimation()

    -- 获取是否在空中的状态
    local in_air = false
    local move_component = self.component_list.move
    if move_component then
        in_air = (move_component.y_status == game.CObjectComponentMove.YStatus.air)
    end

    -- 重新播放声音
    self:PlayRunSound()

    -- 根据角色的状态来更新一个动画
    if not in_air then

        if self.current_move_status == CGamePlayer.MoveStatus.idle then
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.run, true)
            return
        elseif self.current_move_status == CGamePlayer.MoveStatus.accelerate then
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.run, true)
            return
        elseif self.current_move_status == CGamePlayer.MoveStatus.decelerate then
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.brake, true)
            return
        elseif self.current_move_status == CGamePlayer.MoveStatus.cancel_decelerate then
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.cancel_brake, true)
            self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)
            return
        elseif self.current_move_status == CGamePlayer.MoveStatus.stand then
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.stand, true)
            return
        else
            self:PlayAnimation(CGamePlayer.BaseAnimationStatus.run, true)
        end
    end

    -- 设置动画
    self:PlayAnimation(self.base_animation, self.is_base_animation_loop)
end

function CGamePlayer:Pickup(game_object)

    if not game_object then
        return
    end

    -- 将物件从地图上移除
    local world_position = game_object:convertToWorldSpace(cc.p(0,0))
                
    game_object:retain()
    self.map.tmx:RemoveObject(game_object)
    self:addChild(game_object)
    self.waiting_for_pickup_list[game_object] = game_object
    game_object:release()

    local from_pos = self:convertToNodeSpace(world_position)
    game_object:setPosition(from_pos)
    game_object.check_behind = false

    -- 该物件以一定的速度吸收到角色身上
    local body_rect = self:GetBodyRect()
    local body_pos = cc.p(body_rect.x+body_rect.width/2, body_rect.y+body_rect.height/2)
    local dt = cc.pGetDistance(from_pos, body_pos) / 2000
    game_object:runAction(cc.Sequence:create(cc.MoveTo:create(dt, body_pos), 
    cc.CallFunc:create(function() self:_Pickup(game_object) end)))
end

function CGamePlayer:_Pickup(game_object)

    CGamePlayer.super.Pickup(self, game_object)

    -- 拾取音效
    if game_object.config.pickup_sound then
        dxm.audio:playSound(game_object.config.pickup_sound)
    end

    -- 统计捡取数量
    local game_object_id = game_object.config.id
    self.pickup_list[game_object_id] = self.pickup_list[game_object_id] or 0
    self.pickup_list[game_object_id] = self.pickup_list[game_object_id] + 1
end

function CGamePlayer:CheckBehind()
    return false
end

function CGamePlayer:YStatusChangedCallback(y_status)

    -- 着地处理
    if y_status == game.CObjectComponentMove.YStatus.ground then

        -- 重置多段跳次数
        self.current_jump_times = 0

        -- 重置冲刺次数
        self.current_dash_times = 0

        -- 非死亡状态下可以调整落地速度
        if not self.is_dead then

            -- 判断当前是否为掉落保护状态
            if self.current_move_status == CGamePlayer.MoveStatus.drop_protect then
                self:Idle()
            end

            -- 重置速度
            local move_component = self.component_list.move
            if move_component then
                if self.current_move_status == CGamePlayer.MoveStatus.accelerate then
                    move_component:ChgXSpeed(self.map.current_section_group_config.move_speed + self.config.accelerate_speed)
                elseif self.current_move_status == CGamePlayer.MoveStatus.decelerate then
                    move_component:ChgXSpeed(self.map.current_section_group_config.move_speed + self.config.decelerate_speed)
                elseif self.current_move_status == CGamePlayer.MoveStatus.stand then
                    move_component:ChgXSpeed(0)
                else
                    move_component:ChgXSpeed(self.map.current_section_group_config.move_speed)
                end
            end

            -- 更新动画
            self:UpdateAnimation()

            -- 落地保护中则取消掉
            if self.current_move_status == CGamePlayer.MoveStatus.drop_protect2 then
                self:Idle()
            end
        end
    end    
end

function CGamePlayer:Ride(mount_id)

    -- 判断当前坐骑的编号是否与参数一致
    if self.mount_id == mount_id then
        return
    end
    self.mount_id = mount_id

    -- 从当前的坐骑身上下来
    self:_MountLeave()

    -- 判断坐骑ID是否存在
    if mount_id ~= nil then
        self:_MountEnter(mount_id)
    end

    -- 角色恢复正常的动作
    self:UpdateAnimation()
end

function CGamePlayer:_MountLeave()

    local mount_entry = game.CAssetGameMount[mount_id]
    if not mount_entry then
        return
    end

    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(move_component:GetXSpeed() - mount_entry.move_speed)
    end

    self:addChild(self.animation)

    if self.mount_animation ~= nil then

        -- 将坐骑与角色分离
        self.mount_animation:removeFromParent()
        self.mount_animation = nil
    end
end

function CGamePlayer:_MountEnter(mount_id)

    local mount_entry = game.CAssetGameMount[mount_id]
    if not mount_entry then
        return
    end

    local move_component = self.component_list.move
    if move_component then
        move_component:ChgXSpeed(move_component:GetXSpeed() + mount_entry.move_speed)
    end

    -- 创建坐骑动画
    self.mount_animation = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(mount_entry.file_path..".json", 
    mount_entry.file_path..".atlas", true)
    if self.mount_animation then

        -- 安装更新器
        if self.map.updater then
            self.map.updater:InstallOnNode(self.mount_animation)
        end

        -- 初始化动画
        self.mount_animation:setAnchorPoint(cc.p(0.5,0))
        --self.mount_animation:setFlippedX(true)
        self:addChild(self.mount_animation)

        self.animation:retain()
        self.animation:removeFromParent()
        self.mount_animation:setAttachmentWithNode("czw", self.animation)
        --self.mount_animation:addChild(self.animation)
        self.animation:release()

        --self.animation:setPosition(mount_entry.fix_position)
    end
end

function CGamePlayer:PickupWeapon(weapon_id, bullet_count)

    -- 判断是否有效操作
    if weapon_id == self.weapon_id and weapon_id == nil then
        return
    end

    local is_changed = false

    -- 判断是否为有效武器
    if weapon_id ~= nil then

        -- 获取武器配置
        local weapon_config = game.CAssetGameWeapon[weapon_id]
        if weapon_config == nil then
            return
        end

        -- 新武器需要清空数量
        if weapon_id ~= self.weapon_id then
            self.weapon_id = weapon_id
            self.bullet_count = 0
            is_changed = true
        end

        -- 增加子弹数量，需要做一个上限
        local new_bullet_count = self.bullet_count + bullet_count
        if new_bullet_count > weapon_config.max_bullet_count then
            new_bullet_count = weapon_config.max_bullet_count
        end
        if self.bullet_count ~= new_bullet_count then
            self.bullet_count = new_bullet_count
            is_changed = true
        end
    else
        self.weapon_id = nil
        self.bullet_count = 0
        is_changed = true
    end

    -- 触发对应的事件
    if is_changed and self.weapon_changed_event then
        self.weapon_changed_event:Fire(self.weapon_id, self.bullet_count)
    end

    -- TODO: 默认使用道具
    self:UseWeapon()
end

function CGamePlayer:PlayAnimation(base_animation, is_loop)

    self.base_animation = base_animation
    self.is_base_animation_loop = is_loop or false

    local real_animation_name = self:GetRealAnimation(base_animation)
    if self.animation then
        self.animation:setAnimation(0, real_animation_name, is_loop)
    end
    if self.mount_animation then
        self.mount_animation:setAnimation(0, base_animation, is_loop)
    end
end

function CGamePlayer:GetRealAnimation(base_animation)

    -- 获得真实动画
    local real_animation = base_animation
    if base_animation == CGamePlayer.BaseAnimationStatus.death then
        real_animation = CGamePlayer.BaseAnimationStatus.death
    elseif self.mount_animation then
        real_animation = CGamePlayer.BaseAnimationStatus.ride
    elseif self.mount_id and self.mount_id ~= 0 then
        real_animation = self.mount_id .. "_" .. base_animation
    else

        -- 部分动画处理成同样的动画名称
        if real_animation == CGamePlayer.BaseAnimationStatus.brake or
            real_animation == CGamePlayer.BaseAnimationStatus.cancel_brake or
            real_animation == CGamePlayer.BaseAnimationStatus.brake_run then
            real_animation = CGamePlayer.BaseAnimationStatus.run
        end
    end
    return real_animation
end

function CGamePlayer:CalcBodyRect()

    local rect = nil
    if game.CAssetGame.player_rect then
        rect = clone(game.CAssetGame.player_rect)
        rect.x = rect.x * self:getScaleX()
        rect.y = rect.y * self:getScaleY()
        rect.width = rect.width * self:getScaleX()
        rect.height = rect.height * self:getScaleY()
    end
    return rect
end

function CGamePlayer:Revive(is_recover)

    -- 设置状态
    self:_SetMoveStatus(CGamePlayer.MoveStatus.idle)

    -- 判断血量是否恢复
    if is_recover then

        -- 增加一半血量
        self.component_list.hp:ChgHP(self.component_list.hp.max_hp / 2)

        -- 清空将死状态
        self.will_die = false

        -- 进行复活
        self:DropProtect()
    else

        -- 冲刺完就立即死亡
        self.will_die = true

        -- 冲刺一段
        self:SuperDash(game.CAssetGame.die_super_dash_distance)
    end
end

function CGamePlayer:ShowSuperDashCircle()

    if self.super_dash_circle then
            
        if self.super_dash_time > 1 then
            self.super_dash_circle:setVisible(false)
        else
            self.super_dash_circle:setVisible(true)
            self.super_dash_circle:setScale(self.super_dash_time)
        end
    end
end

function CGamePlayer:PlayGoldDivisionAnimation()

    self.animation_gold_count = self.animation_gold_count or 0
    if self.animation_gold_count >= 5 then
        return
    end

    local animation_gold_name = "spine/gold/skeleton"
    local animation_gold = spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(animation_gold_name..".json", 
    animation_gold_name..".atlas", 
    true)
    if animation_gold then
        local body_rect = self:GetBodyRect()
        animation_gold:setPosition(cc.p(body_rect.x+body_rect.width/2, body_rect.y+body_rect.height/2))
        self:addChild(animation_gold)
        self.animation_gold_count = self.animation_gold_count + 1

        animation_gold:setAnimation(0, "division", false)
        animation_gold:setCompleteListener(function() 
            self.animation_gold_count = self.animation_gold_count - 1
            sActionUtils:Delay(0.1, nil, animation_gold)
            animation_gold:removeFromParent() 
        end)
    end
end

return game.CGamePlayer

-- endregion
