-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CGameTMX = class("game.CGameTMX", cc.Node)
local CGameTMX = game.CGameTMX

CGameTMX.ZOrder =
{
    surface = - 100,
    object = 1,
    gournd = 2,

    max = 4
}

CGameTMX.collide_interval_update_times = 3

function CGameTMX:ctor()
end

function CGameTMX:Init(map, tmx_file)

    self.tmx_config = require("app.views.game.tmx_cache"):Add(tmx_file)
    if self.tmx_config == nil then
        return false
    end
    self.tmx_start_x = 0

    -- 地表宽度，默认为无尽
    self.width = nil

    -- 设置地图
    self.map = map

    -- 设置最近一次更新的位置
    self.last_start_x = 0
    self.last_end_x = 0
    self.ground = {}
    self.ground_height = {}
    self.static_objects = {}
    self.dynamic_objects = {}
    self.visible_rect = {}

    -- 掉落地表
    self.drop_ground = { }

    -- 平台
    self.platform = { }

    -- 判断摄像机是否准备减速
    self.is_camera_slow = false

    -- 监听进入离开事件
    self:registerScriptHandler( function(event)
        if event == "enter" then
        elseif event == "exit" then
        elseif event == "cleanup" then
            self:Dispose()
        end
    end )

    -- 创建事件
    self.camera_position_changed_event = dxm.CEvent.new()

    -- 更新次数
    self.update_dt = 0
    self.game_object_collide = {}

    return true
end

function CGameTMX:Dispose()
end

function CGameTMX:Create(...)

    local tmx = CGameTMX.new()
    if tmx == nil or tmx:Init(...) == false then
        return nil
    end
    return tmx
end

function CGameTMX:UpdateCamera(dt)

    -- 角色不允许摄像机跟踪
    if not self.map.player.update_camera then
        return
    end

    -- 判断角色当前的动作，如果是加速中，则默认摄像机速度不变，如果是非加速，则默认摄像机速度加快
    local camera_move_speed = self.map.current_section_group_config.move_speed

    -- 计算本次的角色移动量
    local move_distance = { w = camera_move_speed * dt, h = 0 }

    -- 更新摄像机区域（跟踪角色）
    local fx, fy = self.map.player:getPositionX() + self:getPositionX(), self.map.player:getPositionY() + self:getPositionY()

    -- 不允许角色跑出最远端
    if fx > self.map.camera_rect.x + self.map.camera_rect.width then
        move_distance.w = move_distance.w + (fx - (self.map.camera_rect.x + self.map.camera_rect.width))
        self.is_camera_slow = false
    -- 角色在标准摄像机之前一段距离，则需要摄像机加速拉回
    elseif fx > self.map.camera_rect.x + 10 then
        move_distance.w = move_distance.w + camera_move_speed * 0.3 * dt
        self.is_camera_slow = false
    -- 在摄像机标准区域内不需要做任何速度调整
    elseif fx >= self.map.camera_rect.x and fx <= self.map.camera_rect.x + 10 then
        self.is_camera_slow = false
    -- 在摄像机的减速带区域内则需要摄像机减速拉回
    elseif fx > 0 and fx < self.map.camera_rect.x then

        if self.is_camera_slow then
            move_distance.w = move_distance.w - camera_move_speed * 0.7 * dt
        else
            self.is_camera_slow = true
            move_distance.w = move_distance.w + (fx - self.map.camera_rect.x)
        end
    -- 角色被拉到屏幕最后方则死亡
    elseif fx < 0 then
        self.map.player:Die()
    end 

    -- 高度调整
    if fy > self.map.camera_rect.y + self.map.camera_rect.height then
        move_distance.h = move_distance.h +(fy -(self.map.camera_rect.y + self.map.camera_rect.height))
    elseif fy < self.map.camera_rect.y then
        move_distance.h = move_distance.h +(fy - self.map.camera_rect.y)
    end

    -- 进行摄像机的便捷判断
    local new_x = self:getPositionX() - move_distance.w
    if new_x > 0 then
        new_x = 0
    end
    if self.width and new_x < -self.width+self.map.visible_size.width then
        new_x = -self.width+self.map.visible_size.width
    end
    local new_y = self:getPositionY() - move_distance.h
    if new_y > game.CAssetGame.min_height then
        new_y = game.CAssetGame.min_height
    end
    self:setPosition(cc.p(new_x, new_y))

    -- 触发事件
    self.camera_position_changed_event:Fire()
end

function CGameTMX:Update(dt)

    -- sProfileSystem:StartSampling("CGameCamera:Update1")

    -- 游戏如果结束则不再移动摄像机
    if self.map.is_gameover == false and self.map.player and not self.map.player.is_dead then
        self:UpdateCamera(dt)
    end

    -- sProfileSystem:StopSampling("CGameCamera:Update1")

    -- 计算最新的显示区域
    local visible_rect = self.visible_rect
    visible_rect.x = - self:getPositionX()
    visible_rect.y = - self:getPositionY()
    visible_rect.w = self.map.visible_size.width
    visible_rect.h = self.map.visible_size.height

    -- 更新图片的显示隐藏
    -- sProfileSystem:StartSampling("CGameVisible:Update1")
    self:UpdateVisible()
    -- sProfileSystem:StopSampling("CGameVisible:Update1")

    -- sProfileSystem:StartSampling("CGameGround:Update1")

    -- 更新地表
    for x, grounds in pairs(self.drop_ground) do
        for y, ground in pairs(grounds) do
            if ground:Update(dt) then
                ground:removeFromParent()
                self.drop_ground[x][y] = nil
            end
        end
    end

    --sProfileSystem:StopSampling("CGameGround:Update1")

    -- 遍历所有的物件一次
    -- sProfileSystem:StartSampling("CGameObj:Update1")
    for key, object in pairs(self.static_objects) do
        if object:Update(dt) then
            self:RemoveObject(object)
        end
    end
    for key, object in pairs(self.dynamic_objects) do
        if object:Update(dt) then
            self:RemoveObject(object)
        else
            object.check_collide = false
        end
    end
    -- sProfileSystem:StopSampling("CGameObj:Update1")

    -- 尝试进行碰撞检测
    -- sProfileSystem:StartSampling("CGameCol:Update1")
    for _,game_object_a in pairs(self.dynamic_objects) do

        local collide_objects = {}

        for _,game_object_b in pairs(self.static_objects) do
            if self:CollideGameObject(game_object_a, game_object_b) then
                collide_objects[game_object_b] = game_object_b
            end
        end

        for _,game_object_b in pairs(self.dynamic_objects) do
            if not game_object_b.check_collide and game_object_b ~= game_object_a and self:CollideGameObject(game_object_a, game_object_b) then
                collide_objects[game_object_b] = game_object_b
            end
        end

        game_object_a.collide_objects = collide_objects
        game_object_a.check_collide = true
    end
    -- sProfileSystem:StopSampling("CGameCol:Update1")
end

function CGameTMX:CollideGameObject(game_object_a, game_object_b)

    local is_collided = false

    -- 判断是否有碰撞的必要
    if game_object_a:CanCollide(game_object_b) or game_object_b:CanCollide(game_object_a) then

        -- 进行碰撞计算
        local collide_rect_a = game_object_a:GetCollideRect()
        if collide_rect_a then

            local collide_rect_b = game_object_b:GetCollideRect()
            if collide_rect_b then
                is_collided = cc.rectIntersectsRect(collide_rect_a, collide_rect_b)
            end
        end
    end

    -- 产生碰撞后需要进行逻辑处理
    if is_collided then

        -- 判断上次是否也有碰撞
        local run_collide = true
        if (game_object_a.collide_objects and game_object_a.collide_objects[game_object_b]) or 
        (game_object_b.collide_objects and game_object_b.collide_objects[game_object_a]) then
            run_collide = false
        end

        if run_collide then

            -- 碰撞优先级高的可以优先调用撞击
            if game_object_a.collide_level >= game_object_b.collide_level then
                game_object_a:OnCollide(game_object_b)
                game_object_b:OnCollide(game_object_a)
            else
                game_object_b:OnCollide(game_object_a)
                game_object_a:OnCollide(game_object_b)
            end
        end
    end
    return is_collided
end

function CGameTMX:ClearGround(last_start_x, start_x)

    if self.tmx_config == nil then
        return
    end

    -- 最多保留5块地表，放置物件提前消失
    for i = last_start_x, start_x - 1 do

        if self.ground[i] then

            for j,ground_tile in pairs(self.ground[i]) do
                if ground_tile then
                    ground_tile:removeFromParent()
                    self.ground[i][j] = nil
                end
            end
            self.ground[i] = nil
            self.ground_height[i] = nil
        end
    end
end

function CGameTMX:CreateGround(x)

    if self.tmx_config == nil then
        return
    end

    self.ground[x] = { }
    self.ground_height[x] = 0

    -- 获取当前配置的所在列编号
    local tmx_config = self.tmx_config
    local tmx_x = x - self.tmx_start_x

    -- 获取用于横向检测的下一个地形配置，默认就是本地形配置的下一列
    local next_tmx_config = self.tmx_config
    local next_tmx_x = tmx_x + 1

    -- 下一列可能会更改为新的配置
    if next_tmx_x > self.tmx_config.width then

        -- 新的地表配置
        self.tmx_start_x = x
        self.tmx_config = self.map:GetNextTMXConfig()

        -- 设置下一段
        next_tmx_config = self.tmx_config
        next_tmx_x = 1

        -- 如果没有下一段则表示地图已经到达终点
        if self.tmx_config == nil then
            self.width = x*game.CAssetGame.tile_width
        end
    end

    -- 开始绘制该列地表
    if tmx_config.ground[tmx_x] then

        for i,ground_tile_name in pairs(tmx_config.ground[tmx_x]) do

            -- 获取对应的地表配置
            local ground_tile_config = game.CAssetGameGround[ground_tile_name]

            -- 根据前面的地形和后面的地形来确定该显示拼接地表的哪一部分
            local index = nil
            local pre_x = x - 1
            local pre_same = (self.ground[pre_x] and self.ground[pre_x][i] and self.ground[pre_x][i].config.id == ground_tile_name)
            local next_same = (next_tmx_config and
                    next_tmx_config.ground[next_tmx_x] and
                    next_tmx_config.ground[next_tmx_x][i] and
                    next_tmx_config.ground[next_tmx_x][i] == ground_tile_name)
            if pre_same then
                    
                -- 判断后面的地形是否一致来决定自己在中间还是尾部
                if next_same then

                    index = self.ground[pre_x][i].index + 1
                    if index >= #ground_tile_config.file_path.group then
                        index = 2
                    end
                else
                    index = #ground_tile_config.file_path.group
                end
            else
                    
                -- 判断后面的地形是否一致来决定自己是否为独立一块
                if next_same then
                    index = 1
                else
                    index = 0
                end
            end

            -- 获取地表配置
            local ground_config = game.CAssetGameGround[ground_tile_name]
            if ground_config then

                local class_name = ground_config.class_name or "game_ground"
       
                -- 根据绘制编号来创建地形图片
                local game_ground = require("app.views.game." .. class_name):Create(self.map, ground_config, index, x, i)
                if game_ground then
                    game_ground:setPosition(cc.p((x - 1) * game.CAssetGame.tile_width, (i - 1) * game.CAssetGame.tile_height))
                    self:addChild(game_ground)
                    self.ground[x][i] = game_ground
                    game_ground:setLocalZOrder(-100)
                    self.ground_height[x] = math.max(self.ground_height[x], i)
                end
            end
        end
    end
end

function CGameTMX:CreateObject(x)

    if self.tmx_config == nil then
        return
    end

    -- 获取当前配置的所在列编号
    local tmx_x = x - self.tmx_start_x

    -- 触发新增物件
    if self.tmx_config.object[tmx_x] then
        
        for _,objects in pairs(self.tmx_config.object[tmx_x]) do
            if objects then
                for j=1,#objects do
                    local object = objects[j]
                    if object then
                        self:CreateObjectByID(object.name, self.tmx_start_x*game.CAssetGame.tile_width + object.x, object.y)
                    end
                end
            end
        end
    end
end

function CGameTMX:UpdateVisible()

    if self.tmx_config then

        local remain_count = 5
        local start_x = math.floor(self.visible_rect.x / game.CAssetGame.tile_width) + 1
        local end_x = math.floor((self.visible_rect.x + self.visible_rect.w) / game.CAssetGame.tile_width) + 1 + remain_count

        -- 移除上一次的地表内容
        self:ClearGround(self.last_start_x, start_x)

        -- 新增本次的地表内容和物件内容
        for i = self.last_end_x + 1, end_x do

            -- 创建新的物件
            self:CreateObject(i)

            -- 创建新的地表（必须最后一个创建）
            self:CreateGround(i)
        end

        -- 更新坐标
        self.last_start_x = start_x
        self.last_end_x = end_x
    end
end

function CGameTMX:GetGroundHeight(y)
    return y * game.CAssetGame.tile_height
end

function CGameTMX:CreateObjectByID(object_id, x, y, ...)

    -- 获取物件配置
    local object_config = game.CAssetGameObject[object_id]
    if object_config == nil then
        return nil
    end

    -- 创建对应的物件
    local class_name = object_config.class_name or "game_object"
    local game_object = require("app.views.game.game_object." .. class_name):Create(self.map, object_config, ...)
    if game_object then
        self:AddObject(game_object, x, y, game_object.zorder)
    end
    return game_object
end

function CGameTMX:AddObject(game_object, x, y, z)

    if game_object == nil then
        return
    end

    -- 设置坐标
    game_object:setPosition(cc.p(x, y))

    -- 更新下层级
    -- game_object:UpdateZOrder()

    -- 添加到场景中
    self:addChild(game_object, z or 0)
    -- 根据是否静态物件来分类存储
    if game_object.is_static then
        self.static_objects[game_object] = game_object
    else
        self.dynamic_objects[game_object] = game_object
    end

    -- 触发地图的事件
    self.map:OnObjectEnter(game_object)
end

function CGameTMX:RemoveObject(game_object)

    if game_object == nil then
        return
    end

    -- 触发地图的事件
    self.map:OnObjectExit(game_object)

    -- 不同类型从不同的表单中删除
    if game_object.is_static then
        self.static_objects[game_object] = nil
    else
        self.dynamic_objects[game_object] = nil
    end

    -- 从场景中移除
    game_object:removeFromParent()
end

function CGameTMX:AddPlatform(platform)
    self.platform[platform] = platform
end

function CGameTMX:RemovePlatform(platform)
    self.platform[platform] = nil
end

return game.CGameTMX

-- endregion
