--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

game.CGameUI = class("game.CGameUI", dxm.CBackgroundUI)

local CGameUI = game.CGameUI

function CGameUI:ctor()
    CGameUI.super.ctor(self)

    -- 当前关卡信息
    self.stage_type = nil
    self.stage_id = nil
    self.child_stage_id = nil

    -- 地图元素
    self.map = nil

    -- windows键盘监听
    self.keyboard_listener = nil

    -- 是否暂停
    self.is_pause = false

    -- 平滑进度条
    self.smooth_hp_percent = game.CSmoothNumber:Create(100)
    self.smooth_fury_percent = game.CSmoothNumber:Create(100)
end

function CGameUI:Init(stage_type, stage_id, child_stage_id)
    self.stage_type = stage_type
    self.stage_id = stage_id
    self.child_stage_id = child_stage_id
    return self:InitWithFile("csd/Game/Game")
end

function CGameUI:Create(...)

    local ui = CGameUI.new()
    if ui == nil or ui:Init(...) == false then
        return nil
    end
    return ui
end

CGameUI.global_ui = false

function CGameUI:Show(stage_type, stage_id, child_stage_id)

    -- 判断该界面是否开启
    if CGameUI.global_ui then

        if stage_type ~= CGameUI.global_ui.stage_type or 
        stage_id ~= CGameUI.global_ui.stage_id or
        child_stage_id ~= CGameUI.global_ui.child_stage_id or
        CGameUI.global_ui.is_started then
            CGameUI.global_ui:SetMap(stage_type, stage_id, child_stage_id)
        end
        return CGameUI.global_ui
    else
        return sUI:Open("app.views.game.game_ui", stage_type, stage_id, child_stage_id)
    end
end

function CGameUI:onEnter()

    CGameUI.global_ui = self

    -- 冲刺按钮
    self:AddTouchEvent( function(is_press)

        if is_press and self.map then
            sCtrlGuide:Listen("press_dash")
            if not self.map.is_pause and not self.map.is_gameover then
                self.map:PushCommand(game.GameMapCommand.dash)
            end
        end
    end , "PanelButtonRightTouch")

    -- 跳跃按钮
    self:AddTouchEvent( function(is_press)

        if is_press and self.map then
            sCtrlGuide:Listen("press_jump")
            if not self.map.is_pause and not self.map.is_gameover then
                self.map:PushCommand(game.GameMapCommand.jump)
            end
        end
    end , "PanelButtonATouch")

    -- 暂停按钮
    self:AddReleaseEvent( function() 
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:Pause()
    end, "ButtonStart")

    -- 隐藏星级目标
    self:SetWidgetVisible(false, "PanelTarget")

    -- 暂时隐藏排行
    self:SetWidgetVisible(false, "PanelRank")
    self:SetWidgetVisible(false, "ImageGameSchedule")

    -- 隐藏警告提示
    self:SetWidgetEnableAndVisible(false, "TextWarning")

    -- 隐藏BOSS血量
    self:SetWidgetVisible(false, "ImageBossHP")

    -- 隐藏文本提示
    local panel_tip = self:GetWidget("PanelBossTip")
    if panel_tip then
        panel_tip:setOpacity(0)
    end

    -- 构建游戏地图
    self:SetMap(self.stage_type, self.stage_id, self.child_stage_id)

    -- 初始化按键
    self:InitKeyboard()

    -- 监听总的更新体系
    self.entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc( function(dt) self:Update(dt) end, 0, false)

    if assets.opened.is_dev then
        self.update_listener = sSchedulerSystem:AddSecondly(function()
           self:SetLabelText(sProfileSystem:GetString(), "TextDebug")
        end)
    end

    -- 自定义退出功能
    sCtrlApplication.quit_callback = function() return self:Pause() end
end

function CGameUI:onExit()

    if self.update_listener then
        self.update_listener:Disconnect()
        self.update_listener = nil
    end

    CGameUI.global_ui = nil

    if self.item_collect_listener then
        for _,item_collect_listener in pairs(self.item_collect_listener) do
            self.map.item_collect_event:Disconnect(item_collect_listener)
        end
        self.item_collect_listener = nil
    end

    if self.map then
        self.map:removeFromParent()
        self.map = nil
    end

    if self.entry then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.entry)
        self.entry = nil
    end

    -- 清空自定义退出功能
    sCtrlApplication.quit_callback = nil
end

function CGameUI:Pause()

    if not self.map or not self.map.is_started then
        return false
    end

    if self.is_pause or self.map.is_gameover then
        return true
    end

    self.is_pause = true
    self.map:Pause()
    dxm.audio:pauseAllSounds()
    dxm.UIHelper:Open("app.views.game.game_pause_ui", function(is_ok)
        if is_ok then
            self.is_pause = false
            self.map:Resume()
            dxm.audio:resumeAllSounds()
        else
            -- 立即失败
            self.map:Quit()
            sPlayer.player_modules.stage:StageLose(self.stage_type, self.stage_id, self.child_stage_id, self.map.record, "manual_quit")

            -- 开启不同的后续界面
            if self.stage_type == "infinite" or self.stage_type == "compete" then
                game.CCtrlBattle:GotoMain()
            else
                sUI:Open("app.views.ui.stage.stage_list_ui", self.stage_type, self.stage_id, self.child_stage_id)
            end
        end
    end)
    return true
end

function CGameUI:InitKeyboard()

    self.keyboard_listener = cc.EventListenerKeyboard:create()
    if not self.keyboard_listener then
        return
    end

    local keyborad_pressed_func = function(keycode)

        local command = nil

        if keycode == cc.KeyCode.KEY_RIGHT_ARROW then
            command = game.GameMapCommand.dash
        elseif keycode == cc.KeyCode.KEY_UP_ARROW then
            command = game.GameMapCommand.jump
        elseif keycode == cc.KeyCode.KEY_DOWN_ARROW then
            self.map.player:SuperDash(1000)
        end

        if command and self.map and not self.map.is_pause and not self.map.is_gameover then
            self.map:PushCommand(command)
        end
    end
    self.keyboard_listener:registerScriptHandler(keyborad_pressed_func, cc.Handler.EVENT_KEYBOARD_PRESSED)

    local keyborad_released_func = function(keycode)

        local command = nil

        -- TODO:
        --[[
        if keycode == cc.KeyCode.KEY_ENTER then

            sProfileSystem:StartSampling("CGamePlayer:Test")
            local file_path = "spine/gold/skeleton"
            for i=1,30 do
                spine.CSpineHelper:GetInstancePtr():CreateNodeSkeletonAnimation(file_path .. ".json", file_path .. ".atlas", true)
            end
            sProfileSystem:StopSampling("CGamePlayer:Test")
        end
        ]]--

        if command and self.map then
            self.map:PushCommand(command)
        end
    end
    self.keyboard_listener:registerScriptHandler(keyborad_released_func, cc.Handler.EVENT_KEYBOARD_RELEASED)

    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.keyboard_listener, self.widget)
end

function CGameUI:Update(dt)

    --sProfileSystem:StartSampling("CGameUI:Update")

    self:UpdateHP()
    self:UpdateFury()
    self:UpdateTime()
    self:UpdateScore()
    self:UpdateMeter()
    self:UpdateBOSSHP()

    self:UpdateLoadingbarHP(dt)
    self:UpdateLoadingbarFury(dt)

    --sProfileSystem:StopSampling("CGameUI:Update")
end

function CGameUI:UpdateHP()
    local component_hp = self.map.player.component_list.hp
    self.smooth_hp_percent:SetNumber(component_hp.hp / component_hp.max_hp * 100)
    self:SetLabelText(math.floor(component_hp.hp).."/"..math.floor(component_hp.max_hp), "TextHP")
end

function CGameUI:UpdateLoadingbarHP(dt)
    self.smooth_hp_percent:Update(dt)
    self:SetLoadingbarPercent(string.format("%.2f", self.smooth_hp_percent.current_number), "LoadingBarHp")
end

function CGameUI:UpdateFury()

    local player = self.map.player
    if player.current_move_status == game.CGamePlayer.MoveStatus.super_dash then
        local percent = player.super_dash_time/player.total_super_dash_time*100
        self.smooth_fury_percent:SetNumber(percent, true)
        self:SetLabelText(string.format("%.0f", percent).."%", "TextNQ")
    else

        -- 更新当前怒气的数值
        local percent = self.map.fury / self.map.fury_condition * 100
        self.smooth_fury_percent:SetNumber(percent)
        self:SetLabelText(string.format("%.0f", percent).."%", "TextNQ")
    end
end

function CGameUI:UpdateLoadingbarFury(dt)
    self.smooth_fury_percent:Update(dt)
    self:SetLoadingbarPercent(self.smooth_fury_percent.current_number, "LoadingNQ")
end

function CGameUI:UpdateTime()

    local s = math.floor(self.map.total_dt)

    local h = math.floor(s / 3600)
    s = s - h*3600

    local m = math.floor(s / 60)
    s = s - m*60

    local str = string.format("%02d:%02d:%02d", h, m, s)
    self:SetLabelText(str, "TextTime")
end

function CGameUI:UpdateScore()
    self:SetLabelText(tostring(math.floor(self.map.score)), "TextFraction")
end

function CGameUI:UpdateMeter()
    self:SetLabelText(tostring(math.floor(self.map.meter)), "TextDistance")
end

function CGameUI:UpdateBOSSHP()

    if self.map.boss and not self.map.boss.is_dead then

        self:SetWidgetVisible(true, "ImageBossHP")
        
        local component_hp = self.map.boss.component_list.hp
        if component_hp then

            local loadingbar = self:GetWidget("LoadingBarBossHP")
            if loadingbar then
                loadingbar:setPercent((component_hp.hp / component_hp.max_hp) * 100)
                -- TODO: 可以设置动画
            end
        end
    else
        self:SetWidgetVisible(false, "ImageBossHP")
    end
end

function CGameUI:Start()

    self.is_started = true

    -- 所有道具都遍历一遍，获取可以使用的内容
    local use_item_id_list = {}
    self.item_id_for_death_dash = nil
    self.already_use_item_for_revive = false
    self.already_pay_for_revive = false

    if self.stage_type ~= "compete" then

        local item_list = sPlayer.player_modules.package:GetItemList()
        for item_id_string,item_count in pairs(item_list) do
        
            local can_use = false
            local item_id = tonumber(item_id_string)

            -- 获取道具配置
            local item_entry = assets.items:GetEntry(item_id)
            if item_entry and item_entry.item_type ~= "box" then

                if item_entry.item_type == assets.items.ItemType.kTypeDeathDash then
                    self.item_id_for_death_dash = item_id
                else
                    can_use = true
                end
            end
            if can_use then
                sPlayer.player_modules.package:UseItem(item_id, 1)
                use_item_id_list[#use_item_id_list+1] = item_id
            end
        end

        if sPlayer.player_modules.random_item:GetID() then
            use_item_id_list[#use_item_id_list+1] = sPlayer.player_modules.random_item:GetID()
            sPlayer.player_modules.random_item:Consume()
        end
    end

    --self.widget:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(function()

        self:PlayAnimation("OpenAnimation1", false) 

        if self.map then
            self.map:Start(use_item_id_list, sPlayer.player_modules.attributes:GetAll())
        end
    --end)))

    -- 判断是否为首次游戏
    sCtrlGuide:Listen("join_game_"..self.stage_type.."_"..self.stage_id.."_"..self.child_stage_id)

end

function CGameUI:SetMap(stage_type, stage_id, child_stage_id)

    if self.game_rank_ui then
        self.game_rank_ui:RemoveFromParent()
        self.game_rank_ui = nil
    end
    if self.game_rank_meter_ui then
        self.game_rank_meter_ui:RemoveFromParent()
        self.game_rank_meter_ui = nil
    end

    if self.item_collect_listener then
        for _,item_collect_listener in pairs(self.item_collect_listener) do
            self.map.item_collect_event:Disconnect(item_collect_listener)
        end
        self.item_collect_listener = nil
    end

    if self.map then
        self.map:removeFromParent()
        self.map = nil
    end

    -- 播放动画
    self:PlayAnimation("OpenAnimation", false)

    -- 地图重置
    self.is_started = false
    self.is_pause = false

    -- 重新设置地图内容
    self.stage_type = stage_type
    self.stage_id = stage_id
    self.child_stage_id = child_stage_id
    self.child_stage_entry = sPlayer.player_modules.stage:GetChildStageEntry(self.stage_type, self.stage_id, self.child_stage_id)

    local panel_map = self:GetWidget("PanelMap")
    if not panel_map then
        return
    end

    -- 创建游戏地图节点
    local skin = sPlayer.player_modules.skins:GetActiveSkin()
    local mount = sPlayer.player_modules.mounts:GetActiveMount()
    self.map = require("app.views.game.game_map"):Create(stage_type, stage_id, child_stage_id, 
    player.db.skins.active_skin_id, 
    player.db.mounts.active_mount_id)
    if not self.map then
        return
    end

    -- 进行角色初始化
    if self.map.player then

        -- 注册角色攻击事件
        self.map.player.weapon_changed_event:Connect(function(weapon_id, bullet_count)
            self:SetWidgetEnableAndVisible(weapon_id and bullet_count > 0, "ButtonB")
        end)
    end

    -- 游戏结束回调函数
    self.map.gameover_callback = function(is_win) 
        self:GameOver(is_win)
    end

    self.map.object_enter_callback = function(object) self:OnObjectEnter(object) end
    self.map.section_group_changed_callback = function() self:ShowSectionDesc() end
    panel_map:addChild(self.map)

    if self.map.player then
        self.map.player.buff_changed_event:Connect(function() self:UpdateBuffInfo() end)
    end

    -- 显示一次BUFF
    self:UpdateBuffInfo()

    -- 显示地图名称
    -- self:ShowName()

    -- 显示第一段区域
    self:ShowSectionDesc()

    -- 显示星级信息
    if self.stage_type == "normal" then
        self:SetWidgetVisible(true, "PanelTarget")
        self:ShowStarInfo()

        -- 判断是否有才积累的条件
        local has_item_collect = false
        if self.child_stage_entry.star_condition then
            for i=1,#self.child_stage_entry.star_condition do
                local condition = self.child_stage_entry.star_condition[i]
                if condition then
                    if condition.type == game.CAssetGameCondition.item_collect then
                        self.item_collect_listener = self.item_collect_listener or {}
                        self.item_collect_listener[i] = self.map.item_collect_event:Connect(function(item_id)
                            if item_id == condition.param then
                                self:ShowStarInfoByIndex(i)
                            end
                        end)
                    elseif condition.type == game.CAssetGameCondition.score then
                        self.item_collect_listener = self.item_collect_listener or {}
                        self.item_collect_listener[i] = self.map.score_changed_event:Connect(function(score)
                            self:ShowStarInfoByIndex(i)
                        end)
                    end
                end
            end
        end
    else
        self:SetWidgetVisible(false, "PanelTarget")
    end

    -- 实时对战排行
    self:SetWidgetVisible(false, "PanelRank")
    self:SetWidgetVisible(false, "ImageGameSchedule")

    if self.map.game_rank then

        -- 实时排名界面
        self.game_rank_ui = require("app.views.game.game_rank_ui"):Create(self:GetWidget("PanelRank"), self.map.game_rank)
        if self.game_rank_ui then
            self:SetWidgetVisible(true, "PanelRank")
            self:AddChild(self.game_rank_ui)
        end

        -- 竞技模式下有总米数的地图才显示
        if self.map.total_meter then

            self.game_rank_meter_ui = 
            require("app.views.game.game_rank_meter_ui"):Create(self:GetWidget("ImageGameSchedule"), self.map.game_rank, self.map.total_meter, self.stage_type == "compete") 
            if self.game_rank_meter_ui then
                self:SetWidgetVisible(true, "ImageGameSchedule")
                self:AddChild(self.game_rank_meter_ui)
            end
        end
    end

    -- 怒气系统显示
    self:SetWidgetVisible(not self.child_stage_entry.is_boss, "ImageNQ")
end

function CGameUI:GameOver(is_win)

    if not is_win then

        if not self.already_use_item_for_revive then
            self.already_use_item_for_revive = true

            -- 判断是否使用道具复活
            if self.item_id_for_death_dash then
                sPlayer.player_modules.package:UseItem(self.item_id_for_death_dash, 1)
                self.map:Revive(false)
            else
                self:GameOver(is_win)
            end
            return
        elseif not self.already_pay_for_revive then
            self.already_pay_for_revive = true    
            
            -- 因为游戏结束会调整结算结果，所以再显示一次星级情况
            self:ShowStarInfo()

            -- 开启复活界面
            local stage_type = self.stage_type
            local stage_id = self.stage_id
            local child_stage_id = self.child_stage_id
            local record =  self.map.record

            if stage_type == "normal" then
                sUI:Open("app.views.ui.revive.stage_revive_ui", function(is_successful) 
                    if is_successful then
                        self.map:Revive(true)
                    else
                        sPlayer.player_modules.stage:StageLose(stage_type, stage_id, child_stage_id, record)
                    end
                end, self.stage_type, self.stage_id, self.child_stage_id)
            else
                sUI:Open("app.views.ui.revive.revive_ui", function(is_successful) 
                    if is_successful then
                        self.map:Revive(true)
                    else
                        game.CCtrlBattle:StageCompleted(stage_type, stage_id, child_stage_id, false, record)
                    end
                end)
            end
            return
        end
    end

    -- 因为游戏结束会调整结算结果，所以再显示一次星级情况
    self:ShowStarInfo()

    -- 游戏确认结束
    game.CCtrlBattle:StageCompleted(self.stage_type, self.stage_id, self.child_stage_id, is_win, self.map.record) 
end

function CGameUI:AddTouchEvent(callback, name)
    local widget = self:GetWidget(name)
    if widget ~= nil then
        widget:addTouchEventListener( function(touch, event)
            if event == 0 then
                if callback ~= nil then
                    callback(true)
                end
            elseif event == 2 or event == 3 then
                if callback ~= nil then
                    callback(false)
                end
            end
        end )
    end
end

function CGameUI:OnObjectEnter(object)

    if object.is_monster then
        self:ShowWarning()
    end
end

function CGameUI:ShowWarning()

    self:SetWidgetEnableAndVisible(true, "TextWarning")

    local label_warning = self:GetWidget("TextWarning")
    if label_warning then

        label_warning:setOpacity(0)
        label_warning:runAction(cc.Sequence:create(cc.FadeIn:create(0.5),
        cc.DelayTime:create(2), 
        cc.FadeOut:create(0.5),
        cc.CallFunc:create(function() 
            self:SetWidgetEnableAndVisible(false, "TextWarning")
        end)
        ))
    end
end

function CGameUI:ShowName()

    self:SetWidgetEnableAndVisible(true, "TextWarning")

    local label_name = self:GetWidget("TextWarning")
    if label_name then

        label_name:setText(sStrings:GetString(self.map.config.name))
        
        if self.map then
            self.map:Pause()
        end
        label_name:setColor(cc.c3b(255,0,0))
        label_name:setOpacity(0)
        label_name:runAction(cc.Sequence:create(
        cc.Sequence:create(cc.FadeIn:create(0.5), cc.DelayTime:create(1), cc.FadeOut:create(0.5)),
        cc.CallFunc:create(function() 
            self:SetWidgetEnableAndVisible(false, "TextWarning")

            if self.map then
                self.map:Resume()
            end
        end)
        ))
    end
end

function CGameUI:ShowSectionDesc()

    local desc = nil
    if self.map.current_section_group_config and self.map.current_section_group_config.desc then
        desc = sStrings:GetString(self.map.current_section_group_config.desc)
    end
    self:ShowCustomDesc(desc, true)
end

function CGameUI:ShowCustomDesc(desc, fade)

    fade = fade or false
    local panel_tip = self:GetWidget("PanelBossTip")
    if panel_tip then
        panel_tip:setVisible(true)
        panel_tip:stopAllActions()

        if desc then
            panel_tip:setOpacity(0)
            if fade then
                panel_tip:runAction(cc.Sequence:create(cc.FadeIn:create(0.5), cc.DelayTime:create(3), cc.FadeOut:create(1)))
            else
                panel_tip:runAction(cc.Sequence:create(cc.FadeIn:create(0.5)))
            end
            self:SetLabelText(desc, panel_tip, "TextSectionDesc")
        else
            panel_tip:runAction(cc.Sequence:create(cc.FadeOut:create(1)))
        end
    end
end

function CGameUI:ShowStarInfoByIndex(index)

    self.child_stage_entry.star_condition = self.child_stage_entry.star_condition or {}

    local condition = self.child_stage_entry.star_condition[index]
    if condition then
        local widget = self:GetWidget("PanelTarget", "Panel" .. index .. "Star")
        if widget then
            self:SetWidgetVisible(true, widget)

            -- 根据条件来创建文本描述
            local message = ""
            local is_finish = false
            if condition.type == game.CAssetGameCondition.win then
                message = sStrings:GetString(condition.type.desc)
                is_finish = self.map.is_win
            elseif condition.type == game.CAssetGameCondition.item_collect then
                message = sStringUtils:GetCombinedStringByID(condition.type.desc, 
                    {param=sStrings:GetString(game.CAssetGameObject[condition.param].name), value=condition.value})

                local have_count = self.map.record.item_collect[condition.param] or 0
                if have_count > condition.value then
                    have_count = condition.value
                end
                is_finish = (have_count >= condition.value)
                message = message.."["..have_count.."/"..condition.value.."]"
            elseif condition.type == game.CAssetGameCondition.score then
                message = sStringUtils:GetCombinedStringByID(condition.type.desc, 
                    {value=condition.value})
                if self.map.score then
                    is_finish = (self.map.score >= condition.value)
                end
            elseif condition.type == game.CAssetGameCondition.time then
                message = sStringUtils:GetCombinedStringByID(condition.type.desc, 
                    {value=condition.value})
                if self.map.record.time then
                    is_finish = (self.map.record.time <= condition.value)
                end
            end
            self:SetLabelText(message, widget, "TextCondition")
            self:SetWidgetVisible(not is_finish, widget, "ImageStarIcon1")
            self:SetWidgetVisible(is_finish, widget, "ImageStarIcon2")
        end
    else
        self:SetWidgetVisible(false, "PanelTarget", "Panel" .. index .. "Star")
    end
end

function CGameUI:ShowStarInfo()

    for i = 1, 3 do
        self:ShowStarInfoByIndex(i)
    end
end

function CGameUI:UpdateBuffInfo()

    local buff_count = 0

    -- 显示角色身上的BUFF
    if self.map.player then
        for buff_entry,buff in pairs(self.map.player.buff) do
            if buff_entry.icon_file then
                buff_count = buff_count + 1

                local image_prop = self:GetWidget("PanelBattleProp", "ImageProp"..buff_count)
                if image_prop then
                    image_prop:setVisible(true)
                    image_prop:loadTexture(buff_entry.icon_file)
                end
            end
        end
    end

    -- 清理余下的空位
    for i=buff_count+1,6 do
        self:SetWidgetVisible(false, "PanelBattleProp", "ImageProp"..i)
    end
end

return CGameUI

--endregion
