-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CTmxConfig = class("game.CTmxConfig")
local CTmxConfig = game.CTmxConfig

function CTmxConfig:ctor()

    self.height, self.width  = 0, 0
    self.map_info = {}
    self.ground = {}
    self.object = {}
    self.background = {}
end

function CTmxConfig:Init(tmx_file)

    local fullpath = cc.FileUtils:getInstance():fullPathForFilename(tmx_file)
    if not cc.FileUtils:getInstance():isFileExist(fullpath) then
        return false
    end

    local xml_data = cc.FileUtils:getInstance():getStringFromFile(fullpath)
    local tmx_map = cc.TMXTiledMap:createWithXML(xml_data, "tmx")
    if not tmx_map then 
        return false 
    end

    local tmx_map_size = tmx_map:getMapSize()
    self.width = tmx_map_size.width
    self.height = tmx_map_size.height

    local tmx_tile_size = tmx_map:getTileSize()
    self.tile_width = tmx_tile_size.width
    self.tile_height = tmx_tile_size.height

    self.map_info = tmx_map:getProperties()
    self.file = tmx_file

    self:_LoadGround(tmx_map)
    self:_LoadObject(tmx_map)

    return true
end

function CTmxConfig:Create(tmx_file)

    local config = CTmxConfig:new()
    if config == nil or config:Init(tmx_file) == false then
        return nil
    end
    return config
end

function CTmxConfig:_LoadGround(tmx_map)

    local layer = tmx_map:getLayer("ground")
    if layer then
        local layer_size = layer:getLayerSize()
        for row=1, layer_size.height do

            for col=1, layer_size.width do
                local gid = layer:getTileGIDAt(cc.p(col-1, layer_size.height-row))
                local properties = tmx_map:getPropertiesForGID(gid)
                if type(properties) == "table" then

                    local tile_name = properties.name
                    if tile_name then
                        self.ground[col] = self.ground[col] or {}
                        self.ground[col][row] = tile_name
                    end 
                end
            end
        end   
    end
end

function CTmxConfig:_LoadObject(tmx_map)

    self.object = {}

    -- 格子模式的物件布置方式
    local layer = tmx_map:getLayer("object")
    if layer then
        local layer_size = layer:getLayerSize()
        for row=1, layer_size.height do

            for col=1, layer_size.width do
                local gid = layer:getTileGIDAt(cc.p(col-1, layer_size.height-row))
                local properties = tmx_map:getPropertiesForGID(gid)
                if type(properties) == "table" then

                    local object_name = properties.name
                    if object_name then

                        -- 运行配置加载函数
                        local object_entry = game.CAssetGameObject[object_name]
                        if object_entry and object_entry.onLoad then
                            object_entry.onLoad(self, col, row)
                        end

                        -- 将物件依托在格子上
                        self.object[col] = self.object[col] or {}
                        self.object[col][row] = self.object[col][row] or {}
                        self.object[col][row][#self.object[col][row] + 1] = { name=object_name, x=(col-0.5)*game.CAssetGame.tile_width, y=(row-1)*game.CAssetGame.tile_height }
                    end 
                end
            end
        end   
    end

    -- 自由摆放模式的物件布置方式
    local object_group = tmx_map:getObjectGroup("object")    
    if object_group then

        local objects = object_group:getObjects()
        for key, object in pairs(objects) do

            if object.gid then 
                local properties = tmx_map:getPropertiesForGID(tonumber(object.gid))
                if type(properties) == "table" then
                    
                    local object_name = properties.name
                    if object_name then
                        
                        -- 将坐标转换到格子上
                        local col = math.floor(object.x / self.tile_width) + 1
                        local row = math.floor(object.y / self.tile_height) + 1

                        -- 将物件依托在格子上
                        self.object[col] = self.object[col] or {}
                        self.object[col][row] = self.object[col][row] or {}
                        self.object[col][row][#self.object[col][row] + 1] = { name=object_name, x=(object.x + object.width/2)*game.CAssetGame.tile_width/self.tile_width, y=(object.y + object.height/2)*game.CAssetGame.tile_height/self.tile_height }
                    end
                end
            end
        end
    end
end

return CTmxConfig

-- endregion
