--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

game.CFriendInfoUI = class("game.CFriendInfoUI",dxm.CDialogUI)

local CFriendInfoUI = game.CFriendInfoUI

CFriendInfoUI.Friendship = {
    "myself",
    "friend",
    "not_friend",
}

for key,value in ipairs(CFriendInfoUI.Friendship) do
    CFriendInfoUI.Friendship[value]=key
end

function CFriendInfoUI:ctor()
    CFriendInfoUI.super.ctor(self)

    self.friend = nil
    self.friend_ship = nil

    self.scroll_view_roles = nil
    self.scroll_view_medals = nil
end

function CFriendInfoUI:Init(friend)
    if friend == nil or self:InitWithFile("csd/Friends/MyBusinessCard") == false then
        return false
    end
    self.friend = friend
    sPlayer.player_modules.friend:FillFriendInfo(self.friend)
    return true
end

function CFriendInfoUI:Create(...)
    local ui = CFriendInfoUI.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

function CFriendInfoUI:onEnter()

    -- 显示角色头像
    self:SetImageViewPicture(sPathHelper:GetPlayerIconPath(self.friend.user_info.head_icon), "ImageHeadPortrait")

    -- 显示名字
    self:SetLabelText(self.friend.user_name, "TextPlayerName")

    -- 显示帐号等级
    self:SetLabelText(self.friend.user_info.level, "TextPlayerLv")

    -- 显示编号
    self:SetLabelText((tonumber(self.friend.user_id)+assets.const.min_user_id), "TextID")

    -- 显示关卡数
    local stage_id = "1-1"
    if self.friend.user_info and self.friend.user_info.completed_stage_id then
        stage_id = math.floor(self.friend.user_info.completed_stage_id/100).."-"..(self.friend.user_info.completed_stage_id%100)
    end
    self:SetLabelText(stage_id, "TextCheckpoint")

    -- 显示总星数
    local star_count = "0"
    if self.friend.user_info and self.friend.user_info.star_count then
        star_count = self.friend.user_info.star_count
    end
    self:SetLabelText(star_count, "TextStar")

    -- 显示无尽最高分
    local score_endless = "0"
    if self.friend.user_info and self.friend.user_info.infinite_best_score then
        score_endless = self.friend.user_info.infinite_best_score
    end
    self:SetLabelText(score_endless, "TextEndless")

    -- 索要体力按钮
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        local error_code = player.player_modules.friend:AskForActionPoint(self.friend)
        dxm.CMessageTipUI:Show(sStrings:GetString("ask_for_action_point_error_"..game.CPlayerModuleFriend.ErrorCode[error_code]), 35)
    end,"ButtonApply")

    -- 赠送体力按钮
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        local error_code = player.player_modules.friend:GiveActionPoint(self.friend)
        dxm.CMessageTipUI:Show(sStrings:GetString("give_action_point_error_"..game.CPlayerModuleFriend.ErrorCode[error_code]), 35)
    end,"ButtonGive")

    -- 删除好友按钮
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        sFriendSystem:RequestRemoveFriend(self.friend.user_id, function(error_code) 
            if error_code == sFriendSystem.ErrorCode.ok then
                self:Refresh()
            end
        end)
    end,"ButtonDelete")

    -- 添加好友按钮
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        sFriendSystem:RequestAddFriendQuickly(self.friend, true, function(error_code) 
            dxm.CMessageTipUI:Show(sStrings:GetString("add_friend_error_"..sFriendSystem.ErrorCode[error_code]), 35)
        end)
    end,"ButtonAddFriends")

    -- 分享名片按钮
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        -- TODO: 
    end,"ButtonSend")

    -- 关闭界面
    self:AddReleaseEvent(function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        self:Close()
    end,"ButtonClose")

    self:SetWidgetVisible(false, "ButtonSend")

    -- 刷新面板
    self:Refresh()

    self:ScheduleUpdate(function(dt) self:Update(dt) end)
end

function CFriendInfoUI:onExit()

    if self.scroll_view_roles then
        self.scroll_view_roles:RemoveFromParent()
        self.scroll_view_roles = nil
    end

    if self.scroll_view_medals then
        self.scroll_view_medals:RemoveFromParent()
        self.scroll_view_medals = nil
    end

end

function CFriendInfoUI:Update(dt)

    if self.scroll_view_medals then
        self:SetWidgetEnableAndVisible(self.scroll_view_medals:HasCellOutsideInFront(), "ImageArrowLeft")
        self:SetWidgetEnableAndVisible(self.scroll_view_medals:HasCellOutsideInBack(), "ImageArrowRight")
    end
end

function CFriendInfoUI:Refresh()

    -- 重新确认一下关系
    local friendship = nil
    if self.friend.user_id == sHttpUser.user_id then
        friendship = CFriendInfoUI.Friendship.myself
    elseif sFriendSystem:GetFriend(self.friend.user_id) then
        friendship = CFriendInfoUI.Friendship.friend
    else
        friendship = CFriendInfoUI.Friendship.not_friend
    end

    -- 判断关系是否发生变化
    if friendship == self.friendship then
        return
    end
    self.friendship = friendship

    -- TODO: 显示对应的关系面板
    for i=1,#CFriendInfoUI.Friendship do
        self:SetWidgetEnableAndVisible(i==self.friendship, "PanelButton"..i)
    end
end

return CFriendInfoUI

--endregion
