--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

game.CMailUI = class("game.CMailUI",dxm.CStackUI)

local CMailUI = game.CMailUI

CMailUI.TabType = {
    "ask_for_add_friend",
    "ask_for_action_point",
    "ask_for_unlock_stage",
    "reward",
}

for key,value in ipairs(CMailUI.TabType) do
    CMailUI.TabType[value] = key
end

local tab_name = {
    "ButtonAddFriends",
    "ButtonTL",
    "ButtonUnlock",
    "ButtonMail",
}

local message_action_type = {
    { "ask_for_add_friend" },
    { "ask_for_action_point" },
    { "ask_for_unlock_stage" },
    { "give_action_point", "title_daily_reward" },
}

local cell_name = {
    "app.views.ui.mail.mail_add_friend_cell_ui",
    "app.views.ui.mail.mail_ask_for_action_point_cell_ui",
    "app.views.ui.mail.mail_ask_for_unlock_stage_cell_ui",
    "app.views.ui.mail.mail_reward_cell_ui",
}

function CMailUI:ctor()
    CMailUI.super.ctor(self)

    self.scrollview = nil
    self.tab_type = nil
    self.tab_message = {}
    self.is_message_removed = false
end

function CMailUI:Init(tab_type)
    if self:InitWithFile("csd/Mail/Mail") == false then
        return false
    end
    self.tab_type = tab_type or CMailUI.TabType.ask_for_add_friend
    return true
end

function CMailUI:Create(...)
    local ui = CMailUI.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

function CMailUI:onEnter()

    sShortcutHelper:OpenTop()
    sShortcutHelper:OpenBottom()

    -- 标签按钮监听
    for i=1,#CMailUI.TabType do
        self:AddReleaseEvent(function()
            dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
            self:SwitchTab(i)
        end,tab_name[i])
    end

    -- 发送好友名片按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        sUI:OpenWithPushStack("app.views.ui.friend.friend_ui")
    end,"ButtonInvitation")

    -- 请求添加好友
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        sUI:Open("app.views.ui.friend.friend_add_ui")
    end,"ButtonAddFriends1")

    -- 请求好友赠送体力按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        sUI:OpenWithPushStack("app.views.ui.friend.friend_ui")
    end,"ButtonGive")

    -- 全部忽略按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        self:IgnoreAllMessage()
    end,"ButtonIgnore")

    -- 全部同意按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        self:AgreeAllMessage()
    end,"ButtonAgree")

    -- 全部领取按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        self:TakeAllReward()
    end,"ButtonReceive")

    -- 关闭界面
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:Close()
    end,"ButtonClose")

    self:AddReleaseEvent(function(sender)
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)   
        self:Close()
    end,
    "ButtonReturn") 

    -- 滚动层
    self.scroll_view = dxm.CScrollViewUI:Create(self:GetWidget("ScrollView"))
    if self.scroll_view ~= nil then
        self:AddChild(self.scroll_view)
        self.scroll_view.fix_tail = 380
        self.scroll_view.fix_head = 20
    end

    -- 显示对应标签下内容
    self:ShowTab()

    -- 监听消息移除
    sMessageSystem.remove_player_message_callback = function() self.is_message_removed = true end

    -- 定期更新
    self:ScheduleUpdate(function(dt) self:Update(dt) end)
end

function CMailUI:onExit()

    if self.scroll_view ~= nil then
        self.scroll_view:RemoveFromParent()
        self.scroll_view = nil
    end
end

function CMailUI:SwitchTab(tab_type)

    if self.tab_type == tab_type then
        return
    end
    self.tab_type = tab_type
    self:ShowTab()
end

function CMailUI:UpdateScrollView()

    -- 将个人消息分类别显示
    if self.scroll_view ~= nil then

        -- 清理内容
        if self.scroll_view ~= nil then
            self.scroll_view:RemoveAllCells()
        end

        -- 插入内容
        for _,message in ipairs(self.tab_message) do
            local cell_ui = require(cell_name[self.tab_type]):Create(message)
            if cell_ui ~= nil then
                self.scroll_view:AddCell(cell_ui)
            end
        end
        self.scroll_view:InitPosition()

        -- 判断当前是否存在内容
        self:SetWidgetEnableAndVisible(self.scroll_view.cell_ui_list.size == 0, "PanelNPC")
    end
end

function CMailUI:UpdateButton()
    
    local message_count = #self.tab_message

    self:SetWidgetEnableAndVisible((self.tab_type == CMailUI.TabType.ask_for_add_friend or self.tab_type == CMailUI.TabType.ask_for_unlock_stage) and message_count <= 0, "ButtonInvitation")
    self:SetWidgetEnableAndVisible((self.tab_type == CMailUI.TabType.ask_for_add_friend or self.tab_type == CMailUI.TabType.ask_for_unlock_stage) and message_count <= 0, "ButtonAddFriends1")
    self:SetWidgetEnableAndVisible(self.tab_type == CMailUI.TabType.ask_for_action_point and message_count <= 0, "ButtonGive")
    self:SetWidgetEnableAndVisible((self.tab_type == CMailUI.TabType.ask_for_add_friend or 
    self.tab_type == CMailUI.TabType.ask_for_action_point or
    self.tab_type == CMailUI.TabType.ask_for_unlock_stage) and message_count > 0, "ButtonIgnore")
    self:SetWidgetEnableAndVisible((self.tab_type == CMailUI.TabType.ask_for_add_friend or 
    self.tab_type == CMailUI.TabType.ask_for_action_point or
    self.tab_type == CMailUI.TabType.ask_for_unlock_stage) and message_count > 0, "ButtonAgree")
    self:SetWidgetEnableAndVisible(self.tab_type == CMailUI.TabType.reward and message_count > 0, "ButtonReceive")
end

function CMailUI:ShowTab()

    --  清理内容
    self.tab_message = {}

    -- 刷新按钮
    self:ShowTabMessage()

    -- 所有标签按钮的状态调整
    for i=1,#CMailUI.TabType do
        self:SetWidgetEnable(self.tab_type ~= i, tab_name[i])
    end

    -- 更新内容
    sMessageSystem:GetAllPlayerMessage(function() 
        self:RefreshTabMessage()
        self:RefreshTabMessageRedDot()
    end)
end

function CMailUI:RefreshTabMessageRedDot(tab_type)

    local has_tab_message = {}
    for _,message in pairs(sMessageSystem.message_db.player_message) do

        if message.content ~= nil then

            for i,_ in ipairs(CMailUI.TabType) do

                if (tab_type and tab_type ~= i) or has_tab_message[i] then
                    break
                end
            
                local is_tab_message = false

                local message_action_type = message_action_type[i]
                for _,single_message_action_type in pairs(message_action_type) do
                    if message.content.action_type == single_message_action_type then
                        is_tab_message = true
                        break
                    end
                end

                if is_tab_message then
                    has_tab_message[i] = true
                    break
                end
            end
        end
    end

    -- 标签按钮监听
    for i=1,#CMailUI.TabType do
        self:SetWidgetVisible((has_tab_message[i] or false) and i ~= self.tab_type, tab_name[i], "ImageRedDot")
    end
end

function CMailUI:RefreshTabMessage()

    -- 显示当前选项中的内容
    self.tab_message = {}
    for _,message in pairs(sMessageSystem.message_db.player_message) do

        if message.content ~= nil then

            local is_tab_message = false

            local message_action_type = message_action_type[self.tab_type]
            for _,single_message_action_type in pairs(message_action_type) do
                if message.content.action_type == single_message_action_type then
                    is_tab_message = true
                    break
                end
            end

            if is_tab_message then
                self.tab_message[#self.tab_message+1] = message
            end
        end
    end

    -- 根据message_id做排序
    table.sort(self.tab_message, function(a,b) return a.message_id < b.message_id end)

    -- 显示内容
    self:ShowTabMessage()
end

function CMailUI:ShowTabMessage()
    self:UpdateScrollView()
    self:UpdateButton()
end

function CMailUI:IgnoreAllMessage()

    for _,value in ipairs(self.tab_message) do
        sMessageSystem:RemovePlayerMessage(value.message_id)
    end
end

function CMailUI:AgreeAllMessage()

    for _,value in ipairs(self.tab_message) do
        sMessageSystem:ConfirmPlayerMessage(value.message_id)
    end
end

function CMailUI:TakeAllReward()

    for _,value in ipairs(self.tab_message) do
        sMessageSystem:ConfirmPlayerMessage(value.message_id)
    end
end

function CMailUI:Update(dt)

    if self.is_message_removed then
        self.is_message_removed = false
        self:RefreshTabMessage()
        self:RefreshTabMessageRedDot(self.tab_type)
    end
end

return CMailUI
--endregion
