-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

require "app.views.ui.rank.rank_cell_ui"
require "app.views.ui.rank.stage_rank_cell_ui"
require "app.views.ui.rank.stage_score_rank_ui"

game.CRankUI = class("game.CRankUI", dxm.CStackUI)

local CRankUI = game.CRankUI

CRankUI.RankType = {
    Stage = 1,
    Infinite = 2,
}

CRankUI.RankRange = {
    World = 1,
    Friend = 2,
}

function CRankUI:ctor()
    CRankUI.super.ctor(self)
end

function CRankUI:Init()
    self.rank_type = nil
    self.rank_range = nil
    return self:InitWithFile("csd/Additional/Ranking")
end

function CRankUI:Create()

    local ui = CRankUI.new()
    if ui == nil or ui:Init() == false then
        return nil
    end
    return ui
end

function CRankUI:onEnter()
    self:AddReleaseEvent( function(sender)
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release) 
        self:Close()
    end ,
    "ButtonClose")

    sShortcutHelper:OpenTop()

    -- self:SetWidgetVisible(false, "ImageBottom")
    self:SetWidgetVisible(false, "PanelMyRank")

    local scroll_view = self:GetWidget("ScrollView")
    self.scroll_view_ui = dxm.CScrollViewUI:Create(scroll_view)
    self.scroll_view_ui.fix_tail = 480
    self.scroll_view_ui.fix_head = 20
    self:RefreshScrollview(CRankUI.RankType.Stage, CRankUI.RankRange.World)

    self:AddReleaseEvent( function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:RefreshScrollview(CRankUI.RankType.Stage, self.rank_range)
    end , "ButtonCheckpoint")

    self:AddReleaseEvent( function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:RefreshScrollview(CRankUI.RankType.Infinite, self.rank_range)
    end , "ButtonEndless")

    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:RefreshScrollview(self.rank_type, CRankUI.RankRange.Friend)
    end, "ButtonFriends")

    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)  
        self:RefreshScrollview(self.rank_type, CRankUI.RankRange.World)
    end, "ButtonWorld")

    self:AddReleaseEvent(function(sender)
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)   
        self:Close()
    end,
    "ButtonReturn") 

    self:AddListener("player_name_changed", function()
        self:ShowScrollview()
    end)
end

function CRankUI:RefreshScrollview(rank_type, rank_range)
    if self.rank_type == rank_type and self.range == rank_range then
        return
    end

    self:SetWidgetEnable(rank_type ~= CRankUI.RankType.Infinite, "ButtonEndless")
    self:SetWidgetEnable(rank_type ~= CRankUI.RankType.Stage, "ButtonCheckpoint")

    self:SetWidgetEnable(rank_range ~= CRankUI.RankRange.Friend, "ButtonFriends")
    self:SetWidgetEnable(rank_range ~= CRankUI.RankRange.World, "ButtonWorld")

    self.rank_type = rank_type
    self.rank_range = rank_range

    -- self:SetWidgetVisible(false, "ImageBottom")
    self:SetWidgetVisible(false, "PanelMyRank")

    -- 显示滚动层内容
    self:ShowScrollview()

    -- 如果是好友则需要最新的好友列表
    if rank_range == CRankUI.RankRange.Friend then

        -- 显示滚动层内容
        if sFriendSystem.friend_count <= 0 then

            sFriendSystem:RequestGetFriends(function()
                if self.is_entered ~= true or (self.rank_type ~= rank_type or self.rank_range ~= rank_range) then
                    return
                end
                player.player_modules.friend:MakeSureFriendExist(function(is_changed) self:ShowScrollview() end)
            end)
        end
    end
end

function CRankUI:ShowScrollview()

    -- self:SetWidgetVisible(player.db.info.name ~= nil, "ImageBottom")
    self:SetWidgetVisible(player.db.info.name ~= nil, "PanelMyRank")

    if self.rank_type == CRankUI.RankType.Stage then
        self:ShowStageRankScrollView(self.rank_range)
    else
        self:ShowInfiniteRankScrollView(self.rank_range)
    end

    self:RefreshMyRankInfo(self.rank_type, self.rank_range)
end

function CRankUI:ShowStageRankScrollView(rank_range)
    player.player_modules.rank:RequestGetRank("stage", 1, function()
        if self.scroll_view_ui then
            self.scroll_view_ui:RemoveAllCells()

            local rank_top = {}
            if rank_range == CRankUI.RankRange.Friend then
                rank_top = player.player_modules.rank:GetRankTopWithFriend("stage")
            else
                rank_top = player.player_modules.rank:GetRankTop("stage")
            end
            
            if rank_top then
                self.scroll_view_ui:RemoveAllCells()
                for _, rank_player in pairs(rank_top) do
                    self.scroll_view_ui:AddCell(game.CFBStageRankCellUI:Create(rank_player))
                end

                self.scroll_view_ui:InitPosition()
                self:AddChild(self.scroll_view_ui)
            end
        end
    end )
end

function CRankUI:ShowInfiniteRankScrollView(rank_range)
    player.player_modules.rank:RequestGetRank("infinite", 1, function()
        if self.scroll_view_ui then
            self.scroll_view_ui:RemoveAllCells()

            local rank_top = {}
            if rank_range == CRankUI.RankRange.Friend then
                rank_top = player.player_modules.rank:GetRankTopWithFriend("infinite")
            else
                rank_top = player.player_modules.rank:GetRankTop("infinite")
            end

            if rank_top then
                self.scroll_view_ui:RemoveAllCells()
                table.sort(rank_top, function(a, b) return a.pos < b.pos end)
                for _, rank_player in pairs(rank_top) do
                    self.scroll_view_ui:AddCell(game.CRankCellUI:Create(rank_player))
                end

                self.scroll_view_ui:InitPosition()
                self:AddChild(self.scroll_view_ui)
            end
        end
    end )
end

function CRankUI:onExit()

end

function CRankUI:RefreshMyRankInfo(rank_type, rank_range)
    local rank_info = nil
    if rank_type == CRankUI.RankType.Stage then
        rank_info = player.player_modules.rank:GetMyRankNumber("stage")
    else
        rank_info = player.player_modules.rank:GetMyRankNumber("infinite")
    end

    self:SetLabelText(player.db.info.name, "PanelMyRank", "TextPlayerName")

    local rank_number = nil

    if rank_info then

        local pos = 0
        if rank_range == CRankUI.RankRange.Friend then
            pos = rank_info.pos_with_friend
        else
            pos = rank_info.pos
        end

        self:SetLabelText(pos, "PanelMyRank", "TextMyRankName", "TextMyRank")
        self:SetLabelText(rank_info.value or "", "PanelMyRank", "TextMyScore")
        self:SetLabelText(rank_info.subvalue or "", "PanelMyRank", "TextHaveStar")
        self:SetLabelText(math.floor(rank_info.value/100).."-"..(rank_info.value%100), "PanelMyRank", "TextCheckpoint")
        self:SetWidgetVisible(true, "PanelMyRank", "TextMyRankName")
        self:SetWidgetVisible(rank_type == CRankUI.RankType.Infinite, "PanelMyRank", "TextMyScoreName")
        self:SetWidgetVisible(rank_type == CRankUI.RankType.Stage, "PanelMyRank", "TextHaveStarName")
        self:SetWidgetVisible(rank_type == CRankUI.RankType.Stage, "PanelMyRank", "PanelCheckpoint")
        self:SetWidgetVisible(false, "PanelMyRank", "TextMyRankName2")
        rank_number = pos
    else
        self:SetWidgetVisible(false, "PanelMyRank", "TextMyRankName")
        self:SetWidgetVisible(false, "PanelMyRank", "TextMyScoreName")
        self:SetWidgetVisible(false, "PanelMyRank", "TextHaveStarName")
        self:SetWidgetVisible(false, "PanelMyRank", "PanelCheckpoint")
        self:SetWidgetVisible(true, "PanelMyRank", "TextMyRankName2")
    end

    self:SetImageViewPicture(sPathHelper:GetPlayerIconPath(player.db.head_icon), "ImageHeadPortrait")

    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "AtlasLabelRanking")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking1")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking2")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking3")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai1")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai2")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai3")
    self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai4")

    if rank_number == 1 then
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking1")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai1")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom1")
        self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom2")
    elseif rank_number == 2 then
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking2")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai2")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom1")
        self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom2")
    elseif rank_number == 3 then
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRanking3")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai3")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom1")
        self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom2")
    else
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageRankingGai4")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "AtlasLabelRanking")
        self:SetWidgetVisible(false, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom1")
        self:SetWidgetVisible(true, "PanelMyRank", "ImageHeadPortraitBottom", "ImageBottom2")
    end
end

return CRankUI

-- endregion
