-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CStageReadyUI = class("game.CStageReadyUI", dxm.CStackUI)

local CStageReadyUI = game.CStageReadyUI

function CStageReadyUI:ctor()
    CStageReadyUI.super.ctor(self)
end

function CStageReadyUI:Init(stage_type, stage_id, child_stage_id, close_callback)
    self.stage_type = stage_type
    self.stage_id = stage_id
    self.child_stage_id = child_stage_id
    self.close_callback = close_callback
    return self:InitWithFile("csd/Checkpoint/BattleProp")
end

function CStageReadyUI:Create(...)
    local ui = CStageReadyUI.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

function CStageReadyUI:onEnter()

    self.scroll_view = dxm.CScrollViewUI:Create(self:GetWidget("ScrollView"))
    if self.scroll_view then
        self:AddChild(self.scroll_view)

        -- 获取当前的最高关卡信息
        local top_stage_info = sPlayer.player_modules.stage:GetTopStageInfo(self.stage_type)

        -- 根据ID来做显示排序
        local shop_item_list = {}
        for _, shop_item in pairs(assets.shop_item.config) do
            shop_item_list[#shop_item_list+1] = shop_item
        end
        table.sort(shop_item_list, function(a,b) return a.id < b.id end)

        -- 逐个显示商品界面
        for _, shop_item in ipairs(shop_item_list) do

            local can_use = false

            -- 存在关卡条件
            if shop_item.open_stage_id then
                shop_item.open_child_stage_id = shop_item.open_child_stage_id or 1

                if top_stage_info and 
                ((top_stage_info.stage_id > shop_item.open_stage_id) or 
                (shop_item.open_stage_id == top_stage_info.stage_id and top_stage_info.child_stage_id >= shop_item.open_child_stage_id)) then
                    can_use = true
                end
            else
                can_use = true
            end

            -- 判断能否使用
            if can_use then
                local cell = require("app.views.ui.stage.stage_ready_cell_ui"):Create(shop_item)
                if cell then
                    self.scroll_view:AddCell(cell)
                end
            end
        end

        self.scroll_view:InitPosition()
    end

    -- 开始按钮
    self:AddReleaseEvent( function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        self:Join()
    end , "ButtonStart")

    -- 返回按钮
    self:AddReleaseEvent( function()
        dxm.audio:playSound(assets.sound.config.sound.ui_button_release)
        if self.stage_type == "infinite" then
            game.CCtrlBattle:GotoMain()
        else
            sUI:OpenWithClearStack("app.views.ui.stage.stage_list_ui", self.stage_type, self.stage_id, self.child_stage_id)
        end
    end , "ButtonReturn")

    -- 随机开局BUFF
    self:ShowRandomBuff()

    -- 监听随机BUFF变更
    self.listener = sPlayer.player_modules.random_item.random_item_changed_callback:Connect(function() self:ShowRandomBuff() end)

    -- 引导触发
    --game.CGuideController:Listen("prejoin_stage_"..self.stage_type.."_"..self.stage_id.."_"..self.child_stage_id)

    self:_RefreshAd()
end

function CStageReadyUI:onExit()

    -- 取消监听随机BUFF变更
    if self.listener then
        sPlayer.player_modules.random_item.random_item_changed_callback:Disconnect(self.listener)
    end
end

function CStageReadyUI:ShowRandomBuff()

    local message = sPlayer.player_modules.random_item:GetMessage()
    if message == nil then
        self:SetWidgetEnableAndVisible(false, "TextBlessing")
    else
        self:SetWidgetEnableAndVisible(true, "TextBlessing")
        self:SetLabelText(message, "TextBlessing")
    end
end

function CStageReadyUI:Join()

    if game.CCtrlBattle:StartStage(self.stage_type, self.stage_id, self.child_stage_id) == true then
        dxm.CUIManager:GetInstancePtr():OpenStackUIWithClear()
    end
end

function CStageReadyUI:_RefreshAd()

    local panel_name = "ButtonAdvertisement"
    self:SetWidgetVisible(sPlayer.player_modules.ad:IsPop(2), panel_name)
    self:AddReleaseEvent(function() 
        sPlayer.player_modules.ad:Pop(2, function(is_ok, reward_type, reward_param) 
            if is_ok then
                sCtrlReward:AdReward(reward_type, reward_param)
                
                self:SetWidgetVisible(false, panel_name)
                if self.callback then self.callback(false) end
            end
        end)
    end, panel_name)
end

return CStageReadyUI

-- endregion
