-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

game.CStageResultWinUI = class("game.CStageResultWinUI", dxm.CStackUI)

local CStageResultWinUI = game.CStageResultWinUI

function CStageResultWinUI:Init(stage_type, stage_id, child_stage_id, record)
    self.stage_type = stage_type
    self.stage_id = stage_id
    self.child_stage_id = child_stage_id
    self.record = record

    -- HistoryBest
    if record.best_score > record.old_best_score then
        if self:InitWithFile("csd/Battle/HistoryBest") == false then
            return false
        end
    else
        if self:InitWithFile("csd/Battle/Settlement2") == false then
            return false
        end
    end
    return true
end

function CStageResultWinUI:Create(...)

    local ui = CStageResultWinUI.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

function CStageResultWinUI:onEnter()

    -- 确认按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)
        self:OpenMainMenu()
    end, "ButtonOK")

    -- 继续按钮
    self:AddReleaseEvent(function()
        dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)
        self:OpenStageReadyUI()
    end, "ButtonGoOn")

    -- 显示星级信息
    self:ShowStarInfo()

    -- 显示分数
    self:ShowScore()

    -- 显示关卡奖励
    self:ShowReward()

    -- 更新广告
    self:_RefreshAd()
end

function CStageResultWinUI:onExit()
    
    -- 音乐播放
    dxm.audio:playMusic(assets.sound.config.music.main)
end

function CStageResultWinUI:GetNextStage()

    local stage_id = self.stage_id
    local child_stage_id = self.child_stage_id + 1

    while true do

        local stage_entry = sPlayer.player_modules.stage:GetStageEntry(self.stage_type, stage_id)
        if stage_entry then
        
            if child_stage_id > stage_entry.child_stage_count then
                stage_id = stage_id + 1
                child_stage_id = 1
            else
                return self.stage_type, stage_id, child_stage_id
            end
        else
            return self.stage_type, self.stage_id, self.child_stage_id
        end
    end
end

function CStageResultWinUI:OpenMainMenu()

    -- 关闭界面
    self:Close()

    -- 进入下一关的界面
    sCtrlMainMenu:BackGame()
end

function CStageResultWinUI:OpenStageReadyUI()

    -- 关闭界面
    self:Close()

    -- 创建游戏场景
    require("app.views.game.game_ui"):Show("infinite", 1, 1)
    sUI:Open("app.views.ui.stage.stage_ready_ui", "infinite", 1, 1)
end

function CStageResultWinUI:ShowStarInfo()

    -- 判断关卡是否存在
    local stage_entry = sPlayer.player_modules.stage:GetChildStageEntry(self.stage_type, self.stage_id, self.child_stage_id)
    if stage_entry == nil then
        return
    end

    -- 获取关卡信息
    local stage_info = sPlayer.player_modules.stage:GetStageInfo(self.stage_type, self.stage_id, self.child_stage_id).star_info
    for i = 1, 3 do

        -- 初始化星级条件
        stage_entry.star_condition = stage_entry.star_condition or {}

        -- 条件显示
        local condition = stage_entry.star_condition[i]
        if condition then

            local widget = self:GetWidget("Panel" .. i .. "Star")
            if widget then
                self:SetWidgetVisible(true, widget)

                -- 根据条件来创建文本描述
                local message = ""
                if condition.type == game.CAssetGameCondition.win then
                    message = sStrings:GetString(condition.type.desc)
                elseif condition.type == game.CAssetGameCondition.item_collect then
                    message = sStringUtils:GetCombinedStringByID(condition.type.desc, 
                        {param=sStrings:GetString(game.CAssetGameObject[condition.param].name), value=condition.value})
                elseif condition.type == game.CAssetGameCondition.score or condition.type == game.CAssetGameCondition.time then
                    message = sStringUtils:GetCombinedStringByID(condition.type.desc, 
                        {value=condition.value})
                end
                self:SetLabelText(message, widget, "TextCondition")
                self:SetWidgetVisible(stage_info[i] or false, widget, "ImageState1")
                self:SetWidgetVisible(not(stage_info[i] or false), widget, "ImageState2")
            end
        else
            self:SetWidgetVisible(false, "Panel" .. i .. "Star")
        end
    end

    -- 显示关卡名称
    self:SetLabelText(sStrings:GetString(stage_entry.name), "TextNowCheckpointName")
end

function CStageResultWinUI:ShowReward()

    local count = 1

    if self.record.reward then

        -- 解析奖励
        local reward_list = game.CRewardList:ParseFromString(self.record.reward)
        for _,reward in pairs(reward_list.list) do

            self:SetWidgetVisible(true, "ImageRewardBottom"..count)

            local icon_pos = self:GetWidget("ImageRewardBottom"..count, "ImageRewardIcon")
            self.icon = cc.Sprite:create(sPathHelper:GetRewardIconPath(reward))
            if self.icon then
                icon_pos:addChild(self.icon)
                --self.icon:setScale(0.5)
                self.icon:setPosition(cc.p(icon_pos:getContentSize().width / 2, icon_pos:getContentSize().height / 2))
            end

            self:SetLabelText("x"..reward.count,"ImageRewardBottom"..count, "TextNumber")

            -- 增加奖励数量
            count = count + 1
        end
    end

    for i=count,3 do
        self:SetWidgetVisible(false, "ImageRewardBottom"..i)
    end
end

function CStageResultWinUI:ShowScore()

    self:SetLabelText(math.floor(self.record.score), "TextFractionName", "TextFraction")
    self:SetLabelText(math.floor(self.record.best_score), "TextBestFractionName", "TextFraction")
    self:SetLabelText(self.record.meter, "TextDistanceName", "TextDistance")
    self:SetLabelText(self.record.best_meter, "TextBestDistanceName", "TextDistance")
end

function CStageResultWinUI:_RefreshAd()

    local panel_name = "ButtonAdvertisement"
    self:SetWidgetVisible(sPlayer.player_modules.ad:IsPop(6), panel_name)
    self:AddReleaseEvent(function() 
        sPlayer.player_modules.ad:Pop(6, function(is_ok, reward_type, reward_param) 
            if is_ok then
                self:RewardDouble()
                
                self:SetWidgetVisible(false, panel_name)
                if self.callback then self.callback(false) end
            end
        end)
    end, panel_name)
end

function CStageResultWinUI:RewardDouble()

    if self.record.reward then

        local reward_list = game.CRewardList:ParseFromString(self.record.reward)
        for _,reward in pairs(reward_list.list) do
            if reward.reward_type ~= game.RewardType.kTypeItemBox then

                -- 增加额外的奖励量
                sPlayer.player_modules.reward:Take(reward.reward_type, reward.id, reward.count)

                -- 正统数据翻倍显示
                reward.count = reward.count * 2
            end
        end
        self.record.reward = reward_list:SerializeToString()

        -- 重新刷新界面
        self:ShowReward()
    end
end

return CStageResultWinUI

-- endregion
