-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

dxm.CFriendSystem = { }

local CFriendSystem = dxm.CFriendSystem
cc.exports.sFriendSystem = CFriendSystem

CFriendSystem.ErrorCode = {
    "ok",
    "friend_is_already_exists",
    "send_player_message_failed",
    "network_is_error",
    "friend_is_not_exists",
    "too_busy",
}

for key,value in ipairs(CFriendSystem.ErrorCode) do
    CFriendSystem.ErrorCode[value] = key
end

function CFriendSystem:Init()
    self.friends = {}
    self.friend_count = 0
    self.get_time = nil
    self.add_time = {}

    self.add_friend_callback = nil
    self.remove_friend_callback = nil

    sMessageSystem:RegisterPlayerMessageConfirmProc("ask_for_add_friend", function(message) 
        self:ResponseAddFriend(message.sender.user_id, function(error_code) end)
    end)

    self.init_friend_function = nil
end

function CFriendSystem:RequestGetFriends(callback)

    local current_time = os.time()
    if self.get_time == nil or current_time - self.get_time >= 10 then
        self.get_time = current_time

        -- 获取服务器数据
        sHttpUser:GetFriends( function(code, data)

            local error_code = CFriendSystem.ErrorCode.ok
            if code == 0 then

                self.friends = {}
                self.friend_count = 0

                -- 将在线内容拷贝过来
                for _, value in ipairs(data.friends) do

                    if self.friends[value.user_id] == nil then
                        self.friends[value.user_id] = value
                        self.friend_count = self.friend_count + 1

                        -- 初始化好友数据
                        if self.init_friend_function then
                            self.init_friend_function(value)
                        end
                    end
                end
            else
                error_code = CFriendSystem.ErrorCode.network_is_error
            end

            if callback ~= nil then
                callback(error_code)
            end

            if self.add_friend_callback then
                self.add_friend_callback()
            end
        end )
    else

        if callback ~= nil then
            callback(CFriendSystem.ErrorCode.ok)
        end
    end
end

function CFriendSystem:GetFriend(user_id)
    return self.friends[user_id]
end

function CFriendSystem:RequestAddFriendQuickly(user, check_cd, callback)

    if user == nil or self.friends[user.user_id] then
        return
    end

    -- 发送一个情况让对方也加我做好友
    self:RequestAddFriend(user.user_id, check_cd, callback)

    -- 立即添加好友
    self:AddFriend(user.user_id, nil)

    -- 提前爽起来
    self.friends[user.user_id] = user
    self.friend_count = self.friend_count + 1
    if self.add_friend_callback then
        self.add_friend_callback()
    end
end

function CFriendSystem:RequestAddFriend(user_id, check_cd, callback)

    local current_time = os.time()
    local error_code = CFriendSystem.ErrorCode.ok

    -- 判断用户是否已添加该好友
    local user = self.friends[user_id]
    if user ~= nil then
        error_code = CFriendSystem.ErrorCode.friend_is_already_exists
    elseif check_cd == true and self.add_time[user_id] ~= nil and current_time - self.add_time[user_id] < 5 then
        error_code = CFriendSystem.ErrorCode.too_busy
    else
        self.add_time[user_id] = current_time
        self:SendRequestAddFriendMessage(user_id, callback)
        return
    end

    -- 执行回调
    if callback then
        callback(error_code)
    end
end

function CFriendSystem:RequestAddFriendCallback(user_id, callback)
    self:AddFriend(user_id, callback)
end

function CFriendSystem:ResponseAddFriend(user_id, callback)

    self:AddFriend(user_id, function(error_code)

        -- 执行回调
        if callback then
            callback(error_code)
        end
    end )
end

function CFriendSystem:AddFriend(user_id, callback)

    print("CFriendSystem:AddFriend", user_id)

    -- 立即添加好友
    sHttpUser:AddFriend(user_id, false, function(code, data)

        print("sHttpUser:AddFriend", code)

        local error_code = CFriendSystem.ErrorCode.ok
        if code ~= 0 then
            error_code = CFriendSystem.ErrorCode.network_is_error
        end

        if callback then
            callback(error_code)
        end
    end )
end

function CFriendSystem:SendRequestAddFriendMessage(user_id, callback)

    -- 调用信息系统
    sMessageSystem:SendPlayerMessage(user_id, "ask_for_add_friend", {},
    function(code, data)

        local error_code = CFriendSystem.ErrorCode.ok
        if code ~= 0 then
            error_code = CFriendSystem.ErrorCode.send_player_message_failed
        end

        if callback then
            callback(error_code)
        end
    end )
end

function CFriendSystem:RequestRemoveFriend(user_id, callback)

    local error_code = CFriendSystem.ErrorCode.ok

    -- 判断用户是否已添加该好友
    local user = self.friends[user_id]
    if user == nil then
        error_code = CFriendSystem.ErrorCode.friend_is_not_exists
    else
        self:RemoveFriend(user_id, function(error_code)
            if callback then
                callback(error_code)
            end
        end )
        return
    end

    -- 执行回调
    if callback then
        callback(error_code)
    end
end

function CFriendSystem:RemoveFriend(user_id, callback)

    -- 本地移除好友
    self.friends[user_id] = nil
    self.friend_count = self.friend_count - 1

    if self.remove_friend_callback ~= nil then
        self.remove_friend_callback()
    end

    -- 同步移除好友
    sHttpUser:RemoveFriend(user_id, false, function(code, data) 
        print(code)

        local error_code = CFriendSystem.ErrorCode.ok
        if code ~= 0 then
            error_code = CFriendSystem.ErrorCode.network_is_error
        end

        if callback then
            callback(error_code)
        end
    end)
end

function CFriendSystem:RequestCustom(user_id, action_type, action_data, callback)

    local error_code = CFriendSystem.ErrorCode.ok

    -- 判断用户是否已添加该好友
    local user = self.friends[user_id]
    if user == nil then
        error_code = CFriendSystem.friend_is_not_exists
    else
        self:SendRequestCustomFriendMessage(user_id, action_type, action_data, callback)
        return
    end

    -- 执行回调
    if callback then
        callback(error_code)
    end
end

function CFriendSystem:SendRequestCustomFriendMessage(user_id, action_type, action_data, callback)

    -- 调用信息系统
    sMessageSystem:SendPlayerMessage(user_id, action_type, action_data,
    function(code, data)
        
        local error_code = CFriendSystem.ErrorCode.ok
        if code ~= 0 then
            error_code = CFriendSystem.ErrorCode.send_player_message_failed
        end

        if callback then
            callback(error_code)
        end
    end)
end

return CFriendSystem

--endregion
