--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

dxm.GameRecommender = {}
local GameRecommender = dxm.GameRecommender
cc.exports.sGameRecommender = GameRecommender

-- 下载文件类型
GameRecommender.DownloadFileType = {
    icon = { url_name="icon_url" },
    image = { url_name="image_url" },
}

for key,value in pairs(GameRecommender.DownloadFileType) do
    value.id = key
end

--[[

url/getWebAdInfo?game_id=princess_new

{
    "code" = 1
    "data" = {
        "game_id"              = "recommender_game_id"
        "download_url_android" = "http://itunes.apple.com/app/id1134919719"
        "download_url_ios"     = "http://itunes.apple.com/app/id1134919719"
        "icon_url"             = "http://community.73776.com/Public/Uploads/20161109190151829.png"
        "image_url"            = "http://community.73776.com/Public/Uploads/20161109190154255.png"
        "package_name_android" = "app.zdjoys.recommender_game_id"
        "package_name_ios"     = "app.zdjoys.recommender_game_id"
    }
}
]]--

-- 初始化游戏推荐器
function GameRecommender:Init(game_id)

    -- 初始化数据
    self.game_id = game_id
    self.http_request = dxm.CHttpRequest.new()
    self.url = assets.config.http_url.."/sdk/Verify/"
    self.download_path = string.gsub(device.writablePath, "[\\\\/]+$", "") .. device.directorySeparator .. "game_recommender"
    self.last_recommend_game_id = nil

    -- 本地数据相关
    sDatabase.data.game_recommender.recommend = sDatabase.data.game_recommender.recommend or {}
    sDatabase.data.game_recommender.recommend_tasks = sDatabase.data.game_recommender.recommend_tasks or {}
    sDatabase.data.game_recommender.completed_recommend_tasks = sDatabase.data.game_recommender.completed_recommend_tasks or {}
end

-- 获取已经完成下载推荐的游戏的任务;
function GameRecommender:GetRecommendTaskCompletedGames()

    local game_id_list = '{"ignore_list":['
    local i = 0
    for key, task in pairs(sDatabase.data.game_recommender.completed_recommend_tasks) do
        if i~=0 then game_id_list = "," end
        game_id_list = game_id_list..'"'..key..'"'
        i = i+1
    end
    game_id_list = game_id_list.."]}"
    return game_id_list
end

-- 请求获取当前推荐的游戏
function GameRecommender:RequestRecommendGame(callback)

    local handle = { callback = callback, is_download_file_loaded = {} }
    local task_completed_game_id_list = self:GetRecommendTaskCompletedGames()
    self.http_request:Request(self.url, "getWebAdInfo", "game_id=" .. self.game_id.."&ignore_list="..task_completed_game_id_list, function(code, data)

        if code == 1 and data ~= nil then
            self:_SyncRecommendGame(data, handle)
        elseif handle.callback ~= nil then
            handle.callback(self.last_recommend_game_id)
        end
    end )
    return handle
end

-- 取消可能正在进行的推荐操作
function GameRecommender:CancelRecommendGame(handle)

    if handle ~= nil then
        handle.callback = nil
    end
end

-- 获取图标路径
function GameRecommender:GetIconFile(game_id)
    return self:_GetDownloadFile(game_id, GameRecommender.DownloadFileType.icon)
end

-- 获取图片路径
function GameRecommender:GetImageFile(game_id)
    return self:_GetDownloadFile(game_id, GameRecommender.DownloadFileType.image)
end

-- 获取游戏名字
function GameRecommender:GetGameName(game_id)
    local recommend = sDatabase.data.game_recommender.recommend[game_id]
    if recommend == nil or recommend.download_file == nil then
        return nil
    end
    return recommend.data.name
end

-- 下载游戏
function GameRecommender:DownloadGame(game_id)

    local recommend = sDatabase.data.game_recommender.recommend[game_id]
    if recommend == nil or recommend.data == nil then
        return
    end
    
    -- 打开下载链接
    if device.platform == "ios" then
        CDeviceUtil:GetInstancePtr():OpenURL(recommend.data.download_url_ios)
    else
        CDeviceUtil:GetInstancePtr():RunCommand("download", '{"url":\"'..recommend.data.download_url_android..'"}')
    end
    
end

-- 分析游戏内容
function GameRecommender:_SyncRecommendGame(data, handle)

    -- 加载所有的内容
    sDatabase.data.game_recommender.recommend[data.game_id] = sDatabase.data.game_recommender.recommend[data.game_id] or {}
    local recommend = sDatabase.data.game_recommender.recommend[data.game_id]
    for key,value in pairs(GameRecommender.DownloadFileType) do
        self:_LoadDownloadFile(recommend, data, handle, value)
    end

    -- 复制为最新的配置
    sDatabase.data.game_recommender.recommend.current_game_id = data.game_id
    sDatabase.data.game_recommender.recommend[data.game_id].data = data
end

-- 
function GameRecommender:GetCurrentGameID()
    return sDatabase.data.game_recommender.recommend.current_game_id
end

-- 下载文件是否存在
function GameRecommender:_IsDownloadFileExists(game_id, file_type)

    local file_path = self:_GetDownloadFile(game_id, file_type)
    if file_path ~= nil and io.exists(file_path) == true then
        return true
    end
    return false
end

-- 获取下载内容的路径
function GameRecommender:_GetDownloadFile(game_id, file_type)
    
    local recommend = sDatabase.data.game_recommender.recommend[game_id]
    if recommend == nil or recommend.download_file == nil then
        return nil
    end
    return recommend.download_file[file_type.id]
end

-- 加载下载内容
function GameRecommender:_LoadDownloadFile(recommend, data, handle, file_type)

    recommend.data = recommend.data or {}
    if recommend.data[file_type.url_name] ~= data[file_type.url_name] or self:_IsDownloadFileExists(data.game_id, file_type) == false then
        
        -- 开始下载文件
        CAxel:GetInstancePtr():Download(data[file_type.url_name], 
            self.download_path,
            function(state, result)
                if state==kDownloadFinish then
                    print("game recommender file "..result.file_path.." done")

                    -- 加载完成
                    recommend.download_file = recommend.download_file or {}
                    recommend.download_file[file_type.id] = result.file_path

                    -- 完成加载
                    self:_LoadDownloadFileComplete(recommend, handle, file_type)
                end
            end
        )
    else

        -- 完成加载
        self:_LoadDownloadFileComplete(recommend, handle, file_type)
    end
end

-- 加载下载文件完成
function GameRecommender:_LoadDownloadFileComplete(recommend, handle, file_type)
    handle.is_download_file_loaded[file_type.id] = true
    self:_TryCompleteLoad(recommend, handle)
end

-- 加载完成
function GameRecommender:_TryCompleteLoad(recommend, handle)

    -- 判断是否加载完了所有的内容
    for key,value in pairs(GameRecommender.DownloadFileType) do

        if handle.is_download_file_loaded[key] ~= true then
            return
        end
    end

    -- 调用成功回调
    if handle.callback ~= nil then

        -- 记录最近一次推荐内容
        self.last_recommend_game_id = recommend.data.game_id

        handle.callback(recommend.data.game_id)
    end
end

-- 返回当前任务是否可接;
function GameRecommender:CanAcceptTask()

    local task_game_id = sDatabase.data.game_recommender.recommend.current_game_id
    if not sDatabase.data.game_recommender.completed_recommend_tasks[task_game_id] then
        return true
    end
    return false
end

-- 点击图标接任务;
function GameRecommender:AcceptTask()

    -- if true then return end
    local task_game_id = sDatabase.data.game_recommender.recommend.current_game_id
    -- print("!!!!!!!!!!!!!! AcceptTask", task_game_id)
    if not sDatabase.data.game_recommender.completed_recommend_tasks[task_game_id] and not sDatabase.data.game_recommender.recommend_tasks[task_game_id] then        
        if task_game_id and sDatabase.data.game_recommender.recommend[task_game_id].data then
            sDatabase.data.game_recommender.recommend_tasks[task_game_id] = sDatabase.data.game_recommender.recommend[task_game_id].data
        end
    end
end

-- 返回可完成任务;
function GameRecommender:GetCompleteTaskGameID()
    
    for key, game_info in pairs(sDatabase.data.game_recommender.recommend_tasks) do
        local app_exist = true
		if sDatabase.data.game_recommender.recommend[key] and sDatabase.data.game_recommender.recommend[key].data then
			local game_info = sDatabase.data.game_recommender.recommend[key].data
			if device.platform == "ios" then
				app_exist = sDeviceUtils:IsAppExist(game_info.package_name_ios)
				-- print("!!!!!!!!!!!!!! GetCompleteTaskGameID", key, game_info.package_name_ios, app_exist, device.platform)
			else
				app_exist = sDeviceUtils:IsAppExist(game_info.package_name_android)
				-- print("!!!!!!!!!!!!!! GetCompleteTaskGameID", key, game_info.package_name_android, app_exist, device.platform)
			end
			if app_exist == true then
				return key
			end
		end
    end
    return nil
end

-- 点击奖励完成任务;
function GameRecommender:CompleteTask(task_game_id)

    -- print("!!!!!!!!!!!!!! CompleteTask", task_game_id)
    sDatabase.data.game_recommender.recommend_tasks[task_game_id] = nil
    sDatabase.data.game_recommender.completed_recommend_tasks[task_game_id] = task_game_id
end

function GameRecommender:GetReward()

    return "point/50"
end

return GameRecommend

--endregion
