-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

dxm.CMessageSystem = { }

local CMessageSystem = dxm.CMessageSystem
cc.exports.sMessageSystem = CMessageSystem

--[[
message={
    xxx=xxx...
    sender={
        user_id = xx,
        user_name = xx,
        user_info = {},
    },
    content={
        action_type = "",
        action_data = {},
    }
}
]]--

function CMessageSystem:Init()

    self.message_db = sDatabase.data.message_system

    self.message_db.player_message = self.message_db.player_message or { }
    self.message_db.player_message_id = self.message_db.player_message_id or 0

    self.message_db.world_message = self.message_db.world_message or { }
    self.message_db.world_message_id = self.message_db.world_message_id or 0

    self.sync_player_message_time = nil

    self.message_confirm_proc = { }

    self.remove_player_message_callback = nil
end

function CMessageSystem:HasNewPlayerMessage()
    for _,value in pairs(self.message_db.player_message) do
        return true
    end
    return false
end

function CMessageSystem:RegisterPlayerMessageConfirmProc(action_type, callback, auto_confirm)
    self.message_confirm_proc[action_type] = { callback = callback, auto_confirm = auto_confirm }
end

function CMessageSystem:SendPlayerMessage(target_user_id, action_type, action_data, callback)
    local message = json.encode({
        action_type = action_type,
        action_data = action_data
    }) 
    sHttpUser:SendPlayerMessage(target_user_id, message, callback)
end

function CMessageSystem:ConfirmPlayerMessage(message_id)

    local message = self.message_db.player_message[tostring(message_id)]
    if message == nil then
        return
    end

    local proc = self.message_confirm_proc[message.content.action_type]
    if proc ~= nil and proc.callback ~= nil then
        proc.callback(message)
    end

    self:RemovePlayerMessage(message_id)
end

function CMessageSystem:RemovePlayerMessage(message_id)
    self.message_db.player_message[tostring(message_id)] = nil

    if self.remove_player_message_callback ~= nil then
        self.remove_player_message_callback()
    end

end

function CMessageSystem:GetAllPlayerMessage(callback)

    local current_time = os.time()
    if self.sync_player_message_time == nil or current_time - self.sync_player_message_time > 10 then
        self.sync_player_message_time = current_time

        -- 从服务器获取最新的消息
        print("CMessageSystem:GetAllPlayerMessage", self.message_db.player_message_id)
        sHttpUser:GetPlayerMessage(self.message_db.player_message_id, 100, function(code, data)

            if code == 0 then

                -- dump(data)

                -- 将数据全部写入到本地
                local max_player_message_id = self.message_db.player_message_id
                for _, value in pairs(data.player_message) do

                    -- 在线数据中的json部分翻译成table
                    value.message_id = tonumber(value.message_id)
                    -- value.sender = json.decode(value.sender)
                    value.content = json.decode(value.content)

                    -- 判断是否自动确认而无需在邮件中显示
                    local proc = self.message_confirm_proc[value.content.action_type]
                    if proc ~= nil and proc.auto_confirm then
                        if proc.callback ~= nil then
                            proc.callback(value)
                        end
                    else

                        -- 把过去存在的同类数据摒弃掉
                        for _, message in pairs(self.message_db.player_message) do

                            if message.content ~= nil and value.content ~= nil and
                                message.content.action_type == value.content.action_type and
                                message.sender ~= nil and value.sender ~= nil and
                                message.sender.user_id ~= nil and 
                                message.sender.user_id == value.sender.user_id then
                                self.message_db.player_message[tostring(message.message_id)] = nil
                            end
                        end

                        -- 插入最新数据
                        self.message_db.player_message[tostring(value.message_id)] = value
                    end

                    if max_player_message_id < value.message_id then
                        max_player_message_id = value.message_id
                    end
                end

                -- 更新数据索引
                self.message_db.player_message_id = max_player_message_id

                -- 执行回调
                if callback ~= nil then
                    callback()
                end
            end
        end )
    else

        if callback ~= nil then
            callback()
        end
    end
end

--------------------------------------------------------------------------------
function CMessageSystem:GetAllWorldMessage()
end

function CMessageSystem:UpdateWorldMessage(dt)
end

return CMessageSystem

-- endregion
