-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

dxm.CUpdateSystem = { }
cc.exports.sUpdateSystem = dxm.CUpdateSystem

local CUpdateSystem = dxm.CUpdateSystem

CUpdateSystem.update_state_string ={
    "wait_init",
    "wait_start",
    "wait_get_version_manifest",
    "get_version_manifest",
    "wait_get_files_manifest",
    "get_files_manifest",
    "wait_download_patch",
    "download_patch",
    "wait_install",
}

CUpdateSystem.update_state = {}
for key, value in pairs(CUpdateSystem.update_state_string) do
    CUpdateSystem.update_state[value] = key
end

-- 默认为 wait_init;
CUpdateSystem.state = CUpdateSystem.update_state.wait_init

function CUpdateSystem:Init()   

    self.patch_list = { }
    self.downloaded_list = {}

    -- download patch files process
    self.total_size = 0
    self.downloaded_size = 0

    self.download_version_manifest = nil
    self.download_files_manifest = nil

    require "dxm.init"
    print("update system: init")
    self:_SetState(CUpdateSystem.update_state.wait_init, 100, 100)       
    self:_CheckRemovePatch()
    self:_CheckInstall()
    self:_LoadWithPatch()

    self.slient_update_scheduler = dxm.CTimeScheduler.new()
    self:SlientUpdate()
    self.slient_update_scheduler:AddUpdate(600, os.time(), function(times) self:SlientUpdate() end)
    cc.Director:getInstance():getScheduler():scheduleScriptFunc(function(dt)
		self.slient_update_scheduler:Update(dt)
	end, 0, false)
end

-- 检测是否已经存在新版本, 如存在则弹出"检测到新版本"
function CUpdateSystem:HasNewVersion()

    --[[
    if self.state == CUpdateSystem.update_state.wait_install then
        return self:_CheckInstallFlag()
    end
    ]]--
    return self.state == CUpdateSystem.update_state.wait_install
end

function CUpdateSystem:GetLocalTimestamp()

    if self.state == CUpdateSystem.update_state.wait_install then
        return self:_CheckInstallFlag()
    end
end

function CUpdateSystem:GetRemoteTimestamp()

    if self.state == CUpdateSystem.update_state.wait_install then
        return self:_CheckInstallFlag()
    end
end

-- ----------------
-- 静默更新;
function CUpdateSystem:SlientUpdate()

    print("update system: slient update")
    self.slient_update_start_tick = os.time()
    local successful = self:StartUpdate(function(state, done, total)
            -- print("update system: slient update", state, done, total)
            if state == CUpdateSystem.update_state.wait_get_version_manifest then
                self:GetRemoteVersionManifest()
            elseif state == CUpdateSystem.update_state.wait_get_files_manifest then
                self:GetRemoteFilesMinifest()
            elseif state == CUpdateSystem.update_state.wait_download_patch then
                self:DownloadPatch()
            end
        end)

    if not successful then
        return false
    end
    return true
end


-- ---------------
-- 设置状态并执行回调;
function CUpdateSystem:_SetState(state, done, total)
    self.state = state
    self.percent = done/total*100    
    if self.callback then self.callback(self.state, done, total) end
end

-- -----------------------
-- APK 版本高于 PATCH 版本的时候清理 PATCH;
function CUpdateSystem:_CheckRemovePatch()

    self:_LoadWithoutPatch() 
    sLocale:SetLocale() -- 本地化目录搜索;
    self.local_files_list = sJsonUtils:Load(self.files_path)
    self.local_version = sJsonUtils:Load(self.version_path)
    self.local_version.timestamp = self.local_version.Timestamp or 0
    sFileUtils:RemoveDirectory(self.manifest_save_path)
    if self.app_version.timestamp > self.local_version.timestamp then
        print("update system: remove patch (app:, ", self.app_version.timestamp, ",patch:", self.local_version.timestamp, ")")
        sFileUtils:RemoveDirectory(self.cache_save_path)
        sFileUtils:RemoveDirectory(self.patch_save_path)        
        self.local_version = self.app_version
    end
end

-- -----------------------
-- 加载非 PATCH 配置;
function CUpdateSystem:_LoadWithoutPatch()

    local branch_name = sLocale:GetBranchName()  
    
    -- --------------
    -- >> search_paths: branched/branch/; branched/branch/src/; src/ 
    local search_paths = {}
    -- >> ""
    if branch_name then search_paths[#search_paths+1] = "branches/"..branch_name.."/" end
    -- search_paths[#search_paths+1] = ""
    
    -- >> "src/"
    if branch_name then search_paths[#search_paths+1] = "branches/"..branch_name.."/src/" end
    search_paths[#search_paths+1] = "src/" 
    cc.FileUtils:getInstance():setSearchPaths(search_paths)

    self.version_path = "version.manifest"
    self.files_path = "files.manifest"
    self.cache_save_path = device.writablePath .. "cache/"
    self.manifest_save_path = device.writablePath .. "manifest/"
    self.patch_save_path = device.writablePath .. "patch/"

    self.app_version = sJsonUtils:Load(self.version_path)
    self.app_version.timestamp = self.app_version.Timestamp or 0
end

-- -----------------------
-- 加载 PATCH 配置
function CUpdateSystem:_LoadWithPatch()

    -- 本地版本配置和文件列表
    self.local_files_list = sJsonUtils:Load(self.files_path)
    self.local_version = sJsonUtils:Load(self.version_path)
    self.local_version.timestamp = self.local_version.Timestamp or 0
    sFileUtils:RemoveDirectory(self.manifest_save_path)

    -- 检测配置是否错误;
    if not self.local_version.RemotePackageUrl then
        print("update system: download version manifest RemotePackageUrl is not exist")
        return
    end
    if not self.local_version.RemoteVersionManifest then
        print("update system: download version manifest RemoteVersionManifest is not exist")
        return
    end
    if not self.local_version.RemoteFilesManifest then
        print("update system: download version manifest RemoteFilesManifest is not exist")
        return
    end
    if not self.local_files_list.assets then
        print("update system: download files manifest assets is not exist")
        return
    end

    -- 检测安装;
    if self:_CheckInstall() then
        cc.FileUtils:getInstance():purgeCachedEntries() -- 清理缓存
        self.local_version = sJsonUtils:Load(self.version_path) -- 读取本地版本
        self.local_version.timestamp = self.local_version.Timestamp or 0
    end

    self.remote_package_url = self.local_version.RemotePackageUrl
    self.remote_version_manifest = self.local_version.RemoteVersionManifest
    self.remote_files_manifest = self.local_version.RemoteFilesManifest
    self.local_version_code = self.local_version.Version
    self.timestamp = self.local_version.Timestamp or 0
    self.files_manifest_md5 = self.local_version.FilesManifestMd5
    self:_SetState(CUpdateSystem.update_state.wait_start, 100, 100)
end

-- -----------------------
-- CheckInstall
function CUpdateSystem:_CheckInstall()

    if self:_CheckInstallFlag() then
        local downloaded_list = sJsonUtils:Load(self.cache_save_path .. "/downloaded_list.json")
        for k, patch_file in pairs(downloaded_list) do 
            self.downloaded_list[#self.downloaded_list+1] = patch_file
        end

        self.downloaded_list[#self.downloaded_list+1] = {from=self.cache_save_path .. self.files_path, to=self.patch_save_path .. self.files_path}
        self.downloaded_list[#self.downloaded_list+1] = {from=self.cache_save_path .. self.version_path, to=self.patch_save_path .. self.version_path}
        for k, patch_file in pairs(self.downloaded_list) do
            local file = sFileUtils:Read(patch_file.from)
            if file then
                file:Write(patch_file.to)
                print("update system: patch downloaded ", patch_file.from)
                print("update system: patch downloaded write to ", patch_file.to)
            end
        end
        sFileUtils:RemoveDirectory(self.cache_save_path)
        sFileUtils:RemoveDirectory(self.manifest_save_path)
        self:_SetInstallFlag(false)

        -- 清理并重载配置;
        for k, v in pairs(package.loaded) do
            
            -- update_system 不重新加载, 否则 Init 后的数据会被置空而导致此次启动无法静默更新
            if k ~= "dxm.system.update_system" then
                local sub = string.sub(k, 1, 3)
                if sub == "dxm" or sub == "app" or sub == "con" then
                    package.loaded[k] = nil
                end
            end
        end
        require "dxm.init"
        sSpineUtils:Clear()
        cc.FileUtils:getInstance():purgeCachedEntries() 
        self.local_version = sJsonUtils:Load(self.version_path)
        self.local_version.timestamp = self.local_version.Timestamp or 0
        return true
    end

    return false
end

function CUpdateSystem:_SetInstallFlag(install)

    if install then 
        sJsonUtils:Save(self.cache_save_path .. "/downloaded_list.json", self.downloaded_list)
    else
        self.downloaded_list = {}
    end

    local install_flag = {}
    install_flag.install = install
    sJsonUtils:Save(self.cache_save_path .. "/install_flag.json", install_flag)
end

function CUpdateSystem:_CheckInstallFlag()

    local install_flag = sJsonUtils:Load(self.cache_save_path .. "/install_flag.json")
    return install_flag.install
end

-- --------------------
-- 1. 开始更新, 初始化变量;
function CUpdateSystem:StartUpdate(callback)

    if self.state ~= CUpdateSystem.update_state.wait_start then
        return false
    end
    print("update system: start")
    sFileUtils:RemoveDirectory(self.manifest_save_path)

    self.callback = callback

    self.patch_list = { }
    self.downloaded_list = {}

    -- download patch files process
    self.total_size = 0
    self.downloaded_size = 0

    self.download_version_manifest = nil
    self.download_files_manifest = nil

    self:_SetState(CUpdateSystem.update_state.wait_get_version_manifest, 100, 100)
    return true
end

-- ----------------
-- 2. 获取版本清单
function CUpdateSystem:GetRemoteVersionManifest()    

    print("update system: get remote version manifest")
    if self.state ~= CUpdateSystem.update_state.wait_get_version_manifest then
        return 
    end 

    self:_SetState(CUpdateSystem.update_state.get_version_manifest, 0, 100)
    self:_DownloadVersionManifest()
end

-- ----------------
-- 3. 获取文件清单
function CUpdateSystem:GetRemoteFilesMinifest()
    
    print("update system: get remote files manifest")
    if self.state ~= CUpdateSystem.update_state.wait_get_files_manifest then
        return 
    end

    self:_SetState(CUpdateSystem.update_state.get_files_manifest, 0, 100)
    self:_DownloadFilesManifest()
end

-- ----------------
-- 4. 下载补丁
function CUpdateSystem:DownloadPatch()
    
    print("update system: download patch")
    if self.state ~= CUpdateSystem.update_state.wait_download_patch then return end

    self:_SetState(CUpdateSystem.update_state.download_patch, 0, 100)
    self:_DownloadPatchStart()
end

-- --------------
-- 5. 结束, 重新准备开始;
function CUpdateSystem:_Done()

    print("update system: download done")
    self:_SetState(CUpdateSystem.wait_start, 100, 100)
end

function CUpdateSystem:_DownloadVersionManifest()

    print("update system: download version manifest")
    sDownloadUtils:Download(self.remote_version_manifest, self.manifest_save_path, function(state, result) 

        print("update system: download version manifest", result.bytes_done, result.bytes_total)
        if state == kDownloading then
            self:_SetState(self.state, result.bytes_done, result.bytes_total)
        elseif state == kDownloadFailed then
            print("update system: download version manifest failed")
            self:_Done()
        elseif state==kDownloadFinish then            
            local file = sFileUtils:Read(result.file_path)
            if not file:CheckValid() then
                -- file is not valid download again;    
                sFileUtils:RemoveFile(result.file_path)           
                self:_DownloadVersionManifest()
            else
                print("update system: download version manifest successful")
                -- md5 is diff to download manifest
                self:_SetState(self.state, result.bytes_done, result.bytes_total)
                
                self.download_version_manifest = result.file_path
                local version_data = sJsonUtils:Load(result.file_path)
                local remote_timestamp = version_data.Timestamp
                local remote_version_code = version_data.Version
                if self.local_version_code and remote_version_code and remote_version_code ~= self.local_version_code then
                    print("update system: download version version code is not same") 
                    self:_Done()
                elseif self.timestamp > remote_timestamp then
                    -- version large than remote_version is test
                    test_version = true
                    print("update system: download version manifest timestamp is test")             
                    self:_Done()
                elseif self.timestamp < remote_timestamp or version_data.FilesManifestMd5 ~= self.files_manifest_md5 then
                    print("update system: download version manifest md5 not same")
                    self:_SetState(CUpdateSystem.update_state.wait_get_files_manifest, 100, 100)
                else
                    print("update system: download version manifest version and md5 same")
                    file:Write(self.patch_save_path .. self.version_path) 
                    self:_Done()
                end
            end
        end
    end) 
end

function CUpdateSystem:_DownloadFilesManifest()

    sDownloadUtils:Download(self.remote_files_manifest, self.manifest_save_path, function(state, result) 
        if state == kDownloading then
            self:_SetState(self.state, result.bytes_done, result.bytes_total)
        elseif state == kDownloadFailed then
            self:_DownloadFilesManifest()
        elseif state==kDownloadFinish then
            local file = sFileUtils:Read(result.file_path)
            if not file:CheckValid() then
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadFilesManifest()
            else             
                self:_SetState(self.state, result.bytes_done, result.bytes_total)
                self.download_files_manifest = result.file_path
                self.remote_files_list = sJsonUtils:Load(result.file_path)               
                self:_GetPatchList()
                local total_patch = #self.patch_list
                if total_patch == 0 then
                    self:_Done()
                else
                    self:_SetState(CUpdateSystem.update_state.wait_download_patch, 100, 100)   
                end
            end
        end
    end) 
end

function CUpdateSystem:_GetPatchList()

    print("update system: check patch to download")
    local local_files_map = {}
    local remote_files_map = {}
                    
    if self.local_files_list and  self.local_files_list.assets then
        for _, file_info in pairs(self.local_files_list.assets) do
            local_files_map[file_info.file] = file_info
        end
    end
    if self.remote_files_list and  self.remote_files_list.assets then
        for _, file_info in pairs(self.remote_files_list.assets) do
            remote_files_map[file_info.file] = file_info
        end
    end

    for key, remote_file_info in pairs(remote_files_map) do
        local local_file_info = local_files_map[key]
        if not local_file_info then
            print("update system: check file not exist", remote_file_info.file)
            local_file_info = {id=remote_file_info.id, file=remote_file_info.file, md5="asdasda", size=0}
            self.local_files_list[#self.local_files_list+1] = local_file_info
            self.patch_list[#self.patch_list+1] = {local_file_info = local_file_info, remote_file_info = remote_file_info}
            self.total_size = self.total_size + remote_file_info.size
        elseif local_file_info.md5 ~= remote_file_info.md5 then
            print("update system: check file diff md5", remote_file_info.file)
            self.patch_list[#self.patch_list+1] = {local_file_info = local_file_info, remote_file_info = remote_file_info}
            self.total_size = self.total_size + remote_file_info.size
        end
    end
end


--获取路径  
local function stripfilename(filename)  
    return string.match(filename, "(.+)/[^/]*%.%w+$") --*nix system  
    --return string.match(filename, “(.+)\\[^\\]*%.%w+$”) — windows  
end  
  
--获取文件名  
local function strippath(filename)  
    return string.match(filename, ".+/([^/]*%.%w+)$") -- *nix system  
    --return string.match(filename, “.+\\([^\\]*%.%w+)$”) — *nix system  
end  
  
--去除扩展名  
local function stripextension(filename)  
    local idx = filename:match(".+()%.%w+$")  
    if(idx) then  
        return filename:sub(1, idx-1)  
    else  
        return filename  
    end  
end  

function CUpdateSystem:_DownloadPatch(pos)

    -- collectgarbage("collect")
    local total_patch = #self.patch_list
    if pos > total_patch then
        return false
    end

    local patch_info = self.patch_list[pos]
    local download_url = self.remote_package_url .. patch_info.remote_file_info.file
    local rpath = stripfilename(patch_info.remote_file_info.file)
    rpath = rpath or ""
    local save_path = self.cache_save_path .. rpath
    -- print("update system: download", pos, download_url)
    sDownloadUtils:Download(download_url, save_path, function(state, result)

        if state == kDownloadStart then
            print("update system: download "..pos.."/"..self.patch_total, download_url)
        elseif state == kDownloading then

            self:_SetState(self.state, self.downloaded_size + result.bytes_done, self.total_size)
        elseif state == kDownloadFailed then

            sFileUtils:RemoveFile(result.file_path)
            self:_DownloadPatch(pos)
            print("update system: download failed")
        elseif state == kDownloadFinish then
            
            local file = sFileUtils:Read(result.file_path)
            if not file then
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadPatch(pos)
            elseif file:GetMD5() ~= patch_info.remote_file_info.md5 then
                -- file is not valid download again;
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadPatch(pos)
            else
                -- md5 is diff to download manifest;
                local patch_save_path = self.patch_save_path .. patch_info.remote_file_info.file                
                self.downloaded_list[#self.downloaded_list+1] = {from=result.file_path, to=patch_save_path}
                patch_info.local_file_info.md5 = patch_info.remote_file_info.md5
                patch_info.local_file_info.size = patch_info.remote_file_info.size
                
                self.downloaded_size = self.downloaded_size + patch_info.remote_file_info.size
                self:_SetState(self.state, self.downloaded_size + result.bytes_done, self.total_size)
                self:_DownloadPatchFinish(pos)
            end
        end
    end )
    return true
end

function CUpdateSystem:_DownloadPatchStart()

    self:_SetInstallFlag(false)
    self.patch_total = #self.patch_list
    if self.patch_total==0 then
        self:_DownloadFinish()
        return
    end

    for i=1, self.patch_total do
        self:_DownloadPatch(i)
    end
end

function CUpdateSystem:_DownloadPatchFinish()
    self.patch_downloaded = self.patch_downloaded or 0
    self.patch_downloaded = self.patch_downloaded + 1
    if self.patch_downloaded == self.patch_total then
        self:_DownloadFinish()
    end
end

function CUpdateSystem:_DownloadPatchNext(pos)

    if not self:_DownloadPatch(pos + 1) then
        self:_DownloadFinish()
    end
end

function CUpdateSystem:_DownloadFinish()

    print("update system: download finish wait to install", os.time()-self.slient_update_start_tick)
    local file = sFileUtils:Read(self.download_version_manifest)
    if file then file:Write(self.cache_save_path .. self.version_path) end
    file = sFileUtils:Read(self.download_files_manifest)
    if file then file:Write(self.cache_save_path .. self.files_path) end
    self:_SetInstallFlag(true)
    self:_SetState(CUpdateSystem.update_state.wait_install, 100, 100)
end

return CUpdateSystem 

-- endregion
