-- region NewFile_1.lua
-- Author : IBM
-- Date   : 2015/4/27
-- 此文件由[BabeLua]插件自动生成



dxm.CBaseUI = class("CBaseUI")

local CBaseUI = dxm.CBaseUI

CBaseUI.WidgetType =
{
    invalid = 0,
    -- 无效的类型
    config = 1,
    -- 从配置中获取的widget
    custom = 2,
    -- 不含parent的widget, 该widget在被清理前需要RemoveFromParent
    part = 3,
    -- 含parent的widget, 该widget在被清理前不需要RemoveFromParent
    max = 4,
}

-- 构造函数
function CBaseUI:ctor()
    self.widget = nil
    -- widget_对象
    self.node = nil
    -- 配置节点
    self.widget_type = CBaseUI.WidgetType.invalid
    -- widget_的类型
    self.widget_config = ""
    -- widget配置名称;
    self.parent_base_ui = nil
    -- 父CBaseUI;
    self.parent_widget = nil
    -- 父widget;
    self.ui_children = { }
    -- 挂接在本UI上的其他UI; ui_children_[widget] = base_ui_list
    self.is_entered = false
    -- 是否开启
    self.position = nil
    -- 事件
    self.event_enter = dxm.CEvent.new()
    self.event_exit = dxm.CEvent.new()
    -- 
    self.play_open_animation = true
    self.open_ui_lua_action_enable = false
end

function CBaseUI:Root()
    local root = self.node or self.widget
    return root
end

-- 析构函数
function CBaseUI:Dispose()
    self:_ClearUI()

    -- 析构widget;
    if self.widget ~= nil then
        self.widget:release()
        self.widget = nil
    end

    if self.node ~= nil then
        self.node:release()
        self.node = nil
    end

    if self.action ~= nil then
        self.action:release()
        self.action = nil
    end
end

-- 初始化函数
-- Init(nil)
-- Init(file_name)
-- Init(widget)
function CBaseUI:Init(param)
    
    -- 重载字符串和节点函数;
    if param ~= nil then
        if tolua.type(param) == "string" then
            return self:InitWithFile(param)
        else
            return self:InitWithWidget(param)
        end
    end

    -- 已初始化;
    if self.widget ~= nil then
        return false
    end
    self:set_widget(ccui.Widget:create())
    return true
end

function CBaseUI:InitWithWidget(widget)
    -- 界面参数无效
    if widget == nil then
        return false
    end
    -- 已初始化;
    if self.widget ~= nil then
        return false
    end
    self:set_widget(widget)
    return true
end

function CBaseUI:InitWithFile(config)
    -- 已初始化;
    if self.widget ~= nil then
        return false;
    end

    local file = dxm.CLocale:GetUIFile(config)
    if not cc.FileUtils:getInstance():fullPathForFilename(file) then
        return false
    end

    self.widget_config = config
    self.widget_type = CBaseUI.WidgetType.config
    return true
end

-- 创建函数
function CBaseUI:Create(...)
    local ui = self.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

-- 重置widget
function CBaseUI:set_widget(widget)

    -- 已经显示的界面无法重置 widget;
    if self.widget == widget or self.is_entered then
        return
    end
    
    -- 清理所有的子UI;
    self:RemoveAllChildren()

    -- 清理界面;
    self:_ClearUI()

    -- 重置新界面;
    self.widget = widget
    if widget == nil then
        self.widget_type = CBaseUI.WidgetType.invalid
        return
    end

    -- 添加引用;
    self.widget:retain()

    -- 看看是不是有父节点;
    if self.widget:getParent() ~= nil then
        self.widget_type = CBaseUI.WidgetType.part
    else
        self.widget_type = CBaseUI.WidgetType.custom
    end
end

-- 未显示的节点获取界面大小, 应用场景为滚动层上滚动条的大小
function CBaseUI:GetContentSize()    
    
    if self.content_size == nil or self.last_widget_type ~= self.widget_type or self.last_widget_config ~= self.widget_config then
        self.content_size = { width=0, height=0 }
        if self.widget==nil then
            if self.widget_type == CBaseUI.WidgetType.config then
                self:_InitConfigWidget()
                self:_ClearUI()
            end
        else
            self.content_size.width = self.widget:getContentSize().width
            self.content_size.height = self.widget:getContentSize().height
        end
        self.last_widget_type = self.widget_type
        self.last_widget_config = self.widget_config
    end
    return self.content_size
end

-- 清理界面;
function CBaseUI:_ClearUI()

    if self.action and self.widget_type == CBaseUI.WidgetType.config then
        self.node:stopAction(self.action)
    end

    self:RemoveAllChildren()
    self:RemoveFromParent()

    -- 回收物理widget;
    if self.node and self.widget_type == CBaseUI.WidgetType.config then
        
        -- 回收缓存节点;
        if self.cache_node then 
            dxm.CWidgetCacheManager:GetInstancePtr():AddWidget(self.cache_node)
            self.cache_node = nil
        end

        -- 析构widget;
        if self.widget ~= nil then
            self.widget:release()
            self.widget = nil
        end

        if self.node ~= nil then
            self.node:release()
            self.node = nil
        end

        if self.action ~= nil then
            self.action:release()
            self.action = nil
        end
    end
end

function CBaseUI:_InitConfigWidget()
    -- 创建界面一枚;
    local cache_node = dxm.CWidgetCacheManager:GetInstancePtr():GetWidget(self.widget_config)
    if cache_node == nil then
        return
    end

    self.cache_node = cache_node

    local action = cache_node.action
    local node = cache_node.widget

    -- local widget = node:getChildByTag(5)
    local widget = node
    node:runAction(action)

    -- 初始化界面;
    widget:setVisible(true)
    widget:setOpacity(255)
    widget:setColor(cc.c3b(255, 255, 255))
    widget:setAnchorPoint(cc.p(0, 0))
    widget:setPosition(cc.p(0, 0))
    widget:setScale(1.0)
    widget:setRotation(0)
    node:setPosition(cc.p(0, 0))

    -- 设置界面;
    self.widget = widget
    self.widget:retain()
    self.node = node
    self.node:retain()
    self.action = action
    self.action:retain()

    -- 获取界面大小;
    self.content_size = self.content_size or { width=0, height=0 }
    self.content_size.width = self.widget:getContentSize().width
    self.content_size.height = self.widget:getContentSize().height
    self.last_widget_type = self.widget_type
    self.last_widget_config = self.widget_config
end

function CBaseUI:SetPosition(x, y)

    self.position = self.position or {}
    self.position.x = x
    self.position.y = y
    if self.is_entered ==true then
        self:Root():setPosition(cc.p(self.position.x, self.position.y))
    end
end

function CBaseUI:_SetUIStrings(widget_config)    

    if widget_config=="" then return end

    local k = widget_config
    local find_begin = 1
    local find_end = string.find(k, "/")
    local last_name = nil
    while find_end do        
        last_name = string.sub(k, find_begin, find_end-1)
        find_begin = find_end+1
        find_end = string.find(k, "/", find_begin)
    end

    last_name = string.sub(k, find_begin)
    if not last_name then return end
 
    local ui_strings = assets.strings:GetUIStrings(last_name)
    if not ui_strings then return end

    for k, v in pairs(ui_strings) do
        local widget_tree = v.widget_tree
        local key = v.key
        
        local temp_widget = self.widget
        for _, v in ipairs(widget_tree) do
            temp_widget = ccui.Helper:seekWidgetByName(temp_widget, v)
            if temp_widget == nil then
                break
            end
        end

        if temp_widget then 
            local widget_type = tolua.type(temp_widget)
            if widget_type == "ccui.Text" then
                temp_widget:setString(assets.strings:GetString(key)) 
            elseif widget_type == "ccui.Button" then
                temp_widget:setTitleText(assets.strings:GetString(key))
            elseif widget_type == "ccui.TextField" then
                temp_widget:setPlaceHolder(assets.strings:GetString(key))
            end
        end      
    end
end

function CBaseUI:_DoUIScript(widget_config)

    if widget_config=="" then return end

    local k = widget_config
    local find_begin = 1
    local find_end = string.find(k, "/")
    local last_name = nil
    while find_end do
        last_name = string.sub(k, find_begin, find_end - 1)
        find_begin = find_end + 1
        find_end = string.find(k, "/", find_begin)
    end

    last_name = string.sub(k, find_begin)
    if not last_name then return end

    local widget_scripts = assets.scripts:GetUIScripts(last_name)
    if not widget_scripts then return end

    for k, v in pairs(widget_scripts) do
        local widget_tree = v.widget_tree
        local key = v.key

        local temp_widget = self.widget
        for _, v in ipairs(widget_tree) do
            temp_widget = ccui.Helper:seekWidgetByName(temp_widget, v)
            if temp_widget == nil then
                break
            end
        end

        if temp_widget then
            local f = loadstring(key)
            cc.exports.self_ui = self
            f()
        end
    end

end

-- 打开与关闭事件
function CBaseUI:_Enter()

    if self.is_entered == true then
        return
    end

    -- _TypeAddRef
    self:_TypeAddRef()
    self.is_entered = true

    -- 设置位置
    if self.position ~=nil then
        self:Root():setPosition(cc.p(self.position.x, self.position.y))
    end
    
    -- 设置界面文字
    self:_SetUIStrings(self.widget_config)
    self:_DoUIScript(self.widget_config)

    -- 执行配置回调
    self:onEnter()
    self.event_enter:Fire()

    if self.play_open_animation then      
        self:PlayOpenAnimation()
    end
end

function CBaseUI:_Exit()

    if self.is_entered == false then
        return
    end

    -- _TypeDecRef
    self:_TypeDecRef()

    --移除消息监听
    self:RemoveAllListener()

    -- 执行配置回调
    self:onExit()

    -- 清理
    self:_ClearUI()

    -- set entered
    self.is_entered = false

    self:UnlockTouch()

    if self.base_ui_update_entry then
        -- print(">>>>>>>>>>>>>>>>>> xx", self.base_ui_update_entry)
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.base_ui_update_entry)
        self.base_ui_update_entry = nil
    end

    self.event_exit:Fire()
    self.event_exit:DisconnectAll()
end

function CBaseUI:PlayOpenAnimation()

    if self.open_ui_lua_action_enable then        
        sActionUtils:UIPopup(self.widget)
    else
        self:PlayAnimation("OpenAnimation", false) 
    end      
end

function CBaseUI:LockTouch()

    if self.touch_locking==true then
        return
    end

    self.touch_locking = true
    sUI:LockTouch()  
end

function CBaseUI:UnlockTouch()

    if self.touch_locking==true then
        self.touch_locking = false
        sUI:UnlockTouch()
    end
end

function CBaseUI:onEnter()
    -- 虚函数
end

function CBaseUI:onExit()
    -- 虚函数
end

function CBaseUI:ScheduleUpdate(callback, seconds)
    seconds = seconds or 0
    if self.base_ui_update_entry then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.base_ui_update_entry)
        self.base_ui_update_entry = nil
    end
    if callback then
        self.base_ui_update_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function(dt) 
            callback(dt)
        end, seconds, false)
        -- print(">>>>>>>>>>>>>>>>>> oo", self.base_ui_update_entry)
    end
end

-- 子节点托管操作
-- AddChild(ui, zorder)
-- AddChild("widget_name", ui, zorder)
-- AddChild(widget, ui, zorder)
function CBaseUI:AddChild(...)

    local widget_name
    local base_ui
    local zorder
    local arg = {...}
    if type(arg[1]) == "table" then
        self:AddChildWithWidget(self.widget, arg[1], arg[2])
    elseif type(arg[1]) == "string" then 
        self:AddChildWithWidgetName(arg[1], arg[2], arg[3])
    else
        self:AddChildWithWidget(arg[1], arg[2], arg[3])
    end
end

function CBaseUI:AddChildWithWidgetName(widget_name, base_ui, zorder)

    zorder = zorder or 0

    if self.widget == nil then
        return
    end

    local widget = ccui.Helper:seekWidgetByName(self.widget, widget_name)
    if widget == nil then
        return
    end

    self:AddChildWithWidget(widget, base_ui, zorder)
end

function CBaseUI:AddChildWithWidget(widget, base_ui, zorder)

    if self.is_entered == false or widget == nil or base_ui == nil or base_ui.parent_base_ui ~= nil then
        return
    end

    -- 界面中的层级;
    zorder = zorder or 0

    -- 初始化配置界面;
    if base_ui.widget_type == CBaseUI.WidgetType.config then
        base_ui:_InitConfigWidget()
    end

    -- base_ui没有子节点挂接失败
    if base_ui.widget == nil then
        return
    end

    -- 创建链表;
    local base_ui_list = self:CreateList(widget)

    -- 判断是否已经存在;
    for _, temp_ui in pairs(base_ui_list) do
        if temp_ui == base_ui then
            return
        end
    end

    -- 正式挂接, part类型的不要做添加操作;
    if base_ui.widget_type ~= CBaseUI.WidgetType.part then
        widget:addChild(base_ui:Root(), zorder)
    end

    -- 添加到管理
    base_ui_list[base_ui] = base_ui
    base_ui:SetParent(self, widget)

    -- 父节点已经进入场景，子节点则执行 OnEnter
    base_ui:_Enter()
end

-- 删除节点上所有 UI;
-- RemoveAllChildren(nil)
-- RemoveAllChildren("widget_name", ...)
-- RemoveAllChildren(widget)
function CBaseUI:RemoveAllChildren(...)

    local arg = {...}
    if arg[1] ~= nil then
        if type(arg[1]) == "string" then
            self:RemoveAllChildrenWithWidgetName(...)
        else
            self:RemoveAllChildrenWithWidget(arg[1])
        end
        return
    end

    if self.widget == nil then
        return
    end

    -- 判断是否已经存在;
    for temp_widget, temp_ui_list in pairs(self.ui_children) do

        -- 清空链表;
        for _, temp_ui in pairs(temp_ui_list) do

            temp_ui_list[temp_ui] = nil
            if temp_ui.widget_type ~= CBaseUI.WidgetType.part then 
                temp_widget:removeChild(temp_ui.widget)
            end
            temp_ui:SetParent(nil, nil)

            -- 触发事件
            if self.is_entered then
                temp_ui:_Exit()
            end

            temp_ui:_ClearUI()
        end
    end
    self.ui_children = { }
end

function CBaseUI:RemoveAllChildrenWithWidgetName(...)

    if self.widget == nil then
        return
    end

    local widget = self:GetWidget(...)
    if widget == nil then
        return;
    end
    self:RemoveAllChildrenWithWidget(widget, base_ui)
end

function CBaseUI:RemoveAllChildrenWithWidget(widget)

    if widget == nil then
        return
    end

    -- 获取链表;
    local base_ui_list = self:GetList(widget)
    if base_ui_list == nil then
        return
    end

    -- 判断是否已经存在;
    for _, temp_ui in pairs(base_ui_list) do
        base_ui_list[temp_ui] = nil
        if temp_ui.widget_type ~= CBaseUI.WidgetType.part then
            if temp_ui.widget_type == CBaseUI.WidgetType.config then
                -- print("remove child " .. temp_ui.widget_config)
            end
            widget:removeChild(temp_ui.widget)
        end
        temp_ui:SetParent(nil, nil);
        if self.is_entered == true then
            temp_ui:_Exit()
        end

        temp_ui:Dispose()
        break
    end
    base_ui_list = {}
end

-- 删除 UI 
function CBaseUI:RemoveFromParent()

    if self.parent_base_ui ~= nil and self.parent_widget ~= nil then
        self.parent_base_ui:_RemoveChildWithWidget(self.parent_widget, self)
    end
end

function CBaseUI:RemoveChild(base_ui)

    base_ui:RemoveFromParent()
end

function CBaseUI:_RemoveChildWithWidget(widget, base_ui)

    if widget == nil then
        return
    end

    -- 获取链表;
    local base_ui_list = self:GetList(widget)
    if base_ui_list == nil then
        return
    end

    -- 判断是否已经存在;
    for _, temp_ui in pairs(base_ui_list) do
        if temp_ui == base_ui then

            base_ui_list[temp_ui] = nil
            if temp_ui.widget_type ~= CBaseUI.WidgetType.part then
                
                if temp_ui.widget_type == CBaseUI.WidgetType.config then
                    -- print("remove child " .. temp_ui.widget_config)
                end

                temp_ui.widget:removeFromParent()
            end
            temp_ui:SetParent(nil, nil);
            if self.is_entered == true then
                temp_ui:_Exit()
            end

            temp_ui:Dispose()
            break
        end
    end
end


function CBaseUI:CreateList(widget)

    local list = self.ui_children[widget]
    if list == nil then
        list = { }
        self.ui_children[widget] = list
    end
    return list
end

function CBaseUI:GetList(widget)
    return self.ui_children[widget]
end

function CBaseUI:SetParent(base_ui, widget)
    self.parent_base_ui = base_ui
    self.parent_widget = widget
end

-- 获取 Widget;
-- GetWidget("widget_name", ...)
-- GetWidget(widget, "widget_name", ...)
function CBaseUI:GetWidget(...)
    local arg = {...}
    if type(arg[1]) == "string" or arg[1] == nil then
        return self:_GetWidgetFromWidget(self.widget, ...)
    else
        return self:_GetWidgetFromWidget(...)
    end
end

function CBaseUI:_GetWidgetFromWidget(widget, ...)
    if widget == nil then
        return nil
    end

    local arg = {...}
    local temp_widget = widget
    for _, v in ipairs(arg) do
        -- temp_widget = temp_widget:getChildByName(v)
        temp_widget = ccui.Helper:seekWidgetByName(temp_widget, v)
        if temp_widget == nil then
            return nil
        end
    end
    return temp_widget
end

-- 设置显示性
-- SetWidgetVisible(is_visible, "widget_name", ...)
-- SetWidgetVisible(is_visible, widget, "widget_name", ...)
function CBaseUI:SetWidgetVisible(is_visible, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setVisible(is_visible)
    end
end

-- 设置有效性
-- SetWidgetEnable(is_visible, "widget_name", ...)
-- SetWidgetEnable(is_visible, widget, "widget_name", ...)
function CBaseUI:SetWidgetEnable(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setEnabled(is_enable)
    end
end

-- 设置可触摸性
-- SetWidgetTouchEnable(is_enable, "widget_name", ...)
-- SetWidgetTouchEnable(is_enable, widget, "widget_name", ...)
function CBaseUI:SetWidgetTouchEnable(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setTouchEnabled(is_enable)
    end
end

-- 设置有效性与显示性
-- SetWidgetEnableAndVisible(is_enable, "widget_name", ...)
-- SetWidgetEnableAndVisible(is_enable, widget, "widget_name", ...)
function CBaseUI:SetWidgetEnableAndVisible(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setEnabled(is_enable)
        widget:setVisible(is_enable)
    end
end

-- 设置图片
function CBaseUI:SetImageViewPicture(path, ...)
    local image_view = self:GetWidget(...)
    if image_view ~= nil then
        image_view:loadTexture(path)
        image_view:setContentSize(image_view:getVirtualRenderer():getContentSize())
    end
end

-- 设置文字
function CBaseUI:SetLabelText(text, ...)
    local label =self:GetWidget(...)
    if label ~= nil then
        label:setString(text)
    end
end

-- 设置Atlas文字
function CBaseUI:SetAtlasLabelText(text, ...)
    local label_atlas = self:GetWidget(...)
    if label_atlas ~= nil then
        label_atlas:setString(text)
    end
end

-- 设置进度条
function CBaseUI:SetLoadingbarPercent(percent, ...)
    local loading_bar = self:GetWidget(...)
    if loading_bar ~= nil then
        loading_bar:setPercent(percent)
    end
end

-- 设置触摸事件;
function CBaseUI:AddReleaseEvent(event_func, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then

        local widget_type = tolua.type(widget)
        if widget_type == "ccui.Button" then
        -- if false then
            sUI:ShakeClick(self, widget, function() 
                if event_func then
                    event_func()
                    -- dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)
                end 
            end)
        else
            widget:addClickEventListener(function() 
                if event_func then
                    event_func()
                    -- dxm.audio:playEffect(assets.sound.config.sound.ui_button_release)
                end
            end
            )
        end
        widget:setTouchEnabled(true)
    end
end

-- 取消触摸事件;
function CBaseUI:RemoveReleaseEvent(...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setTouchEnabled(false)
    end
end

-- 设置输入框及关联的触摸区域
function CBaseUI:SetTextFieldTouchWidget(text_field, ...)

    if text_field ~= nil then
        local function ReleaseCallback(sender)
            text_field:attachWithIME()
        end
        self:AddReleaseEvent(ReleaseCallback, ...)
    end
end

-- 播放动画
-- PlayAnimation(animation_name) ==> loop==false end_callback == nil
-- PlayAnimation(animation_name, end_callback) ==> loop==false
-- PlayAnimation(animation_name, loop) ==> end_callback == nil
function CBaseUI:PlayAnimation(animation_name, param)

    local loop = false
    local once_end_callback = nil
    if type(param) == "boolean" then
        loop = param
    elseif type(param) == "function" then
        once_end_callback = param
    end

    local rlt = false
    if self.action then
        rlt = self.action:play(animation_name, loop)
    end
    if once_end_callback then
        if rlt then
            self.action:setLastFrameCallFunc(function() once_end_callback(true) end)
        else
            once_end_callback(false)
        end
    end
    return rlt
end

function CBaseUI:StopAnimation()
    if self.action then
        self.action:stop()
    end
end

-- 类型 UI;
local ui_opened_ref_list = {}
function CBaseUI:CheckTypeRef()
    local ui_ref = ui_opened_ref_list[self.cname] or 0
    return ui_ref>0
end

function CBaseUI:_TypeAddRef()
    ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] or 0
    ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] + 1
end

function CBaseUI:_TypeDecRef()
    if ui_opened_ref_list[self.cname] then 
        ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] - 1
    end
end

-- 监听器;
function CBaseUI:AddListener(message_type, callback)
    return sMessageDispatcher:RegisterGroup(self, message_type, callback)
end

function CBaseUI:RemoveListener(handler)
    sMessageDispatcher:Remove(handler)
end

function CBaseUI:RemoveAllListener()
    sMessageDispatcher:RemoveGroup(self)
end

-- 打开关闭 UI;
function CBaseUI:OpenUI(ui_order)
    dxm.CUIManager:GetInstancePtr():OpenUI(self, ui_order)
end

function CBaseUI:CloseUI()
    dxm.CUIManager:GetInstancePtr():CloseUI(self)
end

return CBaseUI

-- endregion
