-- region NewFile_1.lua
-- Author : IBM
-- Date   : 2015/4/27
-- 此文件由[BabeLua]插件自动生成


-- 动画需要用到节点的时候就用它吧

dxm.CUIWidget = class("CUIWidget", ccui.Widget)

local CUIWidget = dxm.CUIWidget

CUIWidget.WidgetType ={
    -- 无效的类型
    invalid = 0,
    -- 从配置中获取的widget
    config = 1,
    -- 自定义类型
    custom = 2,
    -- 最大枚举
    max = 3,
}

-- 构造函数
function CUIWidget:ctor()
    -- 配置节点
    self.widget = nil
    self.widget_type = CUIWidget.WidgetType.invalid    
    self.widget_config = ""
    -- 事件
    self.event_enter = dxm.CEvent.new()
    self.event_exit = dxm.CEvent.new()
    -- 开场动画
    self.play_open_animation = true
    -- 父 UI 节点
    self.parent_ui = nil
    self.parent_widget = nil
end

function CUIWidget:Root()
    local root = self.widget
    return root
end

-- 初始化函数
-- Init(nil)
-- Init(file_name)
-- Init(widget)
function CUIWidget:Init(param)
    
    -- 重载字符串和节点函数;
    if param ~= nil then
        if tolua.type(param) == "string" then
            return self:InitWithFile(param)
        else
            return self:InitWithWidget(param)
        end
    end

    -- 已初始化;
    if self.widget ~= nil then
        return false
    end
    self:_ResetWidget(ccui.Widget:create())
    return self:_Init()
end

function CUIWidget:InitWithWidget(widget)
    -- 界面参数无效;
    if widget == nil then
        return false
    end
    -- 已初始化;
    if self.widget ~= nil then
        return false
    end
    self:_ResetWidget(widget)
    return self:_Init()
end

function CUIWidget:InitWithFile(config)
    -- 已初始化;
    if self.widget ~= nil then
        return false;
    end
    self.widget_config = config
    self.widget_type = CUIWidget.WidgetType.config
    local file = dxm.CLocale:GetUIFile(config)
    if not cc.FileUtils:getInstance():fullPathForFilename(file) then
        return false
    end
    return self:_Init()
end

function CUIWidget:_Init()
    self:registerScriptHandler( function(state)
        if state == "enter" then                      
            self:_Enter()
        elseif state == "exit" then
            self:_Exit()
        elseif state == "cleanup" then
            -- self:onCleanup()
        end
    end )
    self:setCascadeOpacityEnabled(true)
    self:setCascadeColorEnabled(true)
    return true
end

-- 创建函数
function CUIWidget:Create(...)
    local ui = self.new()
    if ui and ui:Init(...) then
        return ui
    end
    return nil
end

-- 重置widget
function CUIWidget:_ResetWidget(widget)

    -- 已经显示的界面无法重置 widget;
    if self.widget == widget then
        return
    end

    -- 必须清白的 widget;
    if widget:getParent() ~= nil then 
        return
    end
    
    -- 清理所有的子 UI;
    self:RemoveAllChildren()

    -- 清理界面;
    self:_ClearUI()

    -- 重置新界面;
    self.widget = widget
    self.widget:setCascadeOpacityEnabled(true)
    self.widget:setCascadeColorEnabled(true)
    if widget == nil then
        self.widget_type = CUIWidget.WidgetType.invalid
        return
    end

    -- 添加引用;
    self.widget:retain()
    self.widget_type = CUIWidget.WidgetType.custom

    -- 添加到父节点;
    self:addChild(self.widget)
end

-- 未显示的节点获取界面大小, 应用场景为滚动层上滚动条的大小
function CUIWidget:GetContentSize()    
    
    if self.content_size == nil then
        self.content_size = { width=0, height=0 }
        if self.widget==nil then
            if self.widget_type == CUIWidget.WidgetType.config then
                self:_InitConfigWidget()
                self:_ClearUI()
            end
        else
            self.content_size.width = self.widget:getContentSize().width
            self.content_size.height = self.widget:getContentSize().height
        end
    end
    return self.content_size
end

-- 清理界面;
function CUIWidget:_ClearUI()

    if self.action and self.widget_type == CUIWidget.WidgetType.config then
        self.widget:stopAction(self.action)
    end

    -- 回收物理widget;
    if self.widget and self.widget_type == CUIWidget.WidgetType.config then
        
        -- 回收缓存节点;
        if self.cache_node then 
            dxm.CWidgetCacheManager:GetInstancePtr():AddWidget(self.cache_node)
            self.cache_node = nil
        end

        -- 析构widget;
        if self.widget ~= nil then
            self.widget:release()
            self.widget = nil
        end

        if self.action ~= nil then
            self.action:release()
            self.action = nil
        end
    end
end

function CUIWidget:_InitConfigWidget()

    if self.widget_type ~= CUIWidget.WidgetType.config then 
        return
    end

    -- 创建界面一枚;
    local cache_node = dxm.CWidgetCacheManager:GetInstancePtr():GetWidget(self.widget_config)
    if cache_node == nil then
        return
    end

    self.cache_node = cache_node

    local action = cache_node.action
    local widget = cache_node.widget
    widget:runAction(action)

    -- 初始化界面;
    widget:setVisible(true)
    widget:setOpacity(255)
    widget:setColor(cc.c3b(255, 255, 255))
    widget:setAnchorPoint(cc.p(0, 0))
    widget:setPosition(cc.p(0, 0))
    widget:setScale(1.0)
    widget:setRotation(0)
    widget:setPosition(cc.p(0, 0))

    -- 设置界面;
    self.widget = widget
    self.widget:retain()
    self.action = action
    self.action:retain()

    -- 获取界面大小;
    self.content_size = self.content_size or { width=0, height=0 }
    self.content_size.width = self.widget:getContentSize().width
    self.content_size.height = self.widget:getContentSize().height

    -- 加入场景;
    self:addChild(widget)
end


function CUIWidget:_SetUIStrings()    

    local k = self.widget_config
    local find_begin = 1
    local find_end = string.find(k, "/")
    local last_name = nil
    while find_end do        
        last_name = string.sub(k, find_begin, find_end-1)
        find_begin = find_end+1
        find_end = string.find(k, "/", find_begin)
    end

    last_name = string.sub(k, find_begin)
    if not last_name then return end
 
    local ui_strings = assets.strings:GetUIStrings(last_name)
    if not ui_strings then return end

    for k, v in pairs(ui_strings) do
        local widget_tree = v.widget_tree
        local key = v.key
        
        local temp_widget = self.widget
        for _, v in ipairs(widget_tree) do
            temp_widget = ccui.Helper:seekWidgetByName(temp_widget, v)
            if temp_widget == nil then
                break
            end
        end

        if temp_widget then 
            local widget_type = tolua.type(temp_widget)
            if widget_type == "ccui.Text" then
                temp_widget:setString(assets.strings:GetString(key)) 
            elseif widget_type == "ccui.Button" then
                temp_widget:setTitleText(assets.strings:GetString(key))
            elseif widget_type == "ccui.TextField" then
                temp_widget:setPlaceHolder(assets.strings:GetString(key))
            end
        end      
    end
end

-- 打开与关闭事件
function CUIWidget:_Enter()

    self:_InitConfigWidget() 

    -- _TypeAddRef
    self:_TypeAddRef()
    self.is_entered = true

    -- 执行配置回调
    self:onEnter()
    self.event_enter:Fire()

    -- 设置界面文字
    self:_SetUIStrings()
     
    if self.play_open_animation then        
        self:PlayAnimation("OpenAnimation", false) 
    end   
end

function CUIWidget:_Exit()

    -- _TypeDecRef
    self:_TypeDecRef()

    -- 执行配置回调
    self:onExit()
    self.event_exit:Fire()

    -- 清理
    self:_ClearUI()

    -- set entered
    self.is_entered = false
end

function CUIWidget:onEnter()
    -- 虚函数
end

function CUIWidget:onExit()
    -- 虚函数
end

-- 子节点托管操作
-- AddChild(ui, zorder)
-- AddChild("widget_name", ui, zorder)
-- AddChild(widget, ui, zorder)
function CUIWidget:AddChild(...)

    local widget_name
    local base_ui
    local zorder
    local arg = {...}
    if type(arg[1]) == "table" then
        self:AddChildWithWidget(self.widget, arg[1], arg[2])
    elseif type(arg[1]) == "string" then 
        self:AddChildWithWidgetName(arg[1], arg[2], arg[3])
    else
        self:AddChildWithWidget(arg[1], arg[2], arg[3])
    end
end

function CUIWidget:AddChildWithWidgetName(widget_name, base_ui, zorder)

    zorder = zorder or 0

    if self.widget == nil then
        return
    end

    local widget = ccui.Helper:seekWidgetByName(self.widget, widget_name)
    if widget == nil then
        return
    end

    self:AddChildWithWidget(widget, base_ui, zorder)
end

function CUIWidget:AddChildWithWidget(widget, base_ui, zorder)

    -- 有父节点就不要进来了;
    if base_ui:getParent() then
        return
    end

    -- 界面中的层级;
    zorder = zorder or 0

    -- 创建链表;
    local widget_ui_children = self:CreateList(widget)

    -- 添加到节点;
    widget:addChild(base_ui, zorder)

    -- 添加到管理
    widget_ui_children[base_ui] = base_ui
end

-- 删除节点上所有 UI;
-- RemoveAllChildren(nil)
-- RemoveAllChildren("widget_name", ...)
-- RemoveAllChildren(widget)
function CUIWidget:RemoveAllChildren(...)

    local arg = {...}
    if arg[1] ~= nil then
        if type(arg[1]) == "string" then
            self:RemoveAllChildrenWithWidgetName(...)
        else
            self:RemoveAllChildrenWithWidget(arg[1])
        end
        return
    end

    if self.widget == nil then
        return
    end

    -- 清理全部的 ui;
    local wait_remove_ui = {}
    for temp_widget, temp_ui_list in pairs(self.ui_children) do
        -- 清空链表;
        for _, temp_ui in pairs(temp_ui_list) do
            wait_remove_ui[#wait_remove_ui+1] = temp_ui       
        end
    end

    for _, ui in pairs(wait_remove_ui) do
        if ui then ui:removeFromParent() end
    end

    self.ui_children = { }
end

function CUIWidget:RemoveAllChildrenWithWidgetName(...)

    if self.widget == nil then
        return
    end

    local widget = self:GetWidget(...)
    if widget == nil then
        return;
    end
    self:RemoveAllChildrenWithWidget(widget, base_ui)
end

function CUIWidget:RemoveAllChildrenWithWidget(widget)

    if widget == nil then
        return
    end

    -- 获取链表;
    local widget_ui_children = self:GetList(widget)
    if widget_ui_children == nil then
        return
    end

    -- 清理部分 UI;
    local wait_remove_ui = {}
    for _, temp_ui in pairs(widget_ui_children) do
        wait_remove_ui[#wait_remove_ui+1] = temp_ui     
    end
    for _, ui in pairs(wait_remove_ui) do
        if ui then ui:removeFromParent() end
    end
    widget_ui_children = {}
end

-- 删除 UI 
function CUIWidget:RemoveFromParent()

    if self.parent_ui ~= nil and self.parent_widget ~= nil then
        local widget_ui_children = self.parent_ui:GetList(self.parent_widget)
        if widget_ui_children then
            widget_ui_children[self] = nil
        end
        self.parent_ui = nil
        self.parent_widget = nil
    end

    if self:isRunning() then
        self:removeFromParent()
    end
end

function CUIWidget:RemoveChild(base_ui)
    base_ui:RemoveFromParent()
end

function CUIWidget:CreateList(widget)

    local list = self.ui_children[widget]
    if list == nil then
        list = { }
        self.ui_children[widget] = list
    end
    return list
end

function CUIWidget:GetList(widget)
    return self.ui_children[widget]
end

function CUIWidget:SetParent(base_ui, widget)
    self.parent_base_ui = base_ui
    self.parent_widget = widget
end

-- 获取 Widget;
-- GetWidget("widget_name", ...)
-- GetWidget(widget, "widget_name", ...)
function CUIWidget:GetWidget(...)
    local arg = {...}
    if type(arg[1]) == "string" or arg[1] == nil then
        return self:GetWidgetFromWidget(self.widget, ...)
    else
        return self:GetWidgetFromWidget(...)
    end
end

function CUIWidget:GetWidgetFromWidget(widget, ...)
    if widget == nil then
        return nil
    end

    local arg = {...}
    local temp_widget = widget
    for _, v in ipairs(arg) do
        -- temp_widget = temp_widget:getChildByName(v)
        temp_widget = ccui.Helper:seekWidgetByName(temp_widget, v)
        if temp_widget == nil then
            return nil
        end
    end
    return temp_widget
end

-- 设置显示性
-- SetWidgetVisible(is_visible, "widget_name", ...)
-- SetWidgetVisible(is_visible, widget, "widget_name", ...)
function CUIWidget:SetWidgetVisible(is_visible, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setVisible(is_visible)
    end
end

-- 设置有效性
-- SetWidgetEnable(is_visible, "widget_name", ...)
-- SetWidgetEnable(is_visible, widget, "widget_name", ...)
function CUIWidget:SetWidgetEnable(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setEnabled(is_enable)
    end
end

-- 设置可触摸性
-- SetWidgetTouchEnable(is_enable, "widget_name", ...)
-- SetWidgetTouchEnable(is_enable, widget, "widget_name", ...)
function CUIWidget:SetWidgetTouchEnable(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setTouchEnabled(is_enable)
    end
end

-- 设置有效性与显示性
-- SetWidgetEnableAndVisible(is_enable, "widget_name", ...)
-- SetWidgetEnableAndVisible(is_enable, widget, "widget_name", ...)
function CUIWidget:SetWidgetEnableAndVisible(is_enable, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setEnabled(is_enable)
        widget:setVisible(is_enable)
    end
end

-- 设置图片
function CUIWidget:SetImageViewPicture(path, ...)
    local image_view = self:GetWidget(...)
    if image_view ~= nil then
        image_view:loadTexture(path)
    end
end

-- 设置文字
function CUIWidget:SetLabelText(text, ...)
    local label = self:GetWidget(...)
    if label ~= nil then       
        label:setString(text)
    end
end

-- 设置进度条
function CUIWidget:SetLoadingbarPercent(percent, ...)
    local loading_bar = self:GetWidget(...)
    if loading_bar ~= nil then
        loading_bar:setPercent(percent)
    end
end

-- 设置触摸事件;
function CUIWidget:AddReleaseEvent(event_func, ...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:addClickEventListener(event_func)
        widget:setTouchEnabled(true)
    end
end

-- 取消触摸事件;
function CUIWidget:RemoveReleaseEvent(...)
    local widget = self:GetWidget(...)
    if widget ~= nil then
        widget:setTouchEnabled(false)
    end
end

-- 设置输入框及关联的触摸区域
function CUIWidget:SetTextFieldTouchWidget(text_field, ...)

    if text_field ~= nil then
        local function ReleaseCallback(sender)
            text_field:attachWithIME()
        end
        self:AddReleaseEvent(ReleaseCallback, ...)
    end
end

-- 播放动画
function CUIWidget:PlayAnimation(animation_name, loop)
    if self.action then
        self.action:play(animation_name, loop)
    end

end

-- 类型 UI;
local ui_opened_ref_list = {}
function CUIWidget:CheckTypeUnique()
    local ui_ref = ui_opened_ref_list[self.cname] or 0
    return ui_ref>0
end

function CUIWidget:_TypeAddRef()
    ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] or 0
    ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] + 1
end

function CUIWidget:_TypeDecRef()
    if ui_opened_ref_list[self.cname] then 
        ui_opened_ref_list[self.cname] = ui_opened_ref_list[self.cname] - 1
    end
end


return CUIWidget

-- endregion
