--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

require("dxm/base/singleton")
require("dxm/base/list")

dxm = dxm or {}
dxm.CWidgetCache = class("CWidgetCache")

local CWidgetCache = dxm.CWidgetCache

-- 成员函数
function CWidgetCache:ctor(locale_config, config)

    -- 配置名称
    local csb_file = "ccs/"..config..".csb"
    if dxm.CLocale.locale_name then
        local locale_csb_file = "ccs/ccs-" ..dxm.CLocale.locale_name.."/"..config..".csb"
        local fullpath = cc.FileUtils:getInstance():fullPathForFilename(locale_csb_file)
        if cc.FileUtils:getInstance():isFileExist(fullpath) then
             csb_file = locale_csb_file 
        end
    end
    self.config = csb_file
    self.name = locale_config -- 索引名字取本地化配置名
    self.cache_enable = false -- 是否激活高速缓存
    self.widget_unuse_list = dxm.CList.new() -- 缓存对象列表
    self.widget_used = {}
    self.widget_ref_count = 0 -- 缓存构建数量

    --[[
    self.template_widget = nil -- 模版界面
    self.template_widget = self:LoadWidget()
    if self.template_widget == nil then
        print("[CWidgetCache] LoadWidget "..self.config.." failed")
    else        
        self.template_widget:retain()
        print("[CWidgetCache] LoadWidget "..self.config)
    end
    ]]--
end

function CWidgetCache:Dispose()
    --[[
    if self.template_widget then
        self.template_widget:release()
    end   
    ]]--


    -- 释放空闲列表;
    local cache_node = self.widget_unuse_list:PopFront()
    while cache_node  do
        cache_node.widget:release()
        cache_node.action:release()
        cache_node = self.widget_unuse_list:PopFront()
    end

    -- 释放已用列表;
    for k, v in ipairs(self.widget_used) do
        v.widget:release()
        v.action:release()
    end
    self.widget_used = {}
end

function CWidgetCache:LoadWidget()
    local file = self.config
    local node = cc.CSLoader:createNode(file)
    if node ==nil then
        print("[CWidgetCache] widgetFromJsonFile ", self.config, " failed")
        return
    end

    print("[CWidgetCache] load widget: "..file)

    local action = cc.CSLoader:createTimeline(file)

    local name = node:getName()

    -- 重新建一个 widget的 node
    local new_node = ccui.Widget:create()
    new_node:setLayoutComponentEnabled(true)
    new_node:setAnchorPoint(cc.p(0, 0))
	new_node:setCascadeOpacityEnabled(true)
    new_node:setCascadeColorEnabled(true)
    local childs = node:getChildren()
    for _, v in pairs(childs) do 
        v:removeFromParent()
    end 

    for _, v in pairs(childs) do 
        new_node:addChild(v)
    end

    -- 自适应代码
    if name~="Layer" and name~="Node" then
        new_node:setContentSize(cc.size(display.width,display.height))
        ccui.Helper:doLayout(new_node)
    else
        new_node:setContentSize(node:getContentSize())
    end

    node = new_node

    -- 自增, 接管生命周期;
    node:retain()
    action:retain()
    return {name=self.name, widget=node, action=action}
end

function CWidgetCache:Clone()

    --[[
    local function _Clone()

        if self.template_widget then 
          local widget = self.template_widget:clone()
          return widget
        end
        return nil
    end
    ]]--
    
    -- clone 无法将属性 clone 全, 还是用 load 吧。
      -- widget 的 Component 没有 clone, 其中 ComExtensionData 是编辑器动画所需要的组件没有复制
       -- 之前碰到的字体属性, 文件属性没有拷全的。 问题多多哦。
    -- return _Clone()
    return self:LoadWidget()
end

function CWidgetCache:Add(widget)

    if widget == nil then
        return
    end
    
    self.widget_used[widget] = nil
    self.widget_unuse_list:PushBack(widget)
end

function CWidgetCache:Get()

    --[[
    if self.template_widget == nil then
        return nil
    end
    ]]--

    -- 从缓存列表中抽取一枚
    local cache_node = self.widget_unuse_list:PopFront()
    -- 缓存列表已被使用完
    if cache_node == nil then
		cache_node = self:Clone()
        self.widget_ref_count = self.widget_ref_count + 1
        -- print("[CWidgetCache] clone ui_widget "..self.config.."(ref:"..self.widget_ref_count..")")
    end

    -- 已使用的 widget 列表; 由于 autorelease 无法使用, 为保证 widget_unuse_list pop 的 widget 的安全性, 我只能让 cache 自己记住了 
    if cache_node then 
        self.widget_used[cache_node] = cache_node
    end
    return cache_node
end

function CWidgetCache:Cache(count)

    --[[
    if self.template_widget == nil then
        return
    end
    ]]--
    if count == 0 then count = 1 end

    local real_count = count - self.widget_unuse_list.size
    for i=1,real_count do
        
        local widget = self:Clone()
        if widget ~= nil then
            self.widget_unuse_list:PushBack(widget)
            self.widget_ref_count = self.widget_ref_count + 1
        end
    end

    -- cache 引用
    if self.cache_enable==false then 
        self.widget_ref_count = self.widget_ref_count + 1
        self.cache_enable = true
    end
end

function CWidgetCache:Uncache()
    if self.cache_enable==true then 
        self.widget_ref_count = self.widget_ref_count - 1
        self.cache_enable = false
    end
end

-- ---------------------
-- CWidgetCacheManager
dxm.CWidgetCacheManager = class("CWidgetCacheManager")

local CWidgetCacheManager = dxm.CWidgetCacheManager
dxm.SINGLETON(CWidgetCacheManager)

-- 构造函数
function CWidgetCacheManager:ctor()
    self.gc_size = 10 -- gc容量
    self.gc_interval = 1 -- gc间隔
    self.gc_tick = 0 -- gc时间
    self.gc_widget_cache_list = dxm.CList.new() -- GC缓存链表

    self.widget_cache_map = {} -- 临时缓存表单，无用时会被GC
    self.register_widget_cache_map = {} -- 注册缓存表单

    local function Update(dt)
        self:UpdateGC(dt)
    end
    cc.Director:getInstance():getScheduler():scheduleScriptFunc(Update, 0, false)
end

function CWidgetCacheManager:Dispose()
    -- TODO:
end

function CWidgetCacheManager:GetWidget(config)

    local widget_cache = self:GetWidgetCache(config)
    return widget_cache:Get()
end

function CWidgetCacheManager:CacheWidget(config, count)

    local widget_cache = self:GetWidgetCache(config)
    if widget_cache == nil then
        return
    end

    widget_cache:Cache(count or 1)

    self.register_widget_cache_map[widget_cache.name] = widget_cache
end

function CWidgetCacheManager:UncacheWidget(config)

    local widget_cache = self.register_widget_cache_map[config]
    if widget_cache ~= nil then
        widget_cache:Uncache()
        self.register_widget_cache_map[widget_cache.name] = nil
    end
end

function CWidgetCacheManager:UncacheAllWidget()

    for _, widget_cache in pairs(self.register_widget_cache_map) do 
        if widget_cache then widget_cache:Uncache() end
    end

    self.register_widget_cache_map = {}
end

function CWidgetCacheManager:AddWidget(cache_node)

    local widget_cache = self.widget_cache_map[cache_node.name]
    if widget_cache == nil then
        return
    end
    widget_cache:Add(cache_node)
end

function CWidgetCacheManager:GetWidgetCache(config)

    local locale_config = dxm.CLocale:GetLocaleName(config)

    local widget_cache = self.widget_cache_map[locale_config]
    if widget_cache ~= nil then
        return widget_cache
    end

    widget_cache = self:PopFromGC(locale_config)
    if widget_cache ~= nil then
        return widget_cache
    end

    widget_cache = CWidgetCache.new(locale_config, config)
    self.widget_cache_map[locale_config] = widget_cache
    return widget_cache
end

function CWidgetCacheManager:UpdateGC(dt)

    -- 累计时间，判断是否到达自动回收
    self.gc_tick = self.gc_tick + dt
    if self.gc_tick < self.gc_interval then
        return
    end
	self.gc_tick = 0

    
    -- 检测界面回收
    for config, temp_widget_cache in pairs(self.widget_cache_map) do

        -- 判断缓存界面是否全部被回收
        if temp_widget_cache ~= nil and temp_widget_cache.widget_unuse_list.size == temp_widget_cache.widget_ref_count then
            self:Push2GC(config, temp_widget_cache);
            self.widget_cache_map[config] = nil
            break
        end
    end

    -- 清理超过部分
    while self.gc_widget_cache_list.size > self.gc_size do

        local widget_cache = self.gc_widget_cache_list:PopFront()
        if widget_cache ~= nil then
            print("[CWidgetCache] gc widget: "..widget_cache.name)
            widget_cache:Dispose()
        end
    end
end

function CWidgetCacheManager:Push2GC(config, widget_cache)
    -- 添加到链表中
    self.gc_widget_cache_list:PushBack(widget_cache)
end

function CWidgetCacheManager:PopFromGC(config)

    -- 链表中搜索对应的缓存
    local node = self.gc_widget_cache_list.first
    while node ~= nil do

        -- 判断名称是否一致
        local widget_cache = node.value
        if widget_cache.config == config then

            self.widget_cache_map[config] = widget_cache
            self.gc_widget_cache_list:Erase(node)
            return widget_cache
        end
        node = node.next
    end
    return nil
end

return CWidgetCacheManager

--endregion
