-- region *.lua
-- Date
-- 此文件由[BabeLua]插件自动生成

cc.exports.update = cc.exports.update or {}

update.CUpdateUtils = { }
cc.exports.sUpdateUtils = update.CUpdateUtils

local CUpdateUtils = update.CUpdateUtils


CUpdateUtils.update_state_error = 0
CUpdateUtils.update_state_version_check = 1
CUpdateUtils.update_state_version_checking = 2
CUpdateUtils.update_state_files_check = 3
CUpdateUtils.update_state_files_checking = 4
CUpdateUtils.update_state_patch = 5
CUpdateUtils.update_state_patching = 6
CUpdateUtils.update_state_end = 7

function CUpdateUtils:Init()    

    require "dxm.init"

    local branch_name = sLocale:GetBranchName()  
    
    -- --------------
    -- search_paths
    local search_paths = {}
    -- ""
    if branch_name then search_paths[#search_paths+1] = "branches/"..branch_name.."/" end
    -- search_paths[#search_paths+1] = ""
    
    -- "src/
    if branch_name then search_paths[#search_paths+1] = "branches/"..branch_name.."/src/" end
    search_paths[#search_paths+1] = "src/" 
    cc.FileUtils:getInstance():setSearchPaths(search_paths)

    self.version_path = "version.manifest"
    self.files_path = "files.manifest"
    self.cache_save_path = device.writablePath .. "cache/"
    self.manifest_save_path = device.writablePath .. "manifest/"
    self.patch_save_path = device.writablePath .. "patch/"

    self.app_version = sJsonUtils:Load(self.version_path)
    self.app_version.Timestamp = self.app_version.Timestamp or 0
    cc.exports.test_version = false

    self.failed_times = 0
end

function CUpdateUtils:Start(callback)

    self.patch_list = { }
    self.downloaded_list = {}

    -- download patch files process
    self.total_size = 0
    self.downloaded_size = 0

    self.state = CUpdateUtils.update_state_version_check
    self.callback = callback

    self.download_version_manifest = nil
    self.download_files_manifest = nil
end

function CUpdateUtils:Stop()

    self.callback = nil
end

function CUpdateUtils:CheckVersion()
    
    if self.state ~= CUpdateUtils.update_state_version_check then
        self:_Done()
        return 
    end

    self:_Load()

    if not self.init_ok then 
        self:_Done()
        return 
    end

    self:_Next()
    self:_DownloadVersionManifest()
end

function CUpdateUtils:CheckFiles()
    
    if self.state ~= CUpdateUtils.update_state_files_check then
        self:_Done()
        return 
    end

    self:_Next()
    self:_DownloadFilesManifest()
end

function CUpdateUtils:DownloadPatch()
    
    if self.state ~= CUpdateUtils.update_state_patch then return end

    self:_Next()
    self:_DownloadPatchStart()
end

function CUpdateUtils:CheckInstall()

    if self:_CheckInstallFlag() then
        local downloaded_list = sJsonUtils:Load(self.cache_save_path .. "/downloaded_list.json")
        for k, patch_file in pairs(downloaded_list) do 
            self.downloaded_list[#self.downloaded_list+1] = patch_file
        end

        self.downloaded_list[#self.downloaded_list+1] = {from=self.cache_save_path .. self.files_path, to=self.patch_save_path .. self.files_path}
        self.downloaded_list[#self.downloaded_list+1] = {from=self.cache_save_path .. self.version_path, to=self.patch_save_path .. self.version_path}
        for k, patch_file in pairs(self.downloaded_list) do
            local file = sFileUtils:Read(patch_file.from)
            if file then
                file:Write(patch_file.to)
                print("patch downloaded ", patch_file.from)
                print("patch downloaded write to ", patch_file.to)
            end
        end
        sFileUtils:RemoveDirectory(self.cache_save_path)
        sFileUtils:RemoveDirectory(self.manifest_save_path)
        self:_SetInstallFlag(false)
        return true
    end

    return false
end

function CUpdateUtils:_Next()

    self.state = self.state + 1
    if self.callback then self.callback(self.state, 0, 0) end
end

function CUpdateUtils:_Done()

    self.state = CUpdateUtils.update_state_end
    self:CheckInstall()
    if self.callback then self.callback(self.state, 100, 100) end
end

function CUpdateUtils:_Load()

    self.local_files_list = sJsonUtils:Load(self.files_path)
    local local_version = sJsonUtils:Load(self.version_path)
    local_version.Timestamp = local_version.Timestamp or 0
    sFileUtils:RemoveDirectory(self.manifest_save_path)
    if self.app_version.Timestamp > local_version.Timestamp then
        print("remove directory", self.app_version.Timestamp, local_version.Timestamp)
        sFileUtils:RemoveDirectory(self.cache_save_path)
        sFileUtils:RemoveDirectory(self.patch_save_path)        
        local_version = self.app_version
    end

    if not local_version.RemotePackageUrl then
        print("download version manifest RemotePackageUrl is not exist")
        return
    end
    if not local_version.RemoteVersionManifest then
        print("download version manifest RemoteVersionManifest is not exist")
        return
    end
    if not local_version.RemoteFilesManifest then
        print("download version manifest RemoteFilesManifest is not exist")
        return
    end
    if not self.local_files_list.assets then
        print("download files manifest assets is not exist")
        return
    end

    if self:CheckInstall() then
        cc.FileUtils:getInstance():purgeCachedEntries() -- 清理缓存
        local_version = sJsonUtils:Load(self.version_path) -- 读取本地版本
    end

    self.remote_package_url = local_version.RemotePackageUrl
    self.remote_version_manifest = local_version.RemoteVersionManifest
    self.remote_files_manifest = local_version.RemoteFilesManifest
    self.local_version_code = local_version.Version
    self.timestamp = local_version.Timestamp or 0
    self.files_manifest_md5 = local_version.FilesManifestMd5
    self.init_ok = true

    self.patch_list = { }

    -- download patch files process
    self.total_size = 0
    self.downloaded_size = 0

    self.state = CUpdateUtils.update_state_version_check
end

function CUpdateUtils:_DownloadVersionManifest()

    sDownloadUtils:Download(self.remote_version_manifest, self.manifest_save_path, function(state, result) 

        if state == kDownloading then
            if self.callback then self.callback(self.state, result.bytes_done, result.bytes_total) end
        elseif state == kDownloadFailed then
            print("download version manifest failed")
            self:_Done()
        elseif state==kDownloadFinish then            
            local file = sFileUtils:Read(result.file_path)
            if not file:CheckValid() then
                -- file is not valid download again;    
                sFileUtils:RemoveFile(result.file_path)           
                self:_DownloadVersionManifest()
            else
                print("download version manifest successful")
                -- md5 is diff to download manifest
                if self.callback then self.callback(self.state, result.bytes_done, result.bytes_total) end
                
                self.download_version_manifest = result.file_path
                local version_data = sJsonUtils:Load(result.file_path)
                local remote_timestamp = version_data.Timestamp
                local remote_version_code = version_data.Version
                if self.local_version_code and remote_version_code and remote_version_code ~= self.local_version_code then
                    print("download version version code is not same") 
                    self:_Done()
                elseif self.timestamp > remote_timestamp then
                    -- version large than remote_version is test
                    test_version = true
                    print("download version manifest timestamp is test")             
                    self:_Done()
                elseif self.timestamp < remote_timestamp or version_data.FilesManifestMd5 ~= self.files_manifest_md5 then
                    print("download version manifest md5 not same")
                    self:_Next()
                else
                    print("download version manifest version and md5 same")
                    file:Write(self.patch_save_path .. self.version_path) 
                    self:_Done()
                end
            end
        end
    end) 
end

function CUpdateUtils:_DownloadFilesManifest()

    self.state = CUpdateUtils.update_state_files_checking
    sDownloadUtils:Download(self.remote_files_manifest, self.manifest_save_path, function(state, result) 
        if state == kDownloading then
            if self.callback then self.callback(self.state, result.bytes_done, result.bytes_total) end
        elseif state == kDownloadFailed then
            self:_DownloadFilesManifest()
            self.failed_times = self.failed_times + 1
            if self.failed_times>=3 then
                self:_Done()
            end
        elseif state==kDownloadFinish then
            local file = sFileUtils:Read(result.file_path)
            if not file:CheckValid() then
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadFilesManifest()
            else             
                if self.callback then self.callback(self.state, result.bytes_done, result.bytes_total) end
                self.download_files_manifest = result.file_path
                self.remote_files_list = sJsonUtils:Load(result.file_path)               
                self:_CheckPatch()
                self:_Next()           
            end
        end
    end) 
end

function CUpdateUtils:_CheckPatch()

    print("check patch to download")
    local local_files_map = {}
    local remote_files_map = {}
                    
    if self.local_files_list and  self.local_files_list.assets then
        for _, file_info in pairs(self.local_files_list.assets) do
            local_files_map[file_info.file] = file_info
        end
    end
    if self.remote_files_list and  self.remote_files_list.assets then
        for _, file_info in pairs(self.remote_files_list.assets) do
            remote_files_map[file_info.file] = file_info
        end
    end

    for key, remote_file_info in pairs(remote_files_map) do
        local local_file_info = local_files_map[key]
        if not local_file_info then
            print("check file not exist", remote_file_info.file)
            local_file_info = {id=remote_file_info.id, file=remote_file_info.file, md5="asdasda", size=0}
            self.local_files_list[#self.local_files_list+1] = local_file_info
            self.patch_list[#self.patch_list+1] = {local_file_info = local_file_info, remote_file_info = remote_file_info}
            self.total_size = self.total_size + remote_file_info.size
        elseif local_file_info.md5 ~= remote_file_info.md5 then
            print("check file diff md5", remote_file_info.file)
            self.patch_list[#self.patch_list+1] = {local_file_info = local_file_info, remote_file_info = remote_file_info}
            self.total_size = self.total_size + remote_file_info.size
        end
    end
end


--获取路径  
local function stripfilename(filename)  
    return string.match(filename, "(.+)/[^/]*%.%w+$") --*nix system  
    --return string.match(filename, “(.+)\\[^\\]*%.%w+$”) — windows  
end  
  
--获取文件名  
local function strippath(filename)  
    return string.match(filename, ".+/([^/]*%.%w+)$") -- *nix system  
    --return string.match(filename, “.+\\([^\\]*%.%w+)$”) — *nix system  
end  
  
--去除扩展名  
local function stripextension(filename)  
    local idx = filename:match(".+()%.%w+$")  
    if(idx) then  
        return filename:sub(1, idx-1)  
    else  
        return filename  
    end  
end  

function CUpdateUtils:_CheckInstallFlag()
    local install_flag = sJsonUtils:Load(self.cache_save_path .. "/install_flag.json")
    return install_flag.install
end

function CUpdateUtils:_SetInstallFlag(install)
    if install then 
        sJsonUtils:Save(self.cache_save_path .. "/downloaded_list.json", self.downloaded_list)
    else
        self.downloaded_list = {}
    end

    local install_flag = {}
    install_flag.install = install
    sJsonUtils:Save(self.cache_save_path .. "/install_flag.json", install_flag)
end

function CUpdateUtils:_DownloadPatch(pos)

    -- collectgarbage("collect")
    local total_patch = #self.patch_list
    -- print("download", pos)
    if pos > total_patch then
        return false
    end

    local patch_info = self.patch_list[pos]
    local download_url = self.remote_package_url .. patch_info.remote_file_info.file
    local rpath = stripfilename(patch_info.remote_file_info.file)
    local save_path = self.cache_save_path .. rpath
    
    -- print("download", download_url)
    
    sDownloadUtils:Download(download_url, save_path, function(state, result)

        if state == kDownloading then

            if self.callback then self.callback(self.state, self.downloaded_size + result.bytes_done, self.total_size) end
            -- if callback then callback(self.state, result.bytes_done, result.bytes_total) end
        elseif state == kDownloadFailed then

            sFileUtils:RemoveFile(result.file_path)
            self:_DownloadPatch(pos)
            self.failed_times = self.failed_times + 1
            if self.failed_times>=3 then
                self:_Done()
            end
        elseif state == kDownloadFinish then
            
            local file = sFileUtils:Read(result.file_path)
            if not file then
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadPatch(pos)
            elseif file:GetMD5() ~= patch_info.remote_file_info.md5 then
                -- file is not valid download again;
                sFileUtils:RemoveFile(result.file_path)
                self:_DownloadPatch(pos)
            else
                -- md5 is diff to download manifest;
                local patch_save_path = self.patch_save_path .. patch_info.remote_file_info.file                
                self.downloaded_list[#self.downloaded_list+1] = {from=result.file_path, to=patch_save_path}
                patch_info.local_file_info.md5 = patch_info.remote_file_info.md5
                patch_info.local_file_info.size = patch_info.remote_file_info.size
                
                self.downloaded_size = self.downloaded_size + patch_info.remote_file_info.size
                if self.callback then self.callback(self.state, self.downloaded_size, self.total_size) end
                self:_DownloadPatchFinish(pos)
            end
        end
    end )
    return true
end

-- function CUpdateUtils:_DownloadPatchStart()
-- 
--     self:_SetInstallFlag(false)
--     if not self:_DownloadPatch(1) then       
--         self:_DownloadFinish()
--     end
-- end

function CUpdateUtils:_DownloadPatchStart()

    self:_SetInstallFlag(false)
    self.patch_total = #self.patch_list
    if self.patch_total==0 then
        self:_DownloadFinish()
        return
    end

    for i=1, self.patch_total do
        self:_DownloadPatch(i)
    end
end

function CUpdateUtils:_DownloadPatchFinish()
    self.patch_downloaded = self.patch_downloaded or 0
    self.patch_downloaded = self.patch_downloaded + 1
    if self.patch_downloaded == self.patch_total then
        self:_DownloadFinish()
    end
end



function CUpdateUtils:_DownloadPatchNext(pos)

    if not self:_DownloadPatch(pos + 1) then
        self:_DownloadFinish()
    end
end

function CUpdateUtils:_DownloadFinish()

    local file = sFileUtils:Read(self.download_version_manifest)
    if file then file:Write(self.cache_save_path .. self.version_path) end
    file = sFileUtils:Read(self.download_files_manifest)
    if file then file:Write(self.cache_save_path .. self.files_path) end
    self:_SetInstallFlag(true)
    self:_Done()
end

return CUpdateUtils 

-- endregion
